@@ -1,4 +1,4 @@
-/*      $Id: acconfig.h,v 5.22 2002/07/27 09:17:08 lirc Exp $      */
+/*      $Id: acconfig.h,v 5.29 2004/09/11 19:56:51 lirc Exp $      */
 
 /*
  *  are you editing the correct file?
@@ -22,12 +22,12 @@
 #define DEV_LIRCD	"lircd"
 #define DEV_LIRCM	"lircm"
 
-/* config file names - beneath SYSCONFDIR (default /usr/local/etc) */
+/* config file names - beneath SYSCONFDIR (default /etc) */
 #define CFG_LIRCD	"lircd.conf"
 #define CFG_LIRCM	"lircmd.conf"
 
-/* config file names - beneath $HOME */
-#define CFG_USER	".lircrc"
+/* config file names - beneath $HOME or SYSCONFDIR */
+#define CFG_LIRCRC	"lircrc"
 
 /* log files */
 #define LOG_LIRCD	"lircd"
@@ -51,6 +51,9 @@
 /* Define to use long long IR codes */
 #undef LONG_IR_CODE
 
+/* Define to use dynamic IR codes */
+#undef DYNCODES
+
 /* Define to enable debugging output */
 #undef DEBUG
 
@@ -69,6 +72,15 @@
 /* Define if the software only test version of libirman is installed */
 #undef HAVE_LIBIRMAN_SW
 
+/* Define if the portaudio library is installed */
+#undef HAVE_LIBPORTAUDIO
+
+/* Define if the ALSA library is installed */
+#undef HAVE_LIBALSA
+
+/* Define if libusb is installed */
+#undef HAVE_LIBUSB
+
 /* Define if the complete vga libraries (vga, vgagl) are installed */
 #undef HAVE_LIBVGA
 
@@ -108,6 +120,9 @@
 /* Define if you have an IRdeo remote transmitter */
 #undef LIRC_SERIAL_IRDEO_REMOTE
 
+/* Define if you have an Igor Cesko receiver */
+#undef LIRC_SERIAL_IGOR
+
 /* Define if you want to cross-compile for the SA1100 */
 #undef LIRC_ON_SA1100
 
@@ -166,7 +181,8 @@
 #define LIRCDCFGFILE		SYSCONFDIR "/" CFG_LIRCD
 #define LIRCMDCFGFILE		SYSCONFDIR "/" CFG_LIRCM
 
-#define LIRCCFGFILE		CFG_USER
+#define LIRCRC_USER_FILE	"." CFG_LIRCRC
+#define LIRCRC_ROOT_FILE	SYSCONFDIR "/" CFG_LIRCRC
 
 #define LOGFILE			LOCALSTATEDIR "/log/" LOG_LIRCD
 #define LIRMAND_LOGFILE		LOCALSTATEDIR "/log/" LOG_LIRMAND
