@@ -1,4 +1,4 @@
-/*      $Id: hw_default.c,v 5.23 2002/01/06 13:36:43 lirc Exp $      */
+/*      $Id: hw_default.c,v 5.27 2003/11/12 11:52:14 lirc Exp $      */
 
 /****************************************************************************
  ** hw_default.c ************************************************************
@@ -23,6 +23,8 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 
 /* disable daemonise if maintainer mode SIM_REC / SIM_SEND defined */
 #if defined(SIM_REC) || defined (SIM_SEND)
@@ -60,8 +62,6 @@ static unsigned long supported_rec_modes[]=
 	0
 };
 
-lirc_t default_readdata (void);
-
 struct hardware hw_default=
 {
 	LIRC_DRIVER_DEVICE, /* default device */
@@ -96,11 +96,14 @@ static int default_config_frequency();
  *
  **********************************************************************/
 
-lirc_t default_readdata(void)
+lirc_t default_readdata(lirc_t timeout)
 {
 	lirc_t data;
 	int ret;
 
+	if (!waitfordata((long) timeout))
+		return 0;
+
 #if defined(SIM_REC) && !defined(DAEMONIZE)
 	while(1)
 	{
@@ -159,16 +162,50 @@ int default_init()
 	/* FIXME: other modules might need this, too */
 	init_rec_buffer();
 	init_send_buffer();
-	if((hw.fd=open(hw.device,O_RDWR))<0)
+
+	if(stat(hw.device,&s)==-1)
 	{
-		logprintf(LOG_ERR,"could not open %s",hw.device);
+		logprintf(LOG_ERR,"could not get file information for %s",
+			  hw.device);
 		logperror(LOG_ERR,"default_init()");
 		return(0);
 	}
-	if(fstat(hw.fd,&s)==-1)
+	
+        /* file could be unix socket, fifo and native lirc device */
+        if(S_ISSOCK(s.st_mode))
 	{
-		default_deinit();
-		logprintf(LOG_ERR,"could not get file information");
+		struct sockaddr_un addr;
+        	addr.sun_family=AF_UNIX;
+                strncpy(addr.sun_path,hw.device,sizeof(addr.sun_path));
+
+                hw.fd=socket(AF_UNIX,SOCK_STREAM,0);
+                if(hw.fd==-1)
+		{
+			logprintf(LOG_ERR,"could not create socket");
+			logperror(LOG_ERR,"default_init()");
+			return(0);
+        	}
+		
+        	if(connect(hw.fd,(struct sockaddr *) &addr,sizeof(addr))==-1)
+		{
+			logprintf(LOG_ERR,"could not connect to unix socket %s",
+				  hw.device);
+			logperror(LOG_ERR,"default_init()");
+			default_deinit();
+            		close(hw.fd);
+            		return(0);
+        	}
+		
+        	LOGPRINTF(1,"using unix socket lirc device");
+        	hw.features=LIRC_CAN_REC_MODE2 | LIRC_CAN_SEND_PULSE;
+        	hw.rec_mode=LIRC_MODE_MODE2; /* this might change in future */
+        	hw.send_mode=LIRC_MODE_PULSE;
+        	return(1);
+        }
+	
+        if((hw.fd=open(hw.device,O_RDWR))<0)
+	{
+		logprintf(LOG_ERR,"could not open %s",hw.device);
 		logperror(LOG_ERR,"default_init()");
 		return(0);
 	}
@@ -193,8 +230,21 @@ int default_init()
 		logprintf(LOG_ERR,"could not get hardware features");
 		logprintf(LOG_ERR,"this device driver does not "
 			  "support the new LIRC interface");
-		logprintf(LOG_ERR,"make sure you use a current "
-			  "version of the driver");
+		if(major(s.st_rdev) != LIRC_MAJOR)
+		{
+			logprintf(LOG_ERR, "major number of %s is %lu",
+				  hw.device,
+				  (unsigned long) major(s.st_rdev));
+			logprintf(LOG_ERR, "LIRC major number is %lu",
+				  (unsigned long) LIRC_MAJOR);
+			logprintf(LOG_ERR, "check if %s is a LIRC device",
+				  hw.device);
+		}
+		else
+		{
+			logprintf(LOG_ERR,"make sure you use a current "
+				  "version of the driver");
+		}
 		default_deinit();
 		return(0);
 	}
@@ -447,6 +497,16 @@ int default_send(struct ir_remote *remote,struct ir_ncode *code)
 		gettimeofday(&remote->last_send,NULL);
 		remote->last_code=code;
 #if defined(SIM_SEND) && !defined(DAEMONIZE)
+		if(is_raw(remote) && is_const(remote))
+		{
+			int i;
+			
+			for(i=0;remote->remaining_gap>code->signals[i] &&
+				    i<code->length;i++)
+			{
+				remote->remaining_gap-=code->signals[i];
+			}
+		}
 		printf("space %lu\n",(unsigned long) remote->remaining_gap);
 #endif
 	}
