@@ -1,4 +1,4 @@
-/*      $Id: irrecord.c,v 5.37 2002/01/06 13:36:43 lirc Exp $      */
+/*      $Id: irrecord.c,v 5.50 2004/02/05 19:44:50 lirc Exp $      */
 
 /****************************************************************************
  ** irrecord.c **************************************************************
@@ -61,8 +61,12 @@ void set_toggle_bit(struct ir_remote *remote,ir_code xor);
 void get_pre_data(struct ir_remote *remote);
 void get_post_data(struct ir_remote *remote);
 #ifdef DEBUG
+typedef void (*remote_func)(struct ir_remote *remotes);
+
+void for_each_remote(struct ir_remote *remotes, remote_func func);
 void remove_pre_data(struct ir_remote *remote);
 void remove_post_data(struct ir_remote *remote);
+void invert_data(struct ir_remote *remote);
 #endif
 int get_lengths(struct ir_remote *remote,int force);
 struct lengths *new_length(lirc_t length);
@@ -185,8 +189,9 @@ int main(int argc,char **argv)
 	int force;
 	int retries;
 	struct ir_remote *remotes=NULL;
+	char *device=NULL;
 #ifdef DEBUG
-	int get_pre=0,get_post=0;
+	int get_pre=0,get_post=0,test=0,invert=0;
 #endif
 
 	progname=argv[0];
@@ -205,11 +210,13 @@ int main(int argc,char **argv)
 #ifdef DEBUG
 			{"pre",no_argument,NULL,'p'},
 			{"post",no_argument,NULL,'P'},
+			{"test",no_argument,NULL,'t'},
+			{"invert",no_argument,NULL,'i'},
 #endif
 			{0, 0, 0, 0}
 		};
 #ifdef DEBUG
-		c = getopt_long(argc,argv,"hvd:H:fpP",long_options,NULL);
+		c = getopt_long(argc,argv,"hvd:H:fpPti",long_options,NULL);
 #else
 		c = getopt_long(argc,argv,"hvd:H:f",long_options,NULL);
 #endif
@@ -236,7 +243,7 @@ int main(int argc,char **argv)
 				exit (EXIT_FAILURE);
 			}
 		case 'd':
-			hw.device=optarg;
+			device=optarg;
 			break;
 		case 'f':
 			force=1;
@@ -248,6 +255,12 @@ int main(int argc,char **argv)
 		case 'P':
 			get_post=1;
 			break;
+		case 't':
+			test=1;
+			break;
+		case 'i':
+			invert=1;
+			break;
 #endif
 		default:
 			printf("Try %s -h for help!\n",progname);
@@ -263,11 +276,17 @@ int main(int argc,char **argv)
 		fprintf(stderr,"%s: invalid argument count\n",progname);
 		exit(EXIT_FAILURE);
 	}
-#ifdef LIRC_NETWORK_ONLY 
-	fprintf(stderr,"%s: irrecord does not make sense without hardware\n",
-		progname);
-	exit(EXIT_FAILURE);
-#endif
+	if(device!=NULL)
+	{
+		hw.device=device;
+	}
+	if(strcmp(hw.name, "null")==0)
+	{
+		fprintf(stderr,
+			"%s: irrecord does not make sense without hardware\n",
+			progname);
+		exit(EXIT_FAILURE);
+	}
 	filename=argv[optind];
 	fin=fopen(filename,"r");
 	if(fin!=NULL)
@@ -284,14 +303,18 @@ int main(int argc,char **argv)
 			exit(EXIT_FAILURE);
 		}
 #ifdef DEBUG
-		remove_pre_data(remotes);
-		remove_post_data(remotes);
-		if(get_pre) get_pre_data(remotes);
-		if(get_post) get_post_data(remotes);
+		if(test)
+		{
+			for_each_remote(remotes, remove_pre_data);
+			for_each_remote(remotes, remove_post_data);
+			if(get_pre) for_each_remote(remotes, get_pre_data);
+			if(get_post) for_each_remote(remotes, get_post_data);
+			if(invert) for_each_remote(remotes, invert_data);
 			
-		fprint_remotes(stdout,remotes);
-		free_config(remotes);
-		return(EXIT_SUCCESS);
+			fprint_remotes(stdout,remotes);
+			free_config(remotes);
+			return(EXIT_SUCCESS);
+		}
 #endif
 		remote=*remotes;
 		remote.name=NULL;
@@ -299,11 +322,15 @@ int main(int argc,char **argv)
 		remote.last_code=NULL;
 		remote.next=NULL;
 		remote.toggle_bit=0;
-		remote.bits=remote.pre_data_bits+
-			remote.bits+
-			remote.post_data_bits;
-		remote.pre_data_bits=0;
-		remote.post_data_bits=0;
+		if(remote.pre_p==0 && remote.pre_s==0 &&
+		   remote.post_p==0 && remote.post_s==0)
+		{
+			remote.bits=remote.pre_data_bits+
+				remote.bits+
+				remote.post_data_bits;
+			remote.pre_data_bits=0;
+			remote.post_data_bits=0;
+		}
 		if(remotes->next!=NULL)
 		{
 			fprintf(stderr,
@@ -392,18 +419,24 @@ int main(int argc,char **argv)
 "DOCUMENTATION of this package, especially section \"Adding new remote\n"
 "controls\" for how to get help.\n"
 "\n"
-"IMPORTANT: The license of the config files created by this program requires\n"
-"that you send them to the author. If you don't like this license exit this\n"
-"program now by pressing Ctrl-C! Otherwise press RETURN.\n\n");
+"If there already is a remote control of the same brand available at\n"
+"http://www.lirc.org/remotes/ you might also want to try using such a\n"
+"remote as a template. The config files already contain all\n"
+"parameters of the protocol used by remotes of a certain brand and\n"
+"knowing these parameters makes the job of this program much\n"
+"easier. There are also template files for the most common protocols\n"
+"available in the remotes/generic/ directory of the source\n"
+"distribution of this package. You can use a template files by\n"
+"providing the path of the file as command line parameter.\n"
+"\n"
+"Please send the finished config files to <lirc@bartelmus.de> so that I\n"
+"can make them available to others. Don't forget to put all information\n"
+"that you can get about the remote control in the header of the file.\n"
+"\n"
+"Press RETURN to continue.\n\n");
+	
+	getchar();
 	
-	if(getchar()!='\n')
-	{
-		fclose(fout);
-		unlink(filename);
-		if(hw.deinit_func) hw.deinit_func();
-		exit(EXIT_FAILURE);
-	}
-
 	remote.name=filename;
 	switch(hw.rec_mode)
 	{
@@ -413,7 +446,7 @@ int main(int argc,char **argv)
 			if(remote.gap==0)
 			{
 				fprintf(stderr,"%s: gap not found,"
-					" cant continue\n",progname);
+					" can't continue\n",progname);
 				fclose(fout);
 				unlink(filename);
 				if(hw.deinit_func) hw.deinit_func();
@@ -446,7 +479,7 @@ int main(int argc,char **argv)
 		if(!get_gap_length(&remote))
 		{
 			fprintf(stderr,"%s: gap not found,"
-				" cant continue\n",progname);
+				" can't continue\n",progname);
 			fclose(fout);
 			unlink(filename);
 			if(hw.deinit_func) hw.deinit_func();
@@ -528,7 +561,6 @@ int main(int argc,char **argv)
 		{
 			lirc_t data,sum;
 			unsigned int count;
-			int ret;
 
 			count=0;sum=0;
 			while(count<MAX_SIGNALS)
@@ -537,7 +569,8 @@ int main(int argc,char **argv)
 				
 				if(count==0) timeout=10000000;
 				else timeout=remote.gap*5;
-				if(!waitfordata(timeout))
+				data=hw.readdata(timeout);
+				if(!data)
 				{
 					if(count==0)
 					{
@@ -548,18 +581,6 @@ int main(int argc,char **argv)
 					}
 					data=remote.gap;
 				}
-				else
-				{
-					ret=read(hw.fd,&data,sizeof(data));
-					if(ret!=sizeof(unsigned long))
-					{
-						fprintf(stderr,"%s: read() failed\n",
-							progname);
-						perror(progname);
-						retval=EXIT_FAILURE;
-						break;
-					}
-				}
 				if(count==0)
 				{
 					if(!is_space(data) ||
@@ -774,7 +795,7 @@ void flushhw(void)
 		if(hw.code_length%CHAR_BIT) size++;
 		break;
 	}
-	while(read(hw.fd,buffer,size)==size);
+    while(read(hw.fd,buffer,size)==size);
 }
 
 int resethw(void)
@@ -900,12 +921,14 @@ int get_toggle_bit(struct ir_remote *remote)
 			break;
 		}
 		hw.rec_func(remote);
-		if(is_rc6(remote))
+		if(is_rc6(remote) && remote->rc6_mask==0)
 		{
-			for(remote->toggle_bit=1;
-			    remote->toggle_bit<=remote->bits;
-			    remote->toggle_bit++)
+			int i;
+			ir_code mask;
+
+			for(i=0,mask=1;i<remote->bits;i++,mask<<=1)
 			{
+				remote->rc6_mask=mask;
 				success=hw.decode_func(remote,&pre,&code,&post,
 						       &repeat_flag,
 						       &remaining_gap);
@@ -915,7 +938,7 @@ int get_toggle_bit(struct ir_remote *remote)
 					break;
 				}
 			}
-			if(success==0) remote->toggle_bit=0;
+			if(success==0) remote->rc6_mask=0;
 		}
 		else
 		{
@@ -952,6 +975,10 @@ int get_toggle_bit(struct ir_remote *remote)
 				repeats++;
 			}
 		}
+		else
+		{
+			retries--;
+		}
 	}
 	if(!found)
 	{
@@ -963,6 +990,10 @@ int get_toggle_bit(struct ir_remote *remote)
 		{
 			printf("\nToggle bit is %d.\n",remote->toggle_bit);
 		}
+		else if(remote->toggle_mask!=0)
+		{
+			printf("\nToggle mask found.\n");
+		}
 		else
 		{
 			printf("\nInvalid toggle bit.\n");
@@ -980,11 +1011,12 @@ void set_toggle_bit(struct ir_remote *remote,ir_code xor)
 	ir_code mask;
 	int toggle_bit;
 	struct ir_ncode *codes;
+	int bits;
 
 	if(!remote->codes) return;
 
-
-	mask=((ir_code) 1)<<(remote->bits+remote->pre_data_bits+remote->post_data_bits-1);
+	bits=remote->bits+remote->pre_data_bits+remote->post_data_bits;
+	mask=((ir_code) 1)<<(bits-1);
 	toggle_bit=1;
 	while(mask)
 	{
@@ -1003,6 +1035,14 @@ void set_toggle_bit(struct ir_remote *remote,ir_code xor)
 			codes++;
 		}
 	}
+	else
+	{
+		/* Sharp, Denon and some others use a toggle_mask */
+		if(bits==15 && xor==0x3ff)
+		{
+			remote->toggle_mask=xor;
+		}
+	}
 }
 
 void get_pre_data(struct ir_remote *remote)
@@ -1117,59 +1157,102 @@ void get_post_data(struct ir_remote *remote)
 }
 
 #ifdef DEBUG
-void remove_pre_data(struct ir_remote *remotes)
+void for_each_remote(struct ir_remote *remotes, remote_func func)
 {
-	struct ir_ncode *codes;
 	struct ir_remote *remote;
 	
 	remote=remotes;
 	while(remote!=NULL)
 	{
-		if(remote->pre_data_bits==0)
-		{
-			remote=remote->next;
-			continue;
-		}
-
-		codes=remote->codes;
-		while(codes->name!=NULL)
-		{
-			codes->code|=remote->pre_data<<remote->bits;
-			codes++;
-		}
-		remote->bits+=remote->pre_data_bits;
-		remote->pre_data=0;
-		remote->pre_data_bits=0;
+		func(remote);
 		remote=remote->next;
 	}
 }
 
-void remove_post_data(struct ir_remote *remotes)
+void remove_pre_data(struct ir_remote *remote)
 {
 	struct ir_ncode *codes;
 	
-	struct ir_remote *remote;
-	
-	remote=remotes;
-	while(remote!=NULL)
+	if(remote->pre_data_bits==0 ||
+	   remote->pre_p!=0 || remote->pre_s!=0)
 	{
-		if(remote->post_data_bits==0)
-		{
-			remote=remote->next;
-			continue;
-		}
-		
-		codes=remote->codes;
-		while(codes->name!=NULL)
-		{
-			codes->code<<=remote->post_data_bits;
-			codes->code|=remote->post_data;
-			codes++;
-		}
-		remote->bits+=remote->post_data_bits;
-		remote->post_data=0;
-		remote->post_data_bits=0;
 		remote=remote->next;
+		return;
+	}
+	
+	codes=remote->codes;
+	while(codes->name!=NULL)
+	{
+		codes->code|=remote->pre_data<<remote->bits;
+		codes++;
+	}
+	remote->bits+=remote->pre_data_bits;
+	remote->pre_data=0;
+	remote->pre_data_bits=0;
+}
+
+void remove_post_data(struct ir_remote *remote)
+{
+	struct ir_ncode *codes;
+	
+	if(remote->post_data_bits==0)
+	{
+		remote=remote->next;
+		return;
+	}
+	
+	codes=remote->codes;
+	while(codes->name!=NULL)
+	{
+		codes->code<<=remote->post_data_bits;
+		codes->code|=remote->post_data;
+		codes++;
+	}
+	remote->bits+=remote->post_data_bits;
+	remote->post_data=0;
+	remote->post_data_bits=0;
+}
+
+void invert_data(struct ir_remote *remote)
+{
+	struct ir_ncode *codes;
+	ir_code mask;
+	lirc_t p,s;
+		
+	/* swap one, zero */
+	p=remote->pone;
+	s=remote->sone;
+	remote->pone=remote->pzero;
+	remote->sone=remote->szero;
+	remote->pzero=p;
+	remote->szero=s;
+	
+	/* invert pre_data */
+	if(has_pre(remote))
+	{
+		mask=gen_mask(remote->pre_data_bits);
+		remote->pre_data^=mask;
+	}
+	/* invert post_data */
+	if(has_post(remote))
+	{
+		mask=gen_mask(remote->post_data_bits);
+		remote->post_data^=mask;
+	}
+	
+	if(remote->bits==0)
+	{
+		remote=remote->next;
+		return;
+	}
+	
+	/* invert codes */
+	mask=gen_mask(remote->bits);
+	codes=remote->codes;
+	while(codes->name!=NULL)
+	{
+		codes->code^=mask;
+		codes++;
 	}
 }
 #endif
@@ -1202,7 +1285,7 @@ inline lirc_t calc_signal(struct lengths *len)
 
 int get_lengths(struct ir_remote *remote,int force)
 {
-	int ret,retval;
+	int retval;
 	lirc_t data,average,sum,remaining_gap,header;
 	enum analyse_mode mode=MODE_GAP;
 	int first_signal;
@@ -1224,22 +1307,14 @@ int get_lengths(struct ir_remote *remote,int force)
 	first_length=0;
 	while(1)
 	{
-		if(!waitfordata(10000000))
+		data=hw.readdata(10000000);
+		if(!data)
 		{
 			fprintf(stderr,"%s: no data for 10 secs, aborting\n",
 				progname);
 			retval=0;
 			break;
 		}
-		ret=read(hw.fd,&data,sizeof(data));
-		if(ret!=sizeof(data))
-		{
-			fprintf(stderr,"%s: read() failed\n",
-				progname);
-			perror(progname);
-			retval=0;
-			break;
-		}
 		count++;
 		if(mode==MODE_GAP)
 		{
@@ -1651,6 +1726,7 @@ void get_scheme(struct ir_remote *remote)
 		   number of bits */
 		remote->bits=length;
 		remote->flags|=SPACE_ENC;
+		return;
 	}
 	else
 	{
@@ -1695,9 +1771,15 @@ void get_scheme(struct ir_remote *remote)
 					printf("RC-5 remote control found.\n");
 					remote->flags|=RC5;
 				}
+				return;
 			}
 		}
 	}
+	length++;
+	printf("Suspicious data length: %u.\n",length);
+	/* this is not yet the number of bits */
+	remote->bits=length;
+	remote->flags|=SPACE_ENC;
 }
 
 struct lengths *get_max_length(struct lengths *first,unsigned int *sump)
@@ -2046,6 +2128,11 @@ int get_data_length(struct ir_remote *remote)
 					remote->sone=s1;
 					remote->pzero=min(p1,p2);
 					remote->szero=s1;
+					if(expect(remote,remote->ptrail,p1) ||
+					   expect(remote,remote->ptrail,p2))
+					{
+						remote->ptrail=0;
+					}
 				}
 				else
 				{
@@ -2186,20 +2273,8 @@ int get_gap_length(struct ir_remote *remote)
 
 void fprint_copyright(FILE *fout)
 {
-	/* As this program is distributed under GPL you could just
-	   remove this copyright notice and the config files generated
-	   with the modified program would automatically be covered by
-	   the GPL. Although I am aware of this I will not prevent it.
-
-	   I hope that nobody will do so because the license I put on
-	   the config files is not really a restriction. Instead it
-	   emphasizes the spirit of the GPL to make things available
-	   to everybody. */
-
 	fprintf(fout,
 		"\n"
-		"# Copyright (C) 1999 Christoph Bartelmus\n"
-		"#\n"
-		"# You may only use this file if you make it available to others,\n"
-		"# i.e. if you send it to <lirc@bartelmus.de>\n");
+		"# Please make this file available to others\n"
+		"# by sending it to <lirc@bartelmus.de>\n");
 }
