@@ -1,4 +1,4 @@
-/*      $Id: receive.c,v 5.15 2002/09/21 15:25:28 lirc Exp $      */
+/*      $Id: receive.c,v 5.21 2004/08/28 10:07:27 lirc Exp $      */
 
 /****************************************************************************
  ** receive.c ***************************************************************
@@ -30,6 +30,16 @@ inline lirc_t lirc_t_max(lirc_t a,lirc_t b)
 	return(a>b ? a:b);
 }
 
+inline void set_pending_pulse(lirc_t deltap)
+{
+	rec_buffer.pendingp=deltap;
+}
+
+inline void set_pending_space(lirc_t deltas)
+{
+	rec_buffer.pendings=deltas;
+}
+
 lirc_t get_next_rec_buffer(lirc_t maxusec)
 {
 	if(rec_buffer.rptr<rec_buffer.wptr)
@@ -80,8 +90,8 @@ void rewind_rec_buffer(void)
 {
 	rec_buffer.rptr=0;
 	rec_buffer.too_long=0;
-	rec_buffer.pendingp=0;
-	rec_buffer.pendings=0;
+	set_pending_pulse(0);
+	set_pending_space(0);
 	rec_buffer.sum=0;
 }
 
@@ -194,19 +204,40 @@ inline lirc_t get_next_space(lirc_t maxusec)
 	return(data);
 }
 
-int expectpulse(struct ir_remote *remote,int exdelta)
+inline int sync_pending_pulse(struct ir_remote *remote)
+{
+	if(rec_buffer.pendingp>0)
+	{
+		lirc_t deltap;
+		
+		deltap=get_next_pulse(rec_buffer.pendingp);
+		if(deltap==0) return 0;
+		if(!expect(remote,deltap,rec_buffer.pendingp)) return 0;
+		set_pending_pulse(0);
+	}
+	return 1;
+}
+
+inline int sync_pending_space(struct ir_remote *remote)
 {
-	lirc_t deltas,deltap;
-	int retval;
-	
 	if(rec_buffer.pendings>0)
 	{
+		lirc_t deltas;
+		
 		deltas=get_next_space(rec_buffer.pendings);
-		if(deltas==0) return(0);
-		retval=expect(remote,deltas,rec_buffer.pendings);
-		if(!retval) return(0);
-		rec_buffer.pendings=0;
+		if(deltas==0) return 0;
+		if(!expect(remote,deltas,rec_buffer.pendings)) return 0;
+		set_pending_space(0);
 	}
+	return 1;
+}
+
+int expectpulse(struct ir_remote *remote,int exdelta)
+{
+	lirc_t deltap;
+	int retval;
+	
+	if(!sync_pending_space(remote)) return 0;
 	
 	deltap=get_next_pulse(rec_buffer.pendingp+exdelta);
 	if(deltap==0) return(0);
@@ -215,7 +246,7 @@ int expectpulse(struct ir_remote *remote,int exdelta)
 		retval=expect(remote,deltap,
 			      rec_buffer.pendingp+exdelta);
 		if(!retval) return(0);
-		rec_buffer.pendingp=0;
+		set_pending_pulse(0);
 	}
 	else
 	{
@@ -226,17 +257,10 @@ int expectpulse(struct ir_remote *remote,int exdelta)
 
 int expectspace(struct ir_remote *remote,int exdelta)
 {
-	lirc_t deltas,deltap;
+	lirc_t deltas;
 	int retval;
 
-	if(rec_buffer.pendingp>0)
-	{
-		deltap=get_next_pulse(rec_buffer.pendingp);
-		if(deltap==0) return(0);
-		retval=expect(remote,deltap,rec_buffer.pendingp);
-		if(!retval) return(0);
-		rec_buffer.pendingp=0;
-	}
+	if(!sync_pending_pulse(remote)) return 0;
 	
 	deltas=get_next_space(rec_buffer.pendings+exdelta);
 	if(deltas==0) return(0);
@@ -245,7 +269,7 @@ int expectspace(struct ir_remote *remote,int exdelta)
 		retval=expect(remote,deltas,
 			      rec_buffer.pendings+exdelta);
 		if(!retval) return(0);
-		rec_buffer.pendings=0;
+		set_pending_space(0);
 	}
 	else
 	{
@@ -258,9 +282,13 @@ inline int expectone(struct ir_remote *remote,int bit)
 {
 	if(is_biphase(remote))
 	{
-		if(is_rc6(remote) &&
-		   remote->toggle_bit>0 &&
-		   bit==remote->toggle_bit-1)
+		int all_bits = remote->pre_data_bits+
+			remote->bits+
+			remote->post_data_bits;
+		ir_code mask;
+		
+		mask=((ir_code) 1)<<(all_bits-1-bit);
+		if(mask&remote->rc6_mask)
 		{
 			if(remote->sone>0 &&
 			   !expectspace(remote,2*remote->sone))
@@ -268,7 +296,7 @@ inline int expectone(struct ir_remote *remote,int bit)
 				unget_rec_buffer(1);
 				return(0);
 			}
-			rec_buffer.pendingp=2*remote->pone;
+			set_pending_pulse(2*remote->pone);
 		}
 		else
 		{
@@ -277,7 +305,7 @@ inline int expectone(struct ir_remote *remote,int bit)
 				unget_rec_buffer(1);
 				return(0);
 			}
-			rec_buffer.pendingp=remote->pone;
+			set_pending_pulse(remote->pone);
 		}
 	}
 	else
@@ -298,7 +326,7 @@ inline int expectone(struct ir_remote *remote,int bit)
 		}
 		else
 		{
-			rec_buffer.pendings=remote->sone;
+			set_pending_space(remote->sone);
 		}
 	}
 	return(1);
@@ -308,17 +336,20 @@ inline int expectzero(struct ir_remote *remote,int bit)
 {
 	if(is_biphase(remote))
 	{
-		if(is_rc6(remote) &&
-		   remote->toggle_bit>0 &&
-		   bit==remote->toggle_bit-1)
+		int all_bits = remote->pre_data_bits+
+			remote->bits+
+			remote->post_data_bits;
+		ir_code mask;
+		
+		mask=((ir_code) 1)<<(all_bits-1-bit);
+		if(mask&remote->rc6_mask)
 		{
 			if(!expectpulse(remote,2*remote->pzero))
 			{
 				unget_rec_buffer(1);
 				return(0);
 			}
-			rec_buffer.pendings=2*remote->szero;
-			
+			set_pending_space(2*remote->szero);
 		}
 		else
 		{
@@ -327,7 +358,7 @@ inline int expectzero(struct ir_remote *remote,int bit)
 				unget_rec_buffer(1);
 				return(0);
 			}
-			rec_buffer.pendings=remote->szero;
+			set_pending_space(remote->szero);
 		}
 	}
 	else
@@ -347,7 +378,7 @@ inline int expectzero(struct ir_remote *remote,int bit)
 		}
 		else
 		{
-			rec_buffer.pendings=remote->szero;
+			set_pending_space(remote->szero);
 		}
 	}
 	return(1);
@@ -379,6 +410,17 @@ inline lirc_t sync_rec_buffer(struct ir_remote *remote)
 				return(0);
 			}
 		}
+		if(has_toggle_mask(remote))
+		{
+			if(deltas>last_remote->remaining_gap*
+			   (100+last_remote->eps)/100 &&
+			   deltas>last_remote->remaining_gap+last_remote->aeps)
+			{
+				remote->toggle_mask_state=0;
+				remote->toggle_code=NULL;
+			}
+			
+		}
 	}
 	rec_buffer.sum=0;
 	return(deltas);
@@ -410,6 +452,12 @@ inline int get_header(struct ir_remote *remote)
 		unget_rec_buffer(2);
 		return(0);
 	}
+	if(remote->shead==0)
+	{
+		if(!sync_pending_space(remote)) return 0;
+		set_pending_pulse(remote->phead);
+		return 1;
+	}
 	if(!expectpulse(remote,remote->phead))
 	{
 		unget_rec_buffer(1);
@@ -433,7 +481,7 @@ inline int get_header(struct ir_remote *remote)
 		}
 	}
 	
-	rec_buffer.pendings=remote->shead;
+	set_pending_space(remote->shead);
 	return(1);
 }
 
@@ -446,9 +494,10 @@ inline int get_foot(struct ir_remote *remote)
 
 inline int get_lead(struct ir_remote *remote)
 {
-	if(remote->plead==0) return(1);
-	rec_buffer.pendingp=remote->plead;
-	return(1);	
+	if(remote->plead==0) return 1;
+	if(!sync_pending_space(remote)) return 0;
+	set_pending_pulse(remote->plead);
+	return 1;
 }
 
 inline int get_trail(struct ir_remote *remote)
@@ -497,7 +546,7 @@ inline int get_repeat(struct ir_remote *remote)
 	else
 	{
 		if(!expectpulse(remote,remote->prepeat)) return(0);
-		rec_buffer.pendings=remote->srepeat;
+		set_pending_space(remote->srepeat);
 	}
 	if(!get_trail(remote)) return(0);
 	if(!get_gap(remote,
@@ -535,6 +584,7 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 			{
 				logprintf(LOG_ERR,"failed on bit %d",
 					  done+i+1);
+				return((ir_code) -1);
 			}
 			sum=deltap+deltas;
 			LOGPRINTF(3,"rcmm: sum %ld",(unsigned long) sum);
@@ -567,6 +617,220 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 		}
 		return(code);
 	}
+	else if(is_grundig(remote))
+	{
+		lirc_t deltap,deltas,sum;
+		int state,laststate;
+		
+		if(bits%2 || done%2)
+		{
+			logprintf(LOG_ERR,"invalid bit number.");
+			return((ir_code) -1);
+		}
+		if(!sync_pending_pulse(remote)) return ((ir_code) -1);
+		for(laststate=state=-1,i=0;i<bits;)
+		{
+			deltas=get_next_space(remote->szero+remote->sone+
+					      remote->stwo+remote->sthree);
+			deltap=get_next_pulse(remote->pzero+remote->pone+
+					      remote->ptwo+remote->pthree);
+			if(deltas==0 || deltap==0) 
+			{
+				logprintf(LOG_ERR,"failed on bit %d",
+					  done+i+1);
+				return((ir_code) -1);
+			}
+			sum=deltas+deltap;
+			LOGPRINTF(3,"grundig: sum %ld",(unsigned long) sum);
+			if(expect(remote,sum,remote->szero+remote->pzero))
+			{
+				state=0;
+				LOGPRINTF(2,"2T");
+			}
+			else if(expect(remote,sum,remote->sone+remote->pone))
+			{
+				state=1;
+				LOGPRINTF(2,"3T");
+			}
+			else if(expect(remote,sum,remote->stwo+remote->ptwo))
+			{
+				state=2;
+				LOGPRINTF(2,"4T");
+			}
+			else if(expect(remote,sum,remote->sthree+remote->pthree))
+			{
+				state=3;
+				LOGPRINTF(2,"6T");
+			}
+			else
+			{
+				LOGPRINTF(2,"no match for %ld+%ld=%ld",
+					 deltas,deltap,sum);
+				return((ir_code) -1);
+			}
+			if(state==3) /* 6T */
+			{
+				i+=2;code<<=2;state=-1;
+				code|=0;
+			}
+			else if(laststate==2 && state==0) /* 4T2T */
+			{
+				i+=2;code<<=2;state=-1;
+				code|=1;
+			}
+			else if(laststate==1 && state==1) /* 3T3T */
+			{
+				i+=2;code<<=2;state=-1;
+				code|=2;
+			}
+			else if(laststate==0 && state==2) /* 2T4T */
+			{
+				i+=2;code<<=2;state=-1;
+				code|=3;
+			}
+			else if(laststate==-1)
+			{
+				/* 1st bit */
+			}
+			else
+			{
+				logprintf(LOG_ERR,"invalid state %d:%d",
+					  laststate,state);
+				return((ir_code) -1);
+			}
+			laststate=state;
+		}
+		return(code);
+	}
+	else if(is_serial(remote))
+	{
+		int received;
+		int space, start_bit, stop_bit;
+		lirc_t delta,origdelta,pending,expecting;
+		lirc_t base, stop;
+		
+		base=1000000/remote->baud;
+		
+		/* start bit */
+		set_pending_pulse(base);
+		
+		received=0;
+		space=(rec_buffer.pendingp==0); /* expecting space ? */
+		start_bit=0;
+		stop_bit=0;
+		delta=origdelta=0;
+		stop=base*remote->stop_bits/2;
+		
+		while(received<bits)
+		{
+			if(delta==0)
+			{
+				delta=space ?
+					get_next_space(remote->sone*
+						       remote->bits_in_byte+
+						       base*remote->stop_bits/2):
+					get_next_pulse(remote->pzero*
+						       remote->bits_in_byte);
+				if(delta==0 && space &&
+				   received+remote->bits_in_byte>=bits)
+				{
+					/* open end */
+					delta=remote->sone*
+						remote->bits_in_byte+
+						base*remote->stop_bits/2;
+				}
+				origdelta=delta;
+			}
+			if(delta==0)
+			{
+				LOGPRINTF(1,"failed before bit %d",
+					  received+1);
+				return((ir_code) -1);
+			}
+			pending=(space ?
+				 rec_buffer.pendings:rec_buffer.pendingp);
+			if(delta<=pending)
+			{
+				if(!expect(remote, delta, pending))
+				{
+					LOGPRINTF(1,"failed before bit %d",
+						  received+1);
+					return((ir_code) -1);
+				}
+				delta=0;
+			}
+			else
+			{
+				delta-=pending;
+			}
+			if(pending>0)
+			{
+				if(stop_bit)
+				{
+					delta=0;
+					set_pending_pulse(base);
+					stop_bit=0;
+					space=0;
+					LOGPRINTF(3,"stop bit found");
+				}
+				else
+				{
+					LOGPRINTF(3,"pending bit found");
+					set_pending_pulse(0);
+					set_pending_space(0);
+					if(delta==0)
+					{
+						space=(space ? 0:1);
+					}
+				}
+				continue;
+			}
+			expecting=(space ? remote->sone:remote->pzero);
+			if(delta>expecting || expect(remote,delta,expecting))
+			{
+				delta-=(expecting>delta ? delta:expecting);
+				received++;
+				code<<=1;
+				code|=space;
+				LOGPRINTF(2,"adding %d",space);
+				if(received%remote->bits_in_byte==0)
+				{
+					ir_code temp;
+					
+					temp=code>>remote->bits_in_byte;
+					code=temp<<remote->bits_in_byte|
+						reverse(code,remote->bits_in_byte);
+
+					if(space && delta==0)
+					{
+						LOGPRINTF(1,"failed at stop "
+							  "bit after %d bits",
+							  received+1);
+						return((ir_code) -1);
+					}
+					LOGPRINTF(3,"awaiting stop bit");
+					set_pending_space(stop);
+					stop_bit=1;
+				}				
+			}
+			else
+			{
+				if(delta==origdelta)
+				{
+					LOGPRINTF(1,"framing error after "
+						  "%d bits",received+1);
+					return((ir_code) -1);
+				}
+				delta=0;
+			}
+			if(delta==0)
+			{
+				space=(space ? 0:1);
+			}
+		}
+		set_pending_space(0);
+		return code;
+	}
 	
 	for(i=0;i<bits;i++)
 	{
@@ -621,7 +885,7 @@ ir_code get_pre(struct ir_remote *remote)
 	{
 		if(!expectpulse(remote,remote->pre_p))
 			return((ir_code) -1);
-		rec_buffer.pendings=remote->pre_s;
+		set_pending_space(remote->pre_s);
 	}
 	return(pre);
 }
@@ -634,7 +898,7 @@ ir_code get_post(struct ir_remote *remote)
 	{
 		if(!expectpulse(remote,remote->post_p))
 			return((ir_code) -1);
-		rec_buffer.pendings=remote->post_s;
+		set_pending_space(remote->post_s);
 	}
 
 	post=get_data(remote,remote->post_data_bits,remote->pre_data_bits+
@@ -939,7 +1203,7 @@ int receive_decode(struct ir_remote *remote,
 		/* Most TV cards don't pass each signal to the
                    driver. This heuristic should fix repeat in such
                    cases. */
-		if(time_elapsed(&remote->last_send,&current)<300000)
+		if(time_elapsed(&remote->last_send,&current)<325000)
 		{
 			*repeat_flagp=1;
 		}
