@@ -1,7 +1,7 @@
 /*
  * LIRC base driver
  * 
- * (L) by Artur Lipowski <alipowski@kki.net.pl>
+ * (L) by Artur Lipowski <alipowski@interia.pl>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: lirc_dev.c,v 1.18 2001/12/12 20:26:01 ranty Exp $
+ * $Id: lirc_dev.c,v 1.36 2004/09/05 16:48:48 lirc Exp $
  *
  */
 
@@ -26,14 +26,10 @@
 #endif
  
 #include <linux/version.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
-#define LIRC_HAVE_DEVFS
-#endif
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 4)
-#error "********************************************************"
-#error " Sorry, this driver needs kernel version 2.2.4 or higher"
-#error "********************************************************"
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 18)
+#error "**********************************************************"
+#error " Sorry, this driver needs kernel version 2.2.18 or higher "
+#error "**********************************************************"
 #endif
 
 #include <linux/config.h>
@@ -44,32 +40,37 @@
 #include <linux/ioctl.h>
 #include <linux/fs.h>
 #include <linux/poll.h>
-#ifdef LIRC_HAVE_DEVFS
-#include <linux/devfs_fs_kernel.h>
-#endif
 #include <linux/smp_lock.h>
 #include <asm/uaccess.h>
 #include <asm/semaphore.h>
 #include <asm/errno.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 #include <linux/wrapper.h>
+#endif
 #define __KERNEL_SYSCALLS__
 #include <linux/unistd.h>
+/* DevFS header */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
+#include <linux/devfs_fs_kernel.h>
+#endif
+/* SysFS header */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+#include <linux/device.h>
+#endif
 
+#include "drivers/kcompat.h"
 #include "drivers/lirc.h"
-
 #include "lirc_dev.h"
 
 static int debug = 0;
-
-MODULE_PARM(debug,"i");
+#define dprintk(fmt, args...)                                 \
+	do{                                                   \
+		if(debug) printk(KERN_DEBUG fmt, ## args);    \
+	}while(0)
 
 #define IRCTL_DEV_NAME    "BaseRemoteCtl"
-#define IRLOCK            down_interruptible(&ir->lock)
-#define IRUNLOCK          up(&ir->lock)
 #define SUCCESS           0
 #define NOPLUG            -1
-#define dprintk           if (debug) printk
-
 #define LOGHEAD           "lirc_dev (%s[%d]): "
 
 struct irctl
@@ -77,23 +78,15 @@ struct irctl
 	struct lirc_plugin p;
 	int open;
 
-	unsigned long features;
-	unsigned int buf_len;
-	int bytes_in_key;
-
-	unsigned char buffer[BUFLEN];
-	unsigned int in_buf;
-	int head, tail;
+	struct lirc_buffer *buf;
 
 	int tpid;
 	struct semaphore *t_notify;
+	struct semaphore *t_notify2;
 	int shutdown;
 	long jiffies_to_wait;
 
-	wait_queue_head_t wait_poll;
-
-	struct semaphore lock;
-#ifdef LIRC_HAVE_DEVFS
+#ifdef LIRC_HAVE_DEVFS_24
 	devfs_handle_t devfs_handle;
 #endif
 };
@@ -103,6 +96,8 @@ DECLARE_MUTEX(plugin_lock);
 static struct irctl irctls[MAX_IRCTL_DEVICES];
 static struct file_operations fops;
 
+/* Only used for sysfs but defined to void otherwise */
+static struct class_simple *lirc_class;
 
 /*  helper function
  *  initializes the irctl structure
@@ -112,18 +107,12 @@ static inline void init_irctl(struct irctl *ir)
 	memset(&ir->p, 0, sizeof(struct lirc_plugin));
 	ir->p.minor = NOPLUG;
 
-	ir->buf_len = 0;
-	ir->bytes_in_key = 0;
-	ir->features = 0;
-
 	ir->tpid = -1;
 	ir->t_notify = NULL;
+	ir->t_notify2 = NULL;
 	ir->shutdown = 0;
 	ir->jiffies_to_wait = 0;
 
-	memset(&ir->buffer, 0, BUFLEN);
-	ir->in_buf = 0;
-	ir->head = ir->tail = 0;
 	ir->open = 0;
 }
 
@@ -133,34 +122,37 @@ static inline void init_irctl(struct irctl *ir)
  *  buffer free space is checked and locking performed
  *  returns 0 on success
  */
-
-inline static int add_to_buf(struct irctl *ir)
+static inline int add_to_buf(struct irctl *ir)
 {
-	unsigned char buf[BUFLEN];
-	unsigned int i;
-
-	if (ir->in_buf == ir->buf_len) {
+	if (lirc_buffer_full(ir->buf)) {
 		dprintk(LOGHEAD "buffer overflow\n",
 			ir->p.name, ir->p.minor);
 		return -EOVERFLOW;
 	}
 
-	for (i=0; i < ir->bytes_in_key; i++) {
-		if (ir->p.get_key(ir->p.data, &buf[i], i)) {
-			return -ENODATA;
+	if(ir->p.add_to_buf) {
+		int res = -ENODATA;
+		int got_data = 0;
+		
+		/* service the device as long as it is returning
+		 * data and we have space
+		 */
+		while( !lirc_buffer_full(ir->buf) )
+		{
+			res = ir->p.add_to_buf( ir->p.data, ir->buf );
+			if( res == SUCCESS )
+				got_data++;
+			else
+				break;
 		}
-		dprintk(LOGHEAD "remote code (0x%x) now in buffer\n",
-			ir->p.name, ir->p.minor, buf[i]);
+		
+		if( res == -ENODEV )
+		{
+			ir->shutdown = 1;
+		}
+		return (got_data ? SUCCESS : res);
 	}
-
-	/* here is the only point at which we add key codes to the buffer */
-	IRLOCK;
-	memcpy(&ir->buffer[ir->tail], buf, ir->bytes_in_key);
-	ir->tail += ir->bytes_in_key;
-	ir->tail %= ir->buf_len;
-	ir->in_buf += ir->bytes_in_key;
-	IRUNLOCK;
-
+	
 	return SUCCESS;
 }
 
@@ -170,23 +162,10 @@ static int lirc_thread(void *irctl)
 {
 	struct irctl *ir = irctl;
 	
-	lock_kernel();
-	
 	/* This thread doesn't need any user-level access,
 	 * so get rid of all our resources
 	 */
-	exit_mm(current);
-	exit_files(current);
-	exit_fs(current);
-	current->session = 1;
-	current->pgrp = 1;
-	current->euid = 0;
-	current->tty = NULL;
-	sigfillset(&current->blocked);
-	
-	strcpy(current->comm, "lirc_dev");
-	
-	unlock_kernel();
+	daemonize("lirc_dev");
 	
 	if (ir->t_notify != NULL) {
 		up(ir->t_notify);
@@ -202,19 +181,23 @@ static int lirc_thread(void *irctl)
 			} else {
 				interruptible_sleep_on(ir->p.get_queue(ir->p.data));
 			}
-			if (signal_pending(current)) {
+			if (ir->shutdown) {
 				break;
 			}
 			if (!add_to_buf(ir)) {
-				wake_up_interruptible(&ir->wait_poll);
+				wake_up_interruptible(&ir->buf->wait_poll);
 			}
 		} else {
 			/* if device not opened so we can sleep half a second */
 			current->state = TASK_INTERRUPTIBLE;
 			schedule_timeout(HZ/2);
 		}
-	} while (!ir->shutdown && !signal_pending(current));
+	} while (!ir->shutdown);
 	
+	if (ir->t_notify2 != NULL) {
+		down(ir->t_notify2);
+	}
+
 	ir->tpid = -1;
 	if (ir->t_notify != NULL) {
 		up(ir->t_notify);
@@ -232,7 +215,9 @@ int lirc_register_plugin(struct lirc_plugin *p)
 {
 	struct irctl *ir;
 	int minor;
-#ifdef LIRC_HAVE_DEVFS
+	int bytes_in_key;
+	int err;
+#ifdef LIRC_HAVE_DEVFS_24
 	char name[16];
 #endif
 	DECLARE_MUTEX_LOCKED(tn);
@@ -240,14 +225,16 @@ int lirc_register_plugin(struct lirc_plugin *p)
 	if (!p) {
 		printk("lirc_dev: lirc_register_plugin:"
 		       "plugin pointer must be not NULL!\n");
-		return -EBADRQC;
+		err = -EBADRQC;
+		goto out;
 	}
 
 	if (MAX_IRCTL_DEVICES <= p->minor) {
 		printk("lirc_dev: lirc_register_plugin:"
-		       "\" minor\" must be beetween 0 and %d (%d)!\n",
+		       "\" minor\" must be between 0 and %d (%d)!\n",
 		       MAX_IRCTL_DEVICES-1, p->minor);
-		return -EBADRQC;
+		err = -EBADRQC;
+		goto out;
 	}
 
 	if (1 > p->code_length || (BUFLEN*8) < p->code_length) {
@@ -255,20 +242,40 @@ int lirc_register_plugin(struct lirc_plugin *p)
 		       "code length in bits for minor (%d) "
 		       "must be less than %d!\n",
 		       p->minor, BUFLEN*8);
-		return -EBADRQC;
+		err = -EBADRQC;
+		goto out;
 	}
 
+	printk("lirc_dev: lirc_register_plugin:"
+	       "sample_rate: %d\n",p->sample_rate);
 	if (p->sample_rate) {
-		if (2 > p->sample_rate || 50 < p->sample_rate) {
+		if (2 > p->sample_rate || HZ < p->sample_rate) {
 			printk("lirc_dev: lirc_register_plugin:"
-			       "sample_rate must be beetween 2 and 50!\n");
-			return -EBADRQC;
+			       "sample_rate must be between 2 and %d!\n", HZ);
+			err = -EBADRQC;
+			goto out;
 		}
-	} else {
-		if (!p->get_queue) {
+		if (!p->add_to_buf) {
 			printk("lirc_dev: lirc_register_plugin:"
-			       "get_queue cannot be NULL!\n");
-			return -EBADRQC;
+			       "add_to_buf cannot be NULL when "
+			       "sample_rate is set\n");
+			err = -EBADRQC;
+			goto out;
+		}
+	} else if (!(p->fops && p->fops->read)
+		   && !p->get_queue && !p->rbuf) {
+		printk("lirc_dev: lirc_register_plugin:"
+		       "fops->read, get_queue and rbuf "
+		       "cannot all be NULL!\n");
+		err = -EBADRQC;
+		goto out;
+	} else if (!p->get_queue && !p->rbuf) {
+		if (!(p->fops && p->fops->read && p->fops->poll) 
+		    || (!p->fops->ioctl && !p->ioctl)) {
+			printk("lirc_dev: lirc_register_plugin:"
+			       "neither read, poll nor ioctl can be NULL!\n");
+			err = -EBADRQC;
+			goto out;
 		}
 	}
 
@@ -284,14 +291,14 @@ int lirc_register_plugin(struct lirc_plugin *p)
 		if (MAX_IRCTL_DEVICES == minor) {
 			printk("lirc_dev: lirc_register_plugin: "
 			       "no free slots for plugins!\n");
-			up(&plugin_lock);
-			return -ENOMEM;
+			err = -ENOMEM;
+			goto out_lock;
 		}
 	} else if (irctls[minor].p.minor != NOPLUG) {
 		printk("lirc_dev: lirc_register_plugin:"
-		       "minor (%d) just registerd!\n", minor);
-		up(&plugin_lock);
-		return -EBUSY;
+		       "minor (%d) just registered!\n", minor);
+		err = -EBUSY;
+		goto out_lock;
 	}
 
 	ir = &irctls[minor];
@@ -306,45 +313,84 @@ int lirc_register_plugin(struct lirc_plugin *p)
 	/* some safety check 8-) */
 	p->name[sizeof(p->name)-1] = '\0';
 
-	ir->bytes_in_key = p->code_length/8 + (p->code_length%8 ? 1 : 0);
+	bytes_in_key = p->code_length/8 + (p->code_length%8 ? 1 : 0);
 	
-	/* this simplifies boundary checking during buffer access */
-	ir->buf_len = BUFLEN - (BUFLEN%ir->bytes_in_key);
+	if (p->rbuf) {
+		ir->buf = p->rbuf;
+	} else {
+		ir->buf = kmalloc(sizeof(struct lirc_buffer), GFP_KERNEL);
+		if(!ir->buf) {
+			err = -ENOMEM;
+			goto out_lock;
+		}
+		if(lirc_buffer_init
+		   (ir->buf, bytes_in_key, BUFLEN/bytes_in_key) != 0) {
+			kfree(ir->buf);
+			err = -ENOMEM;
+			goto out_lock;
+		}
+	}
 
-	ir->features = (p->code_length > 8) ?
-		       LIRC_CAN_REC_LIRCCODE : LIRC_CAN_REC_CODE;
+	if (p->features==0)
+		p->features = (p->code_length > 8) ?
+			LIRC_CAN_REC_LIRCCODE : LIRC_CAN_REC_CODE;
 
 	ir->p = *p;
 	ir->p.minor = minor;
 
-#ifdef LIRC_HAVE_DEVFS
+#if defined(LIRC_HAVE_DEVFS_24)
 	sprintf (name, DEV_LIRC "/%d", ir->p.minor);
 	ir->devfs_handle = devfs_register(NULL, name, DEVFS_FL_DEFAULT,
 					  IRCTL_DEV_MAJOR, ir->p.minor,
 					  S_IFCHR | S_IRUSR | S_IWUSR,
 					  &fops, NULL);
+#elif defined(LIRC_HAVE_DEVFS_26)
+	devfs_mk_cdev(MKDEV(IRCTL_DEV_MAJOR, ir->p.minor),
+			S_IFCHR|S_IRUSR|S_IWUSR,
+			DEV_LIRC "/%u", ir->p.minor);
 #endif
+	class_simple_device_add(lirc_class, MKDEV(IRCTL_DEV_MAJOR, ir->p.minor),
+				NULL, "lirc%u", ir->p.minor);
 
-	/* try to fire up polling thread */
-	ir->t_notify = &tn;
-	ir->tpid = kernel_thread(lirc_thread, (void*)ir, 0);
-	if (ir->tpid < 0) {
-		IRUNLOCK;
-		up(&plugin_lock);
-		printk("lirc_dev: lirc_register_plugin:"
-		       "cannot run poll thread for minor = %d\n", p->minor);
-		return -ECHILD;
+	if(p->sample_rate || p->get_queue) {
+		/* try to fire up polling thread */
+		ir->t_notify = &tn;
+		ir->tpid = kernel_thread(lirc_thread, (void*)ir, 0);
+		if (ir->tpid < 0) {
+			printk("lirc_dev: lirc_register_plugin:"
+			       "cannot run poll thread for minor = %d\n",
+			       p->minor);
+			err = -ECHILD;
+			goto out_sysfs;
+		}
+		down(&tn);
+		ir->t_notify = NULL;
 	}
-	down(&tn);
-	ir->t_notify = NULL;
 	up(&plugin_lock);
 
+/*
+ * Recent kernels should handle this autmatically by increasing/decreasing
+ * use count when a dependant module is loaded/unloaded.
+ */
+#ifndef KERNEL_2_5
 	MOD_INC_USE_COUNT;
-
+#endif
 	dprintk("lirc_dev: plugin %s registered at minor number = %d\n",
 		ir->p.name, ir->p.minor);
-
 	return minor;
+	
+out_sysfs:
+	class_simple_device_remove(MKDEV(IRCTL_DEV_MAJOR, ir->p.minor));
+#ifdef LIRC_HAVE_DEVFS_24
+	devfs_unregister(ir->devfs_handle);
+#endif
+#ifdef LIRC_HAVE_DEVFS_26
+	devfs_remove(DEV_LIRC "/%i", ir->p.minor);
+#endif
+out_lock:
+	up(&plugin_lock);
+out:
+	return err;
 }
 
 /*
@@ -354,10 +400,11 @@ int lirc_unregister_plugin(int minor)
 {
 	struct irctl *ir;
 	DECLARE_MUTEX_LOCKED(tn);
+	DECLARE_MUTEX_LOCKED(tn2);
 
 	if (minor < 0 || minor >= MAX_IRCTL_DEVICES) {
 		printk("lirc_dev: lirc_unregister_plugin:"
-		       "\" minor\" must be beetween 0 and %d!\n",
+		       "\" minor\" must be between 0 and %d!\n",
 		       MAX_IRCTL_DEVICES-1);
 		return -EBADRQC;
 	}
@@ -383,23 +430,51 @@ int lirc_unregister_plugin(int minor)
 	/* end up polling thread */
 	if (ir->tpid >= 0) {
 		ir->t_notify = &tn;
+		ir->t_notify2 = &tn2;
 		ir->shutdown = 1;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
+		{
+			struct task_struct *p;
+			
+			p = find_task_by_pid(ir->tpid);
+			wake_up_process(p);
+		}
+#else
+		/* 2.2.x does not export wake_up_process() */
+		wake_up_interruptible(ir->p.get_queue(ir->p.data));
+#endif
+		up(&tn2);
 		down(&tn);
 		ir->t_notify = NULL;
+		ir->t_notify2 = NULL;
 	}
 
 	dprintk("lirc_dev: plugin %s unregistered from minor number = %d\n",
 		ir->p.name, ir->p.minor);
 
-#ifdef LIRC_HAVE_DEVFS
+#ifdef LIRC_HAVE_DEVFS_24
 	devfs_unregister(ir->devfs_handle);
 #endif
+#ifdef LIRC_HAVE_DEVFS_26
+	devfs_remove(DEV_LIRC "/%u", ir->p.minor);
+#endif
+	class_simple_device_remove(MKDEV(IRCTL_DEV_MAJOR, ir->p.minor));
 
+	if (ir->buf != ir->p.rbuf){
+		lirc_buffer_free(ir->buf);
+		kfree(ir->buf);
+	}
+	ir->buf = NULL;
 	init_irctl(ir);
-
 	up(&plugin_lock);
 
+/*
+ * Recent kernels should handle this autmatically by increasing/decreasing
+ * use count when a dependant module is loaded/unloaded.
+ */
+#ifndef KERNEL_2_5
 	MOD_DEC_USE_COUNT;
+#endif
 
 	return SUCCESS;
 }
@@ -410,6 +485,7 @@ int lirc_unregister_plugin(int minor)
 static int irctl_open(struct inode *inode, struct file *file)
 {
 	struct irctl *ir;
+	int retval;
 	
 	if (MINOR(inode->i_rdev) >= MAX_IRCTL_DEVICES) {
 		dprintk("lirc_dev [%d]: open result = -ENODEV\n",
@@ -421,6 +497,10 @@ static int irctl_open(struct inode *inode, struct file *file)
 
 	dprintk(LOGHEAD "open called\n", ir->p.name, ir->p.minor);
 
+	/* if the plugin has an open function use it instead */
+	if(ir->p.fops && ir->p.fops->open)
+		return ir->p.fops->open(inode, file);
+
 	down_interruptible(&plugin_lock);
 
 	if (ir->p.minor == NOPLUG) {
@@ -437,17 +517,24 @@ static int irctl_open(struct inode *inode, struct file *file)
 		return -EBUSY;
 	}
 
-	/* rhere is no need for locking here because ir->open is 0 
+	/* there is no need for locking here because ir->open is 0 
          * and lirc_thread isn't using buffer
+	 * plugins which use irq's should allocate them on set_use_inc,
+	 * so there should be no problem with those either.
          */
-	ir->head = ir->tail;
-	ir->in_buf = 0;
+	ir->buf->head = ir->buf->tail;
+	ir->buf->fill = 0;
 
 	++ir->open;
-	ir->p.set_use_inc(ir->p.data);
+	retval = ir->p.set_use_inc(ir->p.data);
 
 	up(&plugin_lock);
 
+	if (retval != SUCCESS) {
+		--ir->open;
+		return retval;
+	}
+
 	dprintk(LOGHEAD "open result = %d\n", ir->p.name, ir->p.minor, SUCCESS);
 
 	return SUCCESS;
@@ -462,6 +549,10 @@ static int irctl_close(struct inode *inode, struct file *file)
 
 	dprintk(LOGHEAD "close called\n", ir->p.name, ir->p.minor);
  
+	/* if the plugin has a close function use it instead */
+	if(ir->p.fops && ir->p.fops->release)
+		return ir->p.fops->release(inode, file);
+
 	down_interruptible(&plugin_lock);
 
 	--ir->open;
@@ -481,15 +572,17 @@ static unsigned int irctl_poll(struct file *file, poll_table *wait)
 
 	dprintk(LOGHEAD "poll called\n", ir->p.name, ir->p.minor);
 
-	if (!ir->in_buf) {
-		poll_wait(file, &ir->wait_poll, wait);
-	}
+	/* if the plugin has a poll function use it instead */
+	if(ir->p.fops && ir->p.fops->poll)
+		return ir->p.fops->poll(file, wait);
+
+	poll_wait(file, &ir->buf->wait_poll, wait);
 
 	dprintk(LOGHEAD "poll result = %s\n",
 		ir->p.name, ir->p.minor, 
-		ir->in_buf ? "POLLIN|POLLRDNORM" : "SUCCESS");
+		lirc_buffer_empty(ir->buf) ? "0" : "POLLIN|POLLRDNORM");
 
-	return ir->in_buf ? (POLLIN|POLLRDNORM) : SUCCESS;
+	return lirc_buffer_empty(ir->buf) ? 0 : (POLLIN|POLLRDNORM);
 }
 
 /*
@@ -499,32 +592,55 @@ static int irctl_ioctl(struct inode *inode, struct file *file,
                        unsigned int cmd, unsigned long arg)
 {
 	unsigned long mode;
-	int result = SUCCESS;
+	int result;
 	struct irctl *ir = &irctls[MINOR(inode->i_rdev)];
 
-	dprintk(LOGHEAD "poll called (%u)\n",
+	dprintk(LOGHEAD "ioctl called (%u)\n",
 		ir->p.name, ir->p.minor, cmd);
 
+	/* if the plugin has a ioctl function use it instead */
+	if(ir->p.fops && ir->p.fops->ioctl)
+		return ir->p.fops->ioctl(inode, file, cmd, arg);
+
 	if (ir->p.minor == NOPLUG) {
 		dprintk(LOGHEAD "ioctl result = -ENODEV\n",
 			ir->p.name, ir->p.minor);
 		return -ENODEV;
 	}
 
+	/* Give the plugin a chance to handle the ioctl */
+	if(ir->p.ioctl){
+		result = ir->p.ioctl(inode, file, cmd, arg);
+		if (result != -ENOIOCTLCMD)
+			return result;
+	}
+	/* The plugin can't handle cmd */
+	result = SUCCESS;
+
 	switch(cmd)
 	{
 	case LIRC_GET_FEATURES:
-		result = put_user(ir->features, (unsigned long*)arg);
+		result = put_user(ir->p.features, (unsigned long*)arg);
 		break;
 	case LIRC_GET_REC_MODE:
-		result = put_user(LIRC_REC2MODE(ir->features),
+		if(!(ir->p.features&LIRC_CAN_REC_MASK))
+			return(-ENOSYS);
+		
+		result = put_user(LIRC_REC2MODE
+				  (ir->p.features&LIRC_CAN_REC_MASK),
 				  (unsigned long*)arg);
 		break;
 	case LIRC_SET_REC_MODE:
+		if(!(ir->p.features&LIRC_CAN_REC_MASK))
+			return(-ENOSYS);
+
 		result = get_user(mode, (unsigned long*)arg);
-		if(!result && !(LIRC_MODE2REC(mode) & ir->features)) {
+		if(!result && !(LIRC_MODE2REC(mode) & ir->p.features)) {
 			result = -EINVAL;
 		}
+		/* FIXME: We should actually set the mode somehow 
+		 * but for now, lirc_serial doesn't support mode changin
+		 * eighter */
 		break;
 	case LIRC_GET_LENGTH:
 		result = put_user((unsigned long)ir->p.code_length, 
@@ -548,72 +664,91 @@ static ssize_t irctl_read(struct file *file,
 			  size_t length, 
 			  loff_t *ppos)     
 {
-	unsigned char buf[BUFLEN];
 	struct irctl *ir = &irctls[MINOR(file->f_dentry->d_inode->i_rdev)];
-	int ret;
+	unsigned char buf[ir->buf->chunk_size];
+	int ret=0, written=0;
 	DECLARE_WAITQUEUE(wait, current);
 
 	dprintk(LOGHEAD "read called\n", ir->p.name, ir->p.minor);
 
-	if (ir->bytes_in_key != length) {
-		dprintk(LOGHEAD "read result = -EIO\n",
+	/* if the plugin has a specific read function use it instead */
+	if(ir->p.fops && ir->p.fops->read)
+		return ir->p.fops->read(file, buffer, length, ppos);
+
+	if (length % ir->buf->chunk_size) {
+		dprintk(LOGHEAD "read result = -EINVAL\n",
 			ir->p.name, ir->p.minor);
-		return -EIO;
+		return -EINVAL;
 	}
 
 	/* we add ourselves to the task queue before buffer check 
          * to avoid losing scan code (in case when queue is awaken somewhere 
 	 * beetwen while condition checking and scheduling)
 	 */
-	add_wait_queue(&ir->wait_poll, &wait);
+	add_wait_queue(&ir->buf->wait_poll, &wait);
 	current->state = TASK_INTERRUPTIBLE;
 
-	/* while input buffer is empty and device opened in blocking mode, 
-	 * wait for input 
+	/* while we did't provide 'length' bytes, device is opened in blocking
+	 * mode and 'copy_to_user' is happy, wait for data.
 	 */
-	while (!ir->in_buf) {
-		if (file->f_flags & O_NONBLOCK) {
-			dprintk(LOGHEAD "read result = -EWOULDBLOCK\n", 
-				ir->p.name, ir->p.minor);
-			remove_wait_queue(&ir->wait_poll, &wait);
-			current->state = TASK_RUNNING;
-			return -EWOULDBLOCK;
+	while (written < length && ret == 0) { 
+		if (lirc_buffer_empty(ir->buf)) {
+			/* According to the read(2) man page, 'written' can be
+			 * returned as less than 'length', instead of blocking
+			 * again, returning -EWOULDBLOCK, or returning
+			 * -ERESTARTSYS */
+			if (written) break;
+			if (file->f_flags & O_NONBLOCK) {
+				dprintk(LOGHEAD "read result = -EWOULDBLOCK\n", 
+						ir->p.name, ir->p.minor);
+				remove_wait_queue(&ir->buf->wait_poll, &wait);
+				current->state = TASK_RUNNING;
+				return -EWOULDBLOCK;
+			}
+			if (signal_pending(current)) {
+				dprintk(LOGHEAD "read result = -ERESTARTSYS\n", 
+						ir->p.name, ir->p.minor);
+				remove_wait_queue(&ir->buf->wait_poll, &wait);
+				current->state = TASK_RUNNING;
+				return -ERESTARTSYS;
+			}
+			schedule();
+			current->state = TASK_INTERRUPTIBLE;
+		} else {
+			lirc_buffer_read_1(ir->buf, buf);
+			ret = copy_to_user((void *)buffer+written, buf,
+					   ir->buf->chunk_size);
+			written += ir->buf->chunk_size;
 		}
-		if (signal_pending(current)) {
-			dprintk(LOGHEAD "read result = -ERESTARTSYS\n", 
-				ir->p.name, ir->p.minor);
-			remove_wait_queue(&ir->wait_poll, &wait);
-			current->state = TASK_RUNNING;
-			return -ERESTARTSYS;
-		}
-		schedule();
-		current->state = TASK_INTERRUPTIBLE;
 	}
 
-	remove_wait_queue(&ir->wait_poll, &wait);
+	remove_wait_queue(&ir->buf->wait_poll, &wait);
 	current->state = TASK_RUNNING;
 
-	/* here is the only point at which we remove key codes from 
-	 * the buffer
-	 */
-	IRLOCK;
-	memcpy(buf, &ir->buffer[ir->head], length);
-	ir->head += length;
-	ir->head %= ir->buf_len;
-	ir->in_buf -= length;
-	IRUNLOCK;
-
-	ret = copy_to_user(buffer, buf, length);
-
 	dprintk(LOGHEAD "read result = %s (%d)\n",
 		ir->p.name, ir->p.minor, ret ? "-EFAULT" : "OK", ret);
 
-	return ret ? -EFAULT : length;
+	return ret ? -EFAULT : written;
+}
+
+static ssize_t irctl_write(struct file *file, const char *buffer,
+			   size_t length, loff_t * ppos)
+{
+	struct irctl *ir = &irctls[MINOR(file->f_dentry->d_inode->i_rdev)];
+
+	dprintk(LOGHEAD "read called\n", ir->p.name, ir->p.minor);
+
+	/* if the plugin has a specific read function use it instead */
+	if(ir->p.fops && ir->p.fops->write)
+		return ir->p.fops->write(file, buffer, length, ppos);
+
+	return -EINVAL;
 }
 
 
 static struct file_operations fops = {
 	read:    irctl_read, 
+	write:   irctl_write,
 	poll:    irctl_poll,
 	ioctl:   irctl_ioctl,
 	open:    irctl_open,
@@ -628,33 +763,35 @@ EXPORT_SYMBOL(lirc_unregister_plugin);
 /*
  *
  */
-int lirc_dev_init(void)
+static int lirc_dev_init(void)
 {  	
 	int i;
 
 	for (i=0; i < MAX_IRCTL_DEVICES; ++i) {
 		init_irctl(&irctls[i]);	
-		init_MUTEX(&irctls[i].lock);
-		init_waitqueue_head(&irctls[i].wait_poll);
 	}
 
-#ifndef LIRC_HAVE_DEVFS
- 	i = register_chrdev(IRCTL_DEV_MAJOR,
-#else
-	i = devfs_register_chrdev(IRCTL_DEV_MAJOR,
-#endif
-				   IRCTL_DEV_NAME,
-				   &fops);
-	
-	if (i < 0) {
-		printk ("lirc_dev: device registration failed with %d\n", i);
-		return i;
+	if(register_chrdev(IRCTL_DEV_MAJOR, IRCTL_DEV_NAME, &fops)) {
+		printk(KERN_ERR "lirc_dev: register_chrdev failed\n");
+		goto out;
 	}
-	
+
+	lirc_class = class_simple_create(THIS_MODULE, "lirc");
+	if(IS_ERR(lirc_class)) {
+		printk(KERN_ERR "lirc_dev: class_simple_create failed\n");
+		goto out_unregister;
+	}
+
 	printk("lirc_dev: IR Remote Control driver registered, at major %d \n", 
 	       IRCTL_DEV_MAJOR);
 
 	return SUCCESS;
+
+out_unregister:
+	if(unregister_chrdev(IRCTL_DEV_MAJOR, IRCTL_DEV_NAME))
+		printk(KERN_ERR "lirc_dev: unregister_chrdev failed!\n");
+out:
+	return -1;
 }
 
 /* ---------------------------------------------------------------------- */
@@ -663,12 +800,6 @@ int lirc_dev_init(void)
 
 #ifdef MODULE
 
-MODULE_DESCRIPTION("LIRC base driver module");
-MODULE_AUTHOR("Artur Lipowski");
-#ifdef MODULE_LICENSE
-MODULE_LICENSE("GPL");
-#endif
-
 /*
  *
  */
@@ -683,21 +814,25 @@ int init_module(void)
 void cleanup_module(void)
 {
 	int ret;
-	
-#ifndef LIRC_HAVE_DEVFS
- 	ret = unregister_chrdev(IRCTL_DEV_MAJOR, IRCTL_DEV_NAME);
-#else
-	ret = devfs_unregister_chrdev(IRCTL_DEV_MAJOR, IRCTL_DEV_NAME);
-#endif
- 
-	if (0 > ret){
-		printk("lirc_dev: error in module_unregister_chrdev: %d\n",
-		       ret);
-	} else {
+
+	ret = unregister_chrdev(IRCTL_DEV_MAJOR, IRCTL_DEV_NAME);
+	class_simple_destroy(lirc_class);
+
+	if(ret)
+		printk("lirc_dev: error in module_unregister_chrdev: %d\n", ret);
+	else
 		dprintk("lirc_dev: module successfully unloaded\n");
-	}
 }
-#endif
+
+MODULE_DESCRIPTION("LIRC base driver module");
+MODULE_AUTHOR("Artur Lipowski");
+MODULE_LICENSE("GPL");
+MODULE_ALIAS_CHARDEV_MAJOR(IRCTL_DEV_MAJOR);
+
+module_param(debug, bool, 0644);
+MODULE_PARM_DESC(debug, "Enable debugging messages");
+
+#endif /* MODULE */
 
 /*
  * Overrides for Emacs so that we follow Linus's tabbing style.
