@@ -2,11 +2,14 @@
  * Remote control driver for the TV-card
  * key codes are obtained from GPIO port
  * 
- * (L) by Artur Lipowski <alipowski@kki.net.pl>
+ * (L) by Artur Lipowski <alipowski@interia.pl>
  *     patch for the AverMedia by Santiago Garcia Mantinan <manty@i.am>
  *                            and Christoph Bartelmus <lirc@bartelmus.de>
  *     patch for the BestBuy by Miguel Angel Alvarez <maacruz@navegalia.com>
- *     patch for the Winfast TV2000 by Juan Toledo <toledo@users.sourceforge.net>
+ *     patch for the Winfast TV2000 by Juan Toledo
+ *     <toledo@users.sourceforge.net>
+ *     patch for the I-O Data GV-BCTV5/PCI by Jens C. Rasmussen
+ *     <jens.rasmussen@ieee.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +25,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: lirc_gpio.c,v 1.16 2002/08/30 11:57:28 lirc Exp $
+ * $Id: lirc_gpio.c,v 1.40 2004/08/07 10:06:08 lirc Exp $
  *
  */
 
@@ -33,16 +36,16 @@
 #error "*******************************************************"
 #endif
 
+#include <linux/version.h>
 #include <linux/module.h>
 #include <linux/kmod.h>
 #include <linux/sched.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 #include <linux/wrapper.h>
+#endif
 #include <linux/errno.h>
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
-/* comes with bttv */
-#include "../drivers/char/kcompat24.h"
-
 #include "../drivers/char/bttv.h"
 #include "../drivers/char/bttvp.h"
 #else
@@ -58,28 +61,25 @@
 #error "*******************************************************"
 #endif
 
-#include "../lirc_dev/lirc_dev.h"
+#include "drivers/kcompat.h"
+#include "drivers/lirc_dev/lirc_dev.h"
 
+/* insmod parameters */
 static int debug = 0;
 static int card = 0;
 static int minor = -1;
+static int bttv_id = BTTV_UNKNOWN;
 static unsigned long gpio_mask = 0;
 static unsigned long gpio_enable = 0;
 static unsigned long gpio_lock_mask = 0;
 static unsigned long gpio_xor_mask = 0;
-static unsigned int soft_gap = 0;
-static unsigned char sample_rate = 10;
+static int soft_gap = 0;
+static int sample_rate = 10;
 
-MODULE_PARM(debug,"i");
-MODULE_PARM(card,"i");
-MODULE_PARM(minor,"i");
-MODULE_PARM(gpio_mask,"l");
-MODULE_PARM(gpio_lock_mask,"l");
-MODULE_PARM(gpio_xor_mask,"l");
-MODULE_PARM(soft_gap,"i");
-MODULE_PARM(sample_rate,"b");
-
-#define dprintk  if (debug) printk
+#define dprintk(fmt, args...)                                 \
+	do{                                                   \
+		if(debug) printk(KERN_DEBUG fmt, ## args);    \
+	}while(0)
 
 struct rcv_info {
 	int bttv_id;
@@ -88,31 +88,36 @@ struct rcv_info {
 	unsigned long gpio_enable;
 	unsigned long gpio_lock_mask;
 	unsigned long gpio_xor_mask;
-	unsigned int soft_gap;
-	unsigned char sample_rate;
+	int soft_gap;
+	int sample_rate;
 	unsigned char code_length;
 };
 
 static struct rcv_info rcv_infos[] = {
 	{BTTV_UNKNOWN,                0,          0,          0,         0,          0,   0,  1,  0},
-#ifdef BTTV_PXELVWPLTVPAK
-	{BTTV_PXELVWPLTVPAK,          0, 0x00003e00,          0, 0x0010000,          0,   0, 15,  0},
-#endif
+	{BTTV_PXELVWPLTVPAK,          0, 0x00003e00,          0, 0x0010000,          0,   0, 15, 32},
 	{BTTV_PXELVWPLTVPRO,          0, 0x00001f00,          0, 0x0008000,          0, 500, 12, 32},
-#ifdef BTTV_PV_BT878P_9B
 	{BTTV_PV_BT878P_9B,           0, 0x00001f00,          0, 0x0008000,          0, 500, 12, 32},
-#endif
+	{BTTV_PV_BT878P_PLUS,         0, 0x00001f00,          0, 0x0008000,          0, 500, 12, 32},
 	{BTTV_AVERMEDIA,              0, 0x00f88000,          0, 0x0010000, 0x00010000,   0, 10, 32},
 	{BTTV_AVPHONE98,     0x00011461, 0x003b8000, 0x00004000, 0x0800000, 0x00800000,   0, 10,  0}, /*mapped to Capture98*/
 	{BTTV_AVERMEDIA98,   0x00021461, 0x003b8000, 0x00004000, 0x0800000, 0x00800000,   0, 10,  0}, /*mapped to Capture98*/
 	{BTTV_AVPHONE98,     0x00031461, 0x00f88000,          0, 0x0010000, 0x00010000,   0, 10, 32}, /*mapped to Phone98*/
 	/* is this one correct? */
 	{BTTV_AVERMEDIA98,   0x00041461, 0x00f88000,          0, 0x0010000, 0x00010000,   0, 10, 32}, /*mapped to Phone98*/
+	/* work-around for VDOMATE */
+	{BTTV_AVERMEDIA98,   0x03001461, 0x00f88000,          0, 0x0010000, 0x00010000,   0, 10, 32}, /*mapped to Phone98*/
+	/* reported by Danijel Korzinek, AVerTV GOw/FM */
+	{BTTV_AVERMEDIA98,   0x00000000, 0x00f88000,          0, 0x0010000, 0x00010000,   0, 10, 32}, /*mapped to Phone98*/
 	{BTTV_CHRONOS_VS2,            0, 0x000000f8,          0, 0x0000100,          0,   0, 20,  0},
 	/* CPH031 and CPH033 cards (?) */
 	/* MIRO was just a work-around */
 	{BTTV_MIRO,                   0, 0x00001f00,          0, 0x0004000,          0,   0, 10, 32},
 	{BTTV_DYNALINK,               0, 0x00001f00,          0, 0x0004000,          0,   0, 10, 32},
+	{BTTV_WINVIEW_601,            0, 0x00001f00,          0, 0x0004000,          0,   0,  0, 32},
+#ifdef BTTV_KWORLD
+	{BTTV_KWORLD,                 0, 0x00007f00,          0, 0x0004000,          0,   0, 12, 32},
+#endif
 	/* just a guess */
 	{BTTV_MAGICTVIEW061,          0, 0x0028e000,          0, 0x0020000,          0,   0, 20, 32},
  	{BTTV_MAGICTVIEW063,          0, 0x0028e000,          0, 0x0020000,          0,   0, 20, 32},
@@ -121,20 +126,29 @@ static struct rcv_info rcv_infos[] = {
         {BTTV_BESTBUY_EASYTV,         0, 0x00007F00,          0, 0x0004000,          0,   0, 10,  8},
         {BTTV_BESTBUY_EASYTV2,        0, 0x00007F00,          0, 0x0008000,          0,   0, 10,  8},
 #endif
-	{BTTV_FLYVIDEO,               0, 0x000000ff,          0,         0,          0,   0,  0, 42},
-	/* probably                      0x000000f8 */
- 	{BTTV_FLYVIDEO_98,            0, 0x000001f8,          0, 0x0000100,          0,   0,  0,  0},
+	/* lock_mask probably also 0x100, or maybe it is 0x0 for all others !?! */
+	{BTTV_FLYVIDEO,               0, 0x000000f8,          0,         0,          0,   0,  0, 42},
+ 	{BTTV_FLYVIDEO_98,            0, 0x000000f8,          0, 0x0000100,          0,   0,  0, 42},
+ 	{BTTV_TYPHOON_TVIEW,          0, 0x000000f8,          0, 0x0000100,          0,   0,  0, 42},
 #ifdef BTTV_FLYVIDEO_98FM
 	/* smorar@alfonzo.smuts.uct.ac.za */
-	/* probably                      0x000000f8 */
- 	{BTTV_FLYVIDEO_98FM,          0, 0x000001f8,          0, 0x0000100,          0,   0,  0,  0},
+	{BTTV_FLYVIDEO_98FM,          0, 0x000000f8,          0, 0x0000100,          0,   0,  0, 42},
+#endif
+	/* The Leadtek WinFast TV 2000 XP card (id 0x6606107d) uses an
+	 * extra gpio bit compared to the original TV 2000 card (id
+	 * 0x217d6606); as the bttv-0.7.100 driver does not
+	 * distinguish between the two cards, we enable the extra bit
+	 * based on the card id: */
+	{BTTV_WINFAST2000,   0x6606107d, 0x000008f8,          0, 0x0000100,          0,   0,  0, 32},
+	/* default: */
+	{BTTV_WINFAST2000,            0, 0x000000f8,          0, 0x0000100,          0,   0,  0, 32},
+#ifdef BTTV_GVBCTV5PCI
+	{BTTV_GVBCTV5PCI,             0, 0x00f0b000,          0,         0,          0,   0, 20,  8},
 #endif
-        {BTTV_WINFAST2000,            0, 0x000000f8,          0, 0x0000100,          0,   0,  0,  0}
 };
 
 static unsigned char code_length = 0;
 static unsigned char code_bytes = 1;
-static int card_type = 0;
 
 #define MAX_BYTES 8
 
@@ -171,8 +185,8 @@ static int build_key(unsigned long gpio_val, unsigned char codes[MAX_BYTES])
 	if (gpio_lock_mask && (gpio_val & gpio_lock_mask)) {
 		return -EBUSY;
 	}
-
-	switch (rcv_infos[card_type].bttv_id)
+	
+	switch (bttv_id)
 	{
 	case BTTV_AVERMEDIA98:
 		if (bttv_write_gpio(card, gpio_enable, gpio_enable)) {
@@ -204,7 +218,7 @@ static int build_key(unsigned long gpio_val, unsigned char codes[MAX_BYTES])
 	}
 	
 	dprintk(LOGHEAD "code is %lx\n",card,(unsigned long) codes[0]);
-	switch (rcv_infos[card_type].bttv_id)
+	switch (bttv_id)
 	{
 	case BTTV_AVERMEDIA:
 		codes[2] = (codes[0]<<2)&0xff;
@@ -226,12 +240,18 @@ static int build_key(unsigned long gpio_val, unsigned char codes[MAX_BYTES])
 	case BTTV_AVERMEDIA98:
 		break;
 	case BTTV_FLYVIDEO:
-		codes[5]=codes[0];
-		codes[0]=0x03;
-		codes[1]=0xE5;
-		codes[2]=0xE0;
-		codes[3]=0xD0;
-		codes[4]=((~codes[0])&0xff);
+	case BTTV_FLYVIDEO_98:
+	case BTTV_TYPHOON_TVIEW:
+#ifdef BTTV_FLYVIDEO_98FM
+	case BTTV_FLYVIDEO_98FM:
+#endif
+		codes[4]=codes[0]<<3;
+		codes[5]=((~codes[4])&0xff);
+		
+		codes[0]=0x00;
+		codes[1]=0x1A;
+		codes[2]=0x1F;
+		codes[3]=0x2F;
 		break;
         case BTTV_MAGICTVIEW061:
         case BTTV_MAGICTVIEW063:
@@ -244,9 +264,12 @@ static int build_key(unsigned long gpio_val, unsigned char codes[MAX_BYTES])
 		/* FALLTHROUGH */
 	case BTTV_MIRO:
 	case BTTV_DYNALINK:
+	case BTTV_PXELVWPLTVPAK:
 	case BTTV_PXELVWPLTVPRO:
-#ifdef BTTV_PV_BT878P_9B
 	case BTTV_PV_BT878P_9B:
+	case BTTV_PV_BT878P_PLUS:
+#ifdef BTTV_KWORLD
+	case BTTV_KWORLD:
 #endif
 		codes[2] = reverse(codes[0],8);
 		codes[3] = (~codes[2])&0xff;
@@ -268,6 +291,15 @@ static int build_key(unsigned long gpio_val, unsigned char codes[MAX_BYTES])
 		codes[3]=0x2F;
 		break;
 #endif
+	case BTTV_WINFAST2000:
+		/* shift extra bit */
+		codes[0] = (codes[0]&0x1f) | ((codes[0]&0x20) << 1);
+	case BTTV_WINVIEW_601:
+		codes[2] = reverse(codes[0],8);
+		codes[3] = (~codes[2])&0xff;
+		codes[0] = 0xC0;
+		codes[1] = 0x3F;
+		break;
 	default:
 		break;
 	}
@@ -275,49 +307,41 @@ static int build_key(unsigned long gpio_val, unsigned char codes[MAX_BYTES])
 	return SUCCESS;
 }
 
-static int get_key(void* data, unsigned char *key, int key_no)
+/* add_to_buf - copy a code to the buffer */
+static int add_to_buf(void* data, struct lirc_buffer* buf)
 {
 	static unsigned long next_time = 0;
-	static unsigned char codes[MAX_BYTES];
+	static unsigned char prev_codes[MAX_BYTES];
 	unsigned long code = 0;
 	unsigned char cur_codes[MAX_BYTES];
-	
-	if (key_no > 0)	{
-		if (code_bytes < 2 || key_no >= code_bytes) {
-			dprintk(LOGHEAD "something wrong in get_key\n", card);
-			return -EBADRQC;
-		}
-		*key = codes[key_no];
-		return SUCCESS;
-	}
-	
+    
 	if (bttv_read_gpio(card, &code)) {
 		dprintk(LOGHEAD "cannot read GPIO\n", card);
 		return -EIO;
 	}
-
+	
 	if (build_key(code, cur_codes)) {
 		return -EFAULT;
 	}
-
+	
 	if (soft_gap) {
-		if (!memcmp(codes, cur_codes, code_bytes) && 
-		    jiffies < next_time) {
+		if (!memcmp(prev_codes, cur_codes, code_bytes) &&
+			jiffies < next_time) {
 			return -EAGAIN;
 		}
 		next_time = jiffies + soft_gap;
 	}
-
-	memcpy(codes, cur_codes, code_bytes);
-
-	*key = codes[0];
-
+	memcpy( prev_codes, cur_codes, code_bytes );
+		
+	lirc_buffer_write_1( buf, cur_codes );
+		
 	return SUCCESS;
 }
 
-static void set_use_inc(void* data)
+static int set_use_inc(void* data)
 {
 	MOD_INC_USE_COUNT;
+	return 0;
 }
 
 static void set_use_dec(void* data)
@@ -331,21 +355,17 @@ static wait_queue_head_t* get_queue(void* data)
 }
 
 static struct lirc_plugin plugin = {
-	"lirc_gpio  ",
-	0,
-	0,
-	0,
-	NULL,
-	get_key,
-	get_queue,
-	set_use_inc,
-	set_use_dec
+	.name		= "lirc_gpio  ",
+	.add_to_buf	= add_to_buf,
+	.get_queue	= get_queue,
+	.set_use_inc	= set_use_inc,
+	.set_use_dec	= set_use_dec,
 };
 
 /*
  *
  */
-int gpio_remote_init(void)
+static int gpio_remote_init(void)
 {  	
 	int ret;
 	unsigned int mask;
@@ -407,23 +427,13 @@ int gpio_remote_init(void)
 	return SUCCESS;
 }
 
-EXPORT_NO_SYMBOLS; 
-
-/* Dont try to use it as a static version !  */
-
 #ifdef MODULE
-MODULE_DESCRIPTION("Driver module for remote control (data from bt848 GPIO port)");
-MODULE_AUTHOR("Artur Lipowski");
-#ifdef MODULE_LICENSE
-MODULE_LICENSE("GPL");
-#endif
-
 /*
  *
  */
 int init_module(void)
 {
-	int type,cardid;
+	int type,cardid,card_type;
 
 	if (MAX_IRCTL_DEVICES < minor) {
 		printk("lirc_gpio: parameter minor (%d) must be less than %d!\n",
@@ -437,22 +447,24 @@ int init_module(void)
 	 * instead of autodetecting TV card
 	 */
 	if (gpio_mask) {
-		if (2 > sample_rate || 50 < sample_rate) {
+		if (sample_rate!=0 &&
+		    (2 > sample_rate || HZ < sample_rate)) {
 			printk(LOGHEAD "parameter sample_rate "
-			       "must be beetween 2 and 50!\n", minor);
+			       "must be between 2 and %d!\n", minor, HZ);
 			return -EBADRQC;
 		}
 
-		if (soft_gap && 
+		if (sample_rate!=0 && soft_gap && 
 		    ((2000/sample_rate) > soft_gap || 1000 < soft_gap)) {
 			printk(LOGHEAD "parameter soft_gap "
-			       "must be beetween %d and 1000!\n",
+			       "must be between %d and 1000!\n",
 			       minor, 2000/sample_rate);
 			return -EBADRQC;
 		}
 	} else {
 		if(bttv_get_cardinfo(card,&type,&cardid)==-1) {
 			printk(LOGHEAD "could not get card type\n", minor);
+			return -EBADRQC;
 		}
 		printk(LOGHEAD "card type 0x%x, id 0x%x\n",minor,
 		       type,cardid);
@@ -468,6 +480,7 @@ int init_module(void)
 			if (rcv_infos[card_type].bttv_id == type &&
 			    (rcv_infos[card_type].card_id == 0 ||
 			     rcv_infos[card_type].card_id == cardid)) {
+				bttv_id = rcv_infos[card_type].bttv_id;
 				gpio_mask = rcv_infos[card_type].gpio_mask;
 				gpio_enable = rcv_infos[card_type].gpio_enable;
 				gpio_lock_mask = rcv_infos[card_type].gpio_lock_mask;
@@ -479,13 +492,19 @@ int init_module(void)
 			}
 		}
 		if (type==BTTV_AVPHONE98 && cardid==0x00011461)	{
-			rcv_infos[card_type].bttv_id = BTTV_AVERMEDIA98;
+			bttv_id = BTTV_AVERMEDIA98;
 		}
 		if (type==BTTV_AVERMEDIA98 && cardid==0x00041461) {
-			rcv_infos[card_type].bttv_id = BTTV_AVPHONE98;
+			bttv_id = BTTV_AVPHONE98;
+		}
+		if (type==BTTV_AVERMEDIA98 && cardid==0x03001461) {
+			bttv_id = BTTV_AVPHONE98;
+		}
+		if (type==BTTV_AVERMEDIA98 && cardid==0x00000000) {
+			bttv_id = BTTV_AVPHONE98;
 		}
 		if (card_type == sizeof(rcv_infos)/sizeof(struct rcv_info)) {
-			printk(LOGHEAD "TV card type %x not supported!\n",
+			printk(LOGHEAD "TV card type 0x%x not supported!\n",
 			       minor, type);
 			return -EBADRQC;
 		}
@@ -524,7 +543,42 @@ void cleanup_module(void)
 
 	dprintk(LOGHEAD "module successfully unloaded\n", minor);
 }
-#endif
+
+/* Dont try to use it as a static version !  */
+MODULE_DESCRIPTION("Driver module for remote control (data from bt848 GPIO port)");
+MODULE_AUTHOR("Artur Lipowski");
+MODULE_LICENSE("GPL");
+
+module_param(minor, int, 0444);
+MODULE_PARM_DESC(minor, "Preferred minor device number");
+
+module_param(card, int, 0444);
+MODULE_PARM_DESC(card, "TV card number to attach to");
+
+module_param(gpio_mask, long, 0444);
+MODULE_PARM_DESC(gpio_mask, "gpio_mask");
+
+module_param(gpio_lock_mask, long, 0444);
+MODULE_PARM_DESC(gpio_lock_mask, "gpio_lock_mask");
+
+module_param(gpio_xor_mask, long, 0444);
+MODULE_PARM_DESC(gpio_xor_mask, "gpio_xor_mask");
+
+module_param(soft_gap, int, 0444);
+MODULE_PARM_DESC(soft_gap, "Time between keypresses (in ms)");
+
+module_param(sample_rate, int, 0444);
+MODULE_PARM_DESC(sample_rate, "Sample rate (between 2 and HZ)");
+
+module_param(bttv_id, int, 0444);
+MODULE_PARM_DESC(bttv_id, "BTTV card type");
+
+module_param(debug, bool, 0644);
+MODULE_PARM_DESC(debug, "Enable debugging messages");
+
+EXPORT_NO_SYMBOLS;
+
+#endif /* MODULE */
 
 /*
  * Overrides for Emacs so that we follow Linus's tabbing style.
