@@ -63,6 +63,8 @@
 #include <linux/timer.h>
 
 #include "drivers/lirc.h"
+#include "drivers/lirc_dev/lirc_dev.h"
+#include "drivers/kcompat.h"
 
 #include "lirc_it87.h"
 
@@ -79,13 +81,19 @@ static unsigned char it87_RXEN_mask = IT87_CIR_RCR_RXEN;
 /* must be longer than TIME_CONST */
 #define IT87_TIMEOUT	(HZ*5/100)
 
-static int major = LIRC_MAJOR;
+/* insmod parameters */
+static int debug = 0;
+#define dprintk(fmt, args...)                                     \
+	do{                                                       \
+		if(debug) printk(KERN_DEBUG LIRC_DRIVER_NAME ": " \
+                                 fmt, ## args);                   \
+	}while(0)
 
 static int io = IT87_CIR_DEFAULT_IOBASE;
 static int irq = IT87_CIR_DEFAULT_IRQ;
 static unsigned char it87_freq = 38; /* kHz */
 /* receiver demodulator default: off */
-static unsigned char it87_enable_demodulator = 0;
+static int it87_enable_demodulator = 0;
 
 static spinlock_t timer_lock = SPIN_LOCK_UNLOCKED;
 static struct timer_list timerlist;
@@ -131,9 +139,8 @@ static int init_chrdev(void);
 static void drop_chrdev(void);
 #endif
 	/* Hardware */
-static void it87_interrupt(int irq,
-			   void * dev_id,
-			   struct pt_regs * regs);
+static irqreturn_t it87_interrupt(int irq, void * dev_id,
+				 struct pt_regs * regs);
 static void send_space(unsigned long len);
 static void send_pulse(unsigned long len);
 static void init_send(void);
@@ -303,10 +310,8 @@ static int lirc_ioctl(struct inode *node,
 			      (it87_freq - IT87_CIR_FREQ_MIN) << 3),
 			     io + IT87_CIR_TCR2);
 			spin_unlock_irqrestore(&hardware_lock, hw_flags);
-#ifdef DEBUG
-			printk(KERN_DEBUG LIRC_DRIVER_NAME 
-			       " demodulation frequency: %d kHz\n", it87_freq);
-#endif
+			dprintk("demodulation frequency: %d kHz\n",
+				it87_freq);
 		}
 
 		break;
@@ -334,11 +339,7 @@ static void add_read_queue(int flag,
 	unsigned int new_rx_tail;
 	lirc_t newval;
 
-#ifdef DEBUG_SIGNAL
-	printk(KERN_DEBUG LIRC_DRIVER_NAME
-	       ": add flag %d with val %lu\n",
-	       flag,val);
-#endif
+	dprintk("add flag %d with val %lu\n", flag,val);
 	
 	newval = val & PULSE_MASK;
 
@@ -358,9 +359,7 @@ static void add_read_queue(int flag,
 	}
 	new_rx_tail = (rx_tail + 1) & (RBUF_LEN - 1);
 	if (new_rx_tail == rx_head) {
-#ifdef DEBUG
-		printk(KERN_WARNING LIRC_DRIVER_NAME ": Buffer overrun.\n");
-#endif
+		dprintk("Buffer overrun.\n");
 		return;
 	}
 	rx_buf[rx_tail] = newval;
@@ -378,17 +377,42 @@ static struct file_operations lirc_fops = {
 	release: lirc_close,
 };
 
-#ifdef MODULE
-int init_chrdev(void)
+static int set_use_inc(void* data)
 {
-	int retval;
+#if WE_DONT_USE_LOCAL_OPEN_CLOSE
+	MOD_INC_USE_COUNT;
+#endif
+       return 0;
+}
+
+static void set_use_dec(void* data)
+{
+#if WE_DONT_USE_LOCAL_OPEN_CLOSE
+	MOD_DEC_USE_COUNT;
+#endif
+}
+static struct lirc_plugin plugin = {
+       name:           LIRC_DRIVER_NAME,
+       minor:          -1,
+       code_length:    1,
+       sample_rate:    0,
+       data:           NULL,
+       add_to_buf:     NULL,
+       get_queue:      NULL,
+       set_use_inc:    set_use_inc,
+       set_use_dec:    set_use_dec,
+       fops:           &lirc_fops,
+};
+
+
+#ifdef MODULE
+static int init_chrdev(void)
+{
+	plugin.minor = lirc_register_plugin(&plugin);
 	
-	retval = register_chrdev(major,
-				 LIRC_DRIVER_NAME,
-				 &lirc_fops);
-	if (retval < 0) {
+	if (plugin.minor < 0) {
 		printk(KERN_ERR LIRC_DRIVER_NAME ": init_chrdev() failed.\n");
-		return retval;
+		return -EIO;
 	}
 	return 0;
 }
@@ -396,8 +420,7 @@ int init_chrdev(void)
 
 static void drop_chrdev(void)
 {
-	unregister_chrdev(major,
-			  LIRC_DRIVER_NAME);
+	lirc_unregister_plugin(plugin.minor);
 }
 #endif
 
@@ -434,12 +457,8 @@ static void it87_timeout(unsigned long data)
 	if (last_value) {
 		/* determine 'virtual' pulse end: */
 	 	pulse_end = delta(&last_tv, &last_intr_tv);
-#ifdef DEBUG_SIGNAL
-		printk(KERN_DEBUG LIRC_DRIVER_NAME
-		       ": timeout add %d for %lu usec\n",
-		       last_value,
-		       pulse_end);
-#endif
+		dprintk("timeout add %d for %lu usec\n", 
+			last_value, pulse_end);
 		add_read_queue(last_value,
 			       pulse_end);
 		last_value = 0;
@@ -449,9 +468,9 @@ static void it87_timeout(unsigned long data)
 }
 
 
-static void it87_interrupt(int irq,
-			   void * dev_id,
-			   struct pt_regs * regs)
+static irqreturn_t it87_interrupt(int irq,
+				 void * dev_id,
+				 struct pt_regs * regs)
 {
 	unsigned char data;
 	struct timeval curr_tv;
@@ -469,10 +488,7 @@ static void it87_interrupt(int irq,
 		lsr = inb(io + IT87_CIR_RSR) & (IT87_CIR_RSR_RXFTO |
 						    IT87_CIR_RSR_RXFBC);
 		fifo = lsr & IT87_CIR_RSR_RXFBC;
-#ifdef DEBUG_SIGNAL
-		printk(KERN_DEBUG LIRC_DRIVER_NAME
-		       "iir: 0x%x fifo: 0x%x\n", iir, lsr);
-#endif
+		dprintk("iir: 0x%x fifo: 0x%x\n", iir, lsr);
 	
 		/* avoid interference with timer */
 		spin_lock_irqsave(&timer_lock, flags);
@@ -480,27 +496,20 @@ static void it87_interrupt(int irq,
 		do {
 			del_timer(&timerlist);
 			data = inb(io + IT87_CIR_DR);
-#ifdef DEBUG_SIGNAL
-			printk(KERN_DEBUG LIRC_DRIVER_NAME
-			       ": data=%.2x\n",
-			       data);
-#endif
+
+			dprintk("data=%.2x\n", data);
 			do_gettimeofday(&curr_tv);
 			deltv = delta(&last_tv, &curr_tv);
 			deltintrtv = delta(&last_intr_tv, &curr_tv);
-#ifdef DEBUG_SIGNAL
-			printk(KERN_DEBUG LIRC_DRIVER_NAME
-			       ": t %lu , d %d\n",
-			       deltintrtv,
-			       (int)data);
-#endif
+
+			dprintk("t %lu , d %d\n", deltintrtv, (int)data);
+			
 			/* if nothing came in last 2 cycles,
 			   it was gap */
 			if (deltintrtv > TIME_CONST * 2) {
 				if (last_value) {
-#ifdef DEBUG_SIGNAL
-					printk(KERN_DEBUG LIRC_DRIVER_NAME ": GAP\n");
-#endif
+					dprintk("GAP\n");
+
 					/* simulate signal change */
 					add_read_queue(last_value,
 						       deltv-
@@ -547,15 +556,13 @@ static void it87_interrupt(int irq,
 		while (fifo != 0);
 		spin_unlock_irqrestore(&hardware_lock, hw_flags);
 		spin_unlock_irqrestore(&timer_lock, flags);
-		break;
+		
+		return IRQ_RETVAL(IRQ_HANDLED);
 
 	default:
 		/* not our irq */
-#ifdef DEBUG_SIGNAL
-		printk(KERN_DEBUG LIRC_DRIVER_NAME
-		       "unknown IRQ (shouldn't happen) !!\n");
-#endif
-		break;
+		dprintk("unknown IRQ (shouldn't happen) !!\n");
+		return IRQ_RETVAL(IRQ_NONE);
 	}
 }
 
@@ -569,12 +576,8 @@ static void send_it87(unsigned long len,
 	long time_left = 0;
 	static unsigned char byte_out = 0;
 
-#ifdef DEBUG_SIGNAL
-	printk(KERN_DEBUG LIRC_DRIVER_NAME
-	       "send_it87: len=%ld, sb=%d\n",
-	       len,
-	       send_byte);
-#endif
+	dprintk("%s: len=%ld, sb=%d\n", __FUNCTION__, len, send_byte);
+
 	time_left = (long)len - (long)count * (long)stime;
 	count += ((2 * time_left) / stime);
 	while (count) {
@@ -584,13 +587,11 @@ static void send_it87(unsigned long len,
 			it87_bits_in_byte_out++;
 		}
 		if (it87_bits_in_byte_out == 8) {
-#ifdef DEBUG_SIGNAL
-			printk(KERN_DEBUG LIRC_DRIVER_NAME
-			       "out=0x%x, tsr_txfbc: 0x%x\n",
-			       byte_out,
-			       inb(io + IT87_CIR_TSR) &
-			       IT87_CIR_TSR_TXFBC);
-#endif
+			dprintk("out=0x%x, tsr_txfbc: 0x%x\n",
+				byte_out,
+				inb(io + IT87_CIR_TSR) &
+				IT87_CIR_TSR_TXFBC);
+
 			while ((inb(io + IT87_CIR_TSR) &
 				IT87_CIR_TSR_TXFBC) >= IT87_CIR_FIFO_SIZE);
 			{
@@ -874,7 +875,7 @@ static void drop_port(void)
 }
 
 
-int init_lirc_it87(void)
+static int init_lirc_it87(void)
 {
 	int retval;
 	
@@ -891,24 +892,6 @@ int init_lirc_it87(void)
 
 #ifdef MODULE
 
-MODULE_AUTHOR("Hans-Gnter Ltke Uphues");
-MODULE_DESCRIPTION("LIRC driver for ITE IT8712/IT8705 CIR port");
-MODULE_PARM(io, "i");
-MODULE_PARM_DESC(io,
-		 "I/O base address (default: 0x310)");
-MODULE_PARM(irq, "i");
-MODULE_PARM_DESC(irq,
-		 "Interrupt (1,3-12) (default: 7)");
-MODULE_PARM(it87_enable_demodulator, "i");
-MODULE_PARM_DESC(it87_enable_demodulator,
-		 "Receiver demodulator enable/disable (1/0), default: 0");
-#ifdef MODULE_LICENSE
-MODULE_LICENSE("GPL");
-#endif
-
-EXPORT_NO_SYMBOLS;
-
-
 int init_module(void)
 {
 	int retval;
@@ -932,7 +915,27 @@ void cleanup_module(void)
 	drop_port();
 	printk(KERN_INFO LIRC_DRIVER_NAME ": Uninstalled.\n");
 }
-#endif
+
+MODULE_DESCRIPTION("LIRC driver for ITE IT8712/IT8705 CIR port");
+MODULE_AUTHOR("Hans-Gnter Ltke Uphues");
+MODULE_LICENSE("GPL");
+
+module_param(io, int, 0444);
+MODULE_PARM_DESC(io, "I/O base address (default: 0x310)");
+
+module_param(irq, int, 0444);
+MODULE_PARM_DESC(irq, "Interrupt (1,3-12) (default: 7)");
+
+module_param(it87_enable_demodulator, bool, 0444);
+MODULE_PARM_DESC(it87_enable_demodulator, 
+		 "Receiver demodulator enable/disable (1/0), default: 0");
+
+module_param(debug, bool, 0644);
+MODULE_PARM_DESC(debug, "Enable debugging messages");
+
+EXPORT_NO_SYMBOLS;
+
+#endif /* MODULE */
 
 
 /*
