@@ -1,4 +1,4 @@
-/*      $Id: lirc_parallel.c,v 5.14 2002/01/06 13:33:30 lirc Exp $      */
+/*      $Id: lirc_parallel.c,v 5.27 2004/11/07 13:31:18 lirc Exp $      */
 
 /****************************************************************************
  ** lirc_parallel.c *********************************************************
@@ -34,15 +34,13 @@
 #endif
 
 #include <linux/version.h>
-#if LINUX_VERSION_CODE >= 0x020100 && LINUX_VERSION_CODE < 0x020200
-#error "--- Sorry, 2.1.x kernels are not supported. ---"
-#elif LINUX_VERSION_CODE >= 0x020200
-#define KERNEL_2_2
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 18)
+#error "**********************************************************"
+#error " Sorry, this driver needs kernel version 2.2.18 or higher "
+#error "**********************************************************"
+#endif
+
 #include <linux/config.h>
-#endif
-#if LINUX_VERSION_CODE >= 0x020400
-#define KERNEL_2_4
-#endif
 
 #ifdef CONFIG_SMP
 #error "--- Sorry, this driver is not SMP safe. ---"
@@ -63,14 +61,17 @@
 #include <asm/io.h>
 #include <asm/signal.h>
 #include <asm/irq.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
+#include <asm/div64.h>
+#endif
 
-#ifdef KERNEL_2_2
 #include <asm/uaccess.h>
 #include <linux/poll.h>
 #include <linux/parport.h>
-#endif
 
 #include "drivers/lirc.h"
+#include "drivers/kcompat.h"
+#include "drivers/lirc_dev/lirc_dev.h"
 
 #include "lirc_parallel.h"
 
@@ -80,6 +81,7 @@
  *************************   Globale Variablen   ***********************
  ***********************************************************************/
 
+static int debug = 0;
 unsigned int irq = LIRC_IRQ;
 unsigned int io = LIRC_PORT;
 #ifdef LIRC_TIMER
@@ -93,29 +95,22 @@ unsigned int default_timer = LIRC_TIMER;
 static lirc_t wbuf[WBUF_SIZE];
 static lirc_t rbuf[RBUF_SIZE];
 
-#ifdef KERNEL_2_4
 DECLARE_WAIT_QUEUE_HEAD(lirc_wait);
-#else
-static struct wait_queue *lirc_wait=NULL;
-#endif
 
 unsigned int rptr=0,wptr=0;
 unsigned int lost_irqs=0;
 int is_open=0;
 
-#ifdef KERNEL_2_2
 struct parport *pport;
 struct pardevice *ppdevice;
 int is_claimed=0;
-#endif
 
 /***********************************************************************
  *************************   Interne Funktionen  ***********************
  ***********************************************************************/
 
-unsigned int __inline__ in(int offset)
+static unsigned int __inline__ in(int offset)
 {
-#ifdef KERNEL_2_2
 	switch(offset)
 	{
 	case LIRC_LP_BASE:
@@ -126,14 +121,10 @@ unsigned int __inline__ in(int offset)
 		return(parport_read_control(pport));
 	}
 	return(0); /* make compiler happy */
-#else
-	return(inb(io+offset));
-#endif
 }
 
-void __inline__ out(int offset, int value)
+static void __inline__ out(int offset, int value)
 {
-#ifdef KERNEL_2_2
 	switch(offset)
 	{
 	case LIRC_LP_BASE:
@@ -147,32 +138,29 @@ void __inline__ out(int offset, int value)
 		       LIRC_DRIVER_NAME);
 		break;
 	}
-#else
-	outb(value,io+offset);
-#endif
 }
 
-unsigned int __inline__ lirc_get_timer(void)
+static unsigned int __inline__ lirc_get_timer(void)
 {
 	return(in(LIRC_PORT_TIMER)&LIRC_PORT_TIMER_BIT);
 }
 
-unsigned int __inline__  lirc_get_signal(void)
+static unsigned int __inline__  lirc_get_signal(void)
 {
 	return(in(LIRC_PORT_SIGNAL)&LIRC_PORT_SIGNAL_BIT);
 }
 
-void __inline__ lirc_on(void)
+static void __inline__ lirc_on(void)
 {
 	out(LIRC_PORT_DATA,LIRC_PORT_DATA_BIT);
 }
 
-void __inline__ lirc_off(void)
+static void __inline__ lirc_off(void)
 {
 	out(LIRC_PORT_DATA,0);
 }
 
-unsigned int init_lirc_timer(void)
+static unsigned int init_lirc_timer(void)
 {
 	struct timeval tv,now;
 	unsigned int level,newlevel,timeelapsed,newtimer;
@@ -190,7 +178,7 @@ unsigned int init_lirc_timer(void)
 	}
 	while(count<1000 && (now.tv_sec<tv.tv_sec 
 			     || (now.tv_sec==tv.tv_sec 
-				 && now.tv_usec<tv.tv_sec)));
+				 && now.tv_usec<tv.tv_usec)));
 
 	timeelapsed=((now.tv_sec+1-tv.tv_sec)*1000000
 		     +(now.tv_usec-tv.tv_usec));
@@ -231,8 +219,7 @@ unsigned int init_lirc_timer(void)
 	}
 }
 
-#ifdef KERNEL_2_2
-int lirc_claim(void)
+static int lirc_claim(void)
 {
 	if(parport_claim(ppdevice)!=0)
 	{
@@ -251,7 +238,6 @@ int lirc_claim(void)
 	is_claimed=1;
 	return(1);
 }
-#endif
 
 /***********************************************************************
  *************************   interrupt handler  ************************
@@ -272,7 +258,7 @@ static inline void rbuf_write(lirc_t signal)
 	wptr=nwptr;
 }
 
-void irq_handler(int i,void *blah,struct pt_regs * regs)
+static void irq_handler(int i,void *blah,struct pt_regs * regs)
 {
 	struct timeval tv;
 	static struct timeval lasttv;
@@ -285,12 +271,10 @@ void irq_handler(int i,void *blah,struct pt_regs * regs)
 	if(!MOD_IN_USE)
 		return;
 
-#ifdef KERNEL_2_2
 	if(!is_claimed)
 	{
 		return;
 	}
-#endif
 
 	/* disable interrupt */
 	/*
@@ -355,7 +339,15 @@ void irq_handler(int i,void *blah,struct pt_regs * regs)
 	if(signal!=0)
 	{
 		/* ajust value to usecs */
-		signal=(long) (((unsigned long long) signal)*1000000)/timer;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
+		unsigned long long helper;
+		
+		helper = ((unsigned long long) signal)*1000000;
+		do_div(helper, timer);
+		signal = (long) helper;
+#else
+		signal=(long) ((((double) signal)*1000000)/timer);
+#endif
 
 		if(signal>LIRC_SFH506_DELAY)
 		{
@@ -385,32 +377,16 @@ void irq_handler(int i,void *blah,struct pt_regs * regs)
  **************************   file_operations   ************************
  ***********************************************************************/
 
-#ifdef KERNEL_2_2
 static loff_t lirc_lseek(struct file *filep,loff_t offset,int orig)
 {
 	return(-ESPIPE);
 }
-#else
-static int lirc_lseek(struct inode *inode,struct file *filep,off_t offset,int orig)
-{
-	return(-ESPIPE);
-}
-#endif
 
-#ifdef KERNEL_2_2
 static ssize_t lirc_read(struct file *filep,char *buf,size_t n,loff_t *ppos)
 {
-#else
-static int lirc_read(struct inode *node,struct file *filep,char *buf,int n)
-{
-#endif
 	int result;
 	int count=0;
-#ifdef KERNEL_2_4
 	DECLARE_WAITQUEUE(wait, current);
-#else
-	struct wait_queue wait={current,NULL};
-#endif
 	
 	if(n%sizeof(lirc_t)) return(-EINVAL);
 	
@@ -423,13 +399,8 @@ static int lirc_read(struct inode *node,struct file *filep,char *buf,int n)
 	{
 		if(rptr!=wptr)
 		{
-#ifdef KERNEL_2_2
 			copy_to_user(buf+count,(char *) &rbuf[rptr],
 				     sizeof(lirc_t));
-#else
-			memcpy_tofs(buf+count,(char *) &rbuf[rptr],
-				    sizeof(lirc_t));
-#endif
 			rptr=(rptr+1)&(RBUF_SIZE-1);
 			count+=sizeof(lirc_t);
 		}
@@ -440,19 +411,11 @@ static int lirc_read(struct inode *node,struct file *filep,char *buf,int n)
 				result=-EAGAIN;
 				break;
 			}
-#ifdef KERNEL_2_2
 			if (signal_pending(current))
 			{
 				result=-ERESTARTSYS;
 				break;
 			}
-#else
-			if(current->signal & ~current->blocked)
-			{
-				result=-EINTR;
-				break;
-			}
-#endif
 			schedule();
 			current->state=TASK_INTERRUPTIBLE;
 		}
@@ -462,13 +425,8 @@ static int lirc_read(struct inode *node,struct file *filep,char *buf,int n)
 	return(count ? count:result);
 }
 
-#ifdef KERNEL_2_2
 static ssize_t lirc_write(struct file *filep,const char *buf,size_t n,
 			  loff_t *ppos)
-#else
-static int lirc_write(struct inode *node,struct file *filep,const char *buf,
-		      int n)
-#endif
 {
 	int result,count;
 	unsigned int i;
@@ -476,12 +434,10 @@ static int lirc_write(struct inode *node,struct file *filep,const char *buf,
 	unsigned long flags;
 	lirc_t counttimer;
 	
-#ifdef KERNEL_2_2
 	if(!is_claimed)
 	{
 		return(-EBUSY);
 	}
-#endif
 	if(n%sizeof(lirc_t)) return(-EINVAL);
 	result=verify_area(VERIFY_READ,buf,n);
 	if(result) return(result);
@@ -490,15 +446,7 @@ static int lirc_write(struct inode *node,struct file *filep,const char *buf,
 	
 	if(count>WBUF_SIZE || count%2==0) return(-EINVAL);
 	
-#ifdef KERNEL_2_2
 	copy_from_user(wbuf,buf,n);
-#else
-	memcpy_fromfs(wbuf,buf,n);
-
-	/*
-	  if(check_region(io,LIRC_PORT_LEN)!=0) return(-EBUSY);
-	*/
-#endif
 	
 #ifdef LIRC_TIMER
 	if(timer==0) /* try again if device is ready */
@@ -510,10 +458,18 @@ static int lirc_write(struct inode *node,struct file *filep,const char *buf,
 	/* ajust values from usecs */
 	for(i=0;i<count;i++)
 	{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
+		unsigned long long helper;
+		
+		helper = ((unsigned long long) wbuf[i])*timer;
+		do_div(helper, 1000000);
+		wbuf[i] = (lirc_t) helper;
+#else
 		wbuf[i]=(lirc_t) (((double) wbuf[i])*timer/1000000);
+#endif
 	}
 	
-	save_flags(flags);cli();
+	local_irq_save(flags);
 	i=0;
 	while(i<count)
 	{
@@ -528,7 +484,7 @@ static int lirc_write(struct inode *node,struct file *filep,const char *buf,
 			if(in(1)&LP_PSELECD)
 			{
 				lirc_off();
-				restore_flags(flags); /* sti(); */
+				local_irq_restore(flags);
 				return(-EIO);
 			}
 		}
@@ -544,13 +500,13 @@ static int lirc_write(struct inode *node,struct file *filep,const char *buf,
 			level=newlevel;
 			if(in(1)&LP_PSELECD)
 			{
-				restore_flags(flags); /* sti(); */
+				local_irq_restore(flags);
 				return(-EIO);
 			}
 		}
 		while(counttimer<wbuf[i]);i++;
 	}
-	restore_flags(flags); /* sti(); */
+	local_irq_restore(flags);
 #else
 	/* 
 	   place code that handles write
@@ -560,7 +516,6 @@ static int lirc_write(struct inode *node,struct file *filep,const char *buf,
 	return(n);
 }
 
-#ifdef KERNEL_2_2
 static unsigned int lirc_poll(struct file *file, poll_table * wait)
 {
 	poll_wait(file, &lirc_wait,wait);
@@ -568,18 +523,6 @@ static unsigned int lirc_poll(struct file *file, poll_table * wait)
 		return(POLLIN|POLLRDNORM);
 	return(0);
 }
-#else
-static int lirc_select(struct inode *node,struct file *file,int sel_type,
-		       select_table *wait)
-{
-	if(sel_type!=SEL_IN)
-		return 0;
-	if(rptr!=wptr)
-		return(1);
-	select_wait(&lirc_wait,wait);
-	return(0);
-}
-#endif
 
 static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 		      unsigned long arg)
@@ -590,60 +533,25 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 	switch(cmd)
 	{
 	case LIRC_GET_FEATURES:
-#               ifdef KERNEL_2_2
 		result=put_user(features,(unsigned long *) arg);
 		if(result) return(result); 
-#               else
-		result=verify_area(VERIFY_WRITE,(unsigned long*) arg,
-				   sizeof(unsigned long));
-		if(result) return(result);
-		put_user(features,(unsigned long *) arg);
-#               endif
 		break;
 	case LIRC_GET_SEND_MODE:
-#               ifdef KERNEL_2_2
 		result=put_user(LIRC_MODE_PULSE,(unsigned long *) arg);
 		if(result) return(result); 
-#               else
-		result=verify_area(VERIFY_WRITE,(unsigned long *) arg,
-				   sizeof(unsigned long));
-		if(result) return(result);
-		put_user(LIRC_MODE_PULSE,(unsigned long *) arg);
-#               endif
 		break;
 	case LIRC_GET_REC_MODE:
-#               ifdef KERNEL_2_2
 		result=put_user(LIRC_MODE_MODE2,(unsigned long *) arg);
 		if(result) return(result); 
-#               else
-		result=verify_area(VERIFY_WRITE,(unsigned long *) arg,
-				   sizeof(unsigned long));
-		if(result) return(result);
-		put_user(LIRC_MODE_MODE2,(unsigned long *) arg);
-#               endif
 		break;
 	case LIRC_SET_SEND_MODE:
-#               ifdef KERNEL_2_2
 		result=get_user(mode,(unsigned long *) arg);
 		if(result) return(result);
-#               else
-		result=verify_area(VERIFY_READ,(unsigned long *) arg,
-				   sizeof(unsigned long));
-		if(result) return(result);
-		mode=get_user((unsigned long *) arg);
-#               endif
 		if(mode!=LIRC_MODE_PULSE) return(-EINVAL);
 		break;
 	case LIRC_SET_REC_MODE:
-#               ifdef KERNEL_2_2
 		result=get_user(mode,(unsigned long *) arg);
 		if(result) return(result);
-#               else
-		result=verify_area(VERIFY_READ,(unsigned long *) arg,
-				   sizeof(unsigned long));
-		if(result) return(result);
-		mode=get_user((unsigned long *) arg);
-#               endif
 		if(mode!=LIRC_MODE_MODE2) return(-ENOSYS);
 		break;
 	default:
@@ -654,36 +562,15 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 
 static int lirc_open(struct inode* node,struct file* filep)
 {
-#ifndef KERNEL_2_2
-	int result;
-#endif
-	
 	if(MOD_IN_USE)
 	{
 		return(-EBUSY);
 	}
-#ifdef KERNEL_2_2
 	if(!lirc_claim())
 	{
 		return(-EBUSY);
 	}
 	pport->ops->enable_irq(pport);
-#else
-	result=request_irq(irq,irq_handler,SA_INTERRUPT,
-			   LIRC_DRIVER_NAME,NULL);
-	switch(result)
-	{
-	case -EBUSY:
-		printk(KERN_NOTICE "%s: IRQ %u busy\n",LIRC_DRIVER_NAME,irq);
-		return(-EBUSY);
-	case -EINVAL:
-		printk(KERN_NOTICE "%s: bad irq number or handler\n",
-		       LIRC_DRIVER_NAME);
-		return(-EINVAL);
-	}
-	/* enable interrupt */
-	out(LIRC_PORT_IRQ,LP_PINITP|LP_PSELECP|LP_PINTEN);
-#endif
 
 	/* init read ptr */
 	rptr=wptr=0;
@@ -694,74 +581,61 @@ static int lirc_open(struct inode* node,struct file* filep)
 	return(0);
 }
 
-#ifdef KERNEL_2_2
 static int lirc_close(struct inode* node,struct file* filep)
-#else
-static void lirc_close(struct inode* node,struct file* filep)
-#endif
 {
-#ifdef KERNEL_2_2
 	if(is_claimed)
 	{
 		is_claimed=0;
 		parport_release(ppdevice);
 	}
-#else
-	free_irq(irq,NULL);
-	
-	/* disable interrupt */
-	/*
-	  FIXME
-	  out(LIRC_PORT_IRQ,LP_PINITP|LP_PSELECP);
-	*/
-#endif
 	is_open=0;
 	MOD_DEC_USE_COUNT;
-#ifdef KERNEL_2_2
 	return(0);
-#endif
 }
 
 static struct file_operations lirc_fops = 
 {
-#ifdef KERNEL_2_2
 	llseek:  lirc_lseek,
-#else
-	lseek:   lirc_lseek,
-#endif
 	read:    lirc_read,
 	write:   lirc_write,
-#ifdef KERNEL_2_2
 	poll:    lirc_poll,
-#else
-	select:  lirc_select,
-#endif
 	ioctl:   lirc_ioctl,
 	open:    lirc_open,
 	release: lirc_close
 };
 
+static int set_use_inc(void* data)
+{
+#if WE_DONT_USE_LOCAL_OPEN_CLOSE
+	MOD_INC_USE_COUNT;
+#endif
+	return 0;
+}
+
+static void set_use_dec(void* data)
+{
+#if WE_DONT_USE_LOCAL_OPEN_CLOSE
+	MOD_DEC_USE_COUNT;
+#endif
+}
+
+static struct lirc_plugin plugin = {
+       name:           LIRC_DRIVER_NAME,
+       minor:          -1,
+       code_length:    1,
+       sample_rate:    0,
+       data:           NULL,
+       add_to_buf:     NULL,
+       get_queue:      NULL,
+       set_use_inc:    set_use_inc,
+       set_use_dec:    set_use_dec,
+       fops:           &lirc_fops,
+};
+
 #ifdef MODULE
 
-#if LINUX_VERSION_CODE >= 0x020200
-MODULE_AUTHOR("Christoph Bartelmus");
-MODULE_DESCRIPTION("Infrared receiver driver for parallel ports.");
-#ifdef MODULE_LICENSE
-MODULE_LICENSE("GPL");
-#endif
-
-MODULE_PARM(io, "i");
-MODULE_PARM_DESC(io, "I/O address base (0x3bc, 0x378 or 0x278)");
-
-MODULE_PARM(irq, "i");
-MODULE_PARM_DESC(irq, "Interrupt (7 or 5)");
-
-EXPORT_NO_SYMBOLS;
-#endif
-
-#ifdef KERNEL_2_2
-int pf(void *handle);
-void kf(void *handle);
+static int pf(void *handle);
+static void kf(void *handle);
 
 static struct timer_list poll_timer;
 static void poll_state(unsigned long ignored);
@@ -786,15 +660,14 @@ static void poll_state(unsigned long ignored)
 	}
 }
 
-int pf(void *handle)
+static int pf(void *handle)
 {
 	pport->ops->disable_irq(pport);
 	is_claimed=0;
 	return(0);
 }
 
-
-void kf(void *handle)
+static void kf(void *handle)
 {
 	if(!is_open)
 		return;
@@ -806,7 +679,6 @@ void kf(void *handle)
 	printk(KERN_INFO "%s: reclaimed port\n",LIRC_DRIVER_NAME);
 	*/
 }
-#endif
 
 /***********************************************************************
  ******************   init_module()/cleanup_module()  ******************
@@ -814,7 +686,9 @@ void kf(void *handle)
 
 int init_module(void)
 {
-#ifdef KERNEL_2_2
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 3)
+	pport=parport_find_base(io);
+#else
 	pport=parport_enumerate();
 	while(pport!=NULL)
 	{
@@ -824,6 +698,7 @@ int init_module(void)
 		}
 		pport=pport->next;
 	}
+#endif
 	if(pport==NULL)
 	{
 		printk(KERN_NOTICE "%s: no port at %x found\n",
@@ -832,6 +707,9 @@ int init_module(void)
 	}
 	ppdevice=parport_register_device(pport,LIRC_DRIVER_NAME,
 					 pf,kf,irq_handler,0,NULL);
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 3)
+	parport_put_port(pport);
+#endif
 	if(ppdevice==NULL)
 	{
 		printk(KERN_NOTICE "%s: parport_register_device() failed\n",
@@ -842,54 +720,38 @@ int init_module(void)
 		goto skip_init;
 	is_claimed=1;
 	out(LIRC_LP_CONTROL,LP_PSELECP|LP_PINITP);
-#else
-	if(check_region(io,LIRC_PORT_LEN)!=0)
-	{
-		printk(KERN_NOTICE "%s: port already in use\n",LIRC_DRIVER_NAME);
-		return(-EBUSY);
-	}
-	request_region(io,LIRC_PORT_LEN,LIRC_DRIVER_NAME);
-#endif
 
 #ifdef LIRC_TIMER
-#       ifdef DEBUG
-	out(LIRC_PORT_DATA,LIRC_PORT_DATA_BIT);
-#       endif
+	if(debug) 
+	{
+		out(LIRC_PORT_DATA,LIRC_PORT_DATA_BIT);
+	}
 	
 	timer=init_lirc_timer();
 
 #       if 0 	/* continue even if device is offline */
 	if(timer==0) 
 	{
-#       ifdef KERNEL_2_2
 		is_claimed=0;
 		parport_release(pport);
 		parport_unregister_device(ppdevice);
-#       else
-		release_region(io,LIRC_PORT_LEN);
-#       endif
 		return(-EIO);
 	}
 	
 #       endif
-#       ifdef DEBUG
-	out(LIRC_PORT_DATA,0);
-#       endif
+	if(debug)
+	{
+		out(LIRC_PORT_DATA,0);
+	}
 #endif 
 
-#ifdef KERNEL_2_2
 	is_claimed=0;
 	parport_release(ppdevice);
  skip_init:
-#endif
-	if(register_chrdev(LIRC_MAJOR, LIRC_DRIVER_NAME, &lirc_fops)<0)
+	if ((plugin.minor = lirc_register_plugin(&plugin)) < 0)
 	{
 		printk(KERN_NOTICE "%s: register_chrdev() failed\n",LIRC_DRIVER_NAME);
-#ifdef KERNEL_2_2
 		parport_unregister_device(ppdevice);
-#else
-		release_region(io,LIRC_PORT_LEN);
-#endif
 		return(-EIO);
 	}
 	printk(KERN_INFO "%s: installed using port 0x%04x irq %d\n",LIRC_DRIVER_NAME,io,irq);
@@ -899,11 +761,23 @@ int init_module(void)
 void cleanup_module(void)
 {
 	if(MOD_IN_USE) return;
-#ifdef KERNEL_2_2
 	parport_unregister_device(ppdevice);
-#else
-	release_region(io,LIRC_PORT_LEN);
-#endif
-	unregister_chrdev(LIRC_MAJOR,LIRC_DRIVER_NAME);
+	lirc_unregister_plugin(plugin.minor);
 }
-#endif
+
+MODULE_DESCRIPTION("Infrared receiver driver for parallel ports.");
+MODULE_AUTHOR("Christoph Bartelmus");
+MODULE_LICENSE("GPL");
+
+module_param(io, int, 0444);
+MODULE_PARM_DESC(io, "I/O address base (0x3bc, 0x378 or 0x278)");
+
+module_param(irq, int, 0444);
+MODULE_PARM_DESC(irq, "Interrupt (7 or 5)");
+
+module_param(debug, bool, 0644);
+MODULE_PARM_DESC(debug, "Enable debugging messages");
+
+EXPORT_NO_SYMBOLS;
+
+#endif /* MODULE */
