@@ -1,4 +1,4 @@
-/*      $Id: lirc_serial.c,v 5.39 2002/10/02 18:35:32 lirc Exp $      */
+/*      $Id: lirc_serial.c,v 5.56 2004/09/05 16:48:49 lirc Exp $      */
 
 /****************************************************************************
  ** lirc_serial.c ***********************************************************
@@ -55,15 +55,10 @@
 #endif
  
 #include <linux/version.h>
-#if LINUX_VERSION_CODE >= 0x020100
-#define KERNEL_2_1
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
-#define KERNEL_2_3
-#endif
-#endif
-
-#if LINUX_VERSION_CODE >= 0x020212
-#define LIRC_LOOPS_PER_JIFFY
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 18)
+#error "**********************************************************"
+#error " Sorry, this driver needs kernel version 2.2.18 or higher "
+#error "**********************************************************"
 #endif
 
 #include <linux/config.h>
@@ -93,9 +88,7 @@
 #include <linux/wait.h>
 #include <linux/mm.h>
 #include <linux/delay.h>
-#ifdef KERNEL_2_1
 #include <linux/poll.h>
-#endif
 
 #include <asm/system.h>
 #include <asm/segment.h>
@@ -104,16 +97,17 @@
 #include <asm/fcntl.h>
 
 #include "drivers/lirc.h"
+#include "drivers/kcompat.h"
+#include "drivers/lirc_dev/lirc_dev.h"
 
 #if defined(LIRC_SERIAL_SOFTCARRIER) && !defined(LIRC_SERIAL_TRANSMITTER)
 #warning "Software carrier only affects transmitting"
 #endif
 
-#if defined(rdtsc) && defined(KERNEL_2_1)
+#if defined(rdtsc)
 
 #define USE_RDTSC
 #warning "Note: using rdtsc instruction"
-
 #endif
 
 #ifdef LIRC_SERIAL_ANIMAX
@@ -125,6 +119,8 @@
 #endif
 #endif
 
+#define LIRC_DRIVER_NAME "lirc_serial"
+
 struct lirc_serial
 {
 	int type;
@@ -141,30 +137,41 @@ struct lirc_serial
 #define LIRC_IRDEO           1
 #define LIRC_IRDEO_REMOTE    2
 #define LIRC_ANIMAX          3
+#define LIRC_IGOR            4
 
 #ifdef LIRC_SERIAL_IRDEO
-int type=LIRC_IRDEO;
+static int type=LIRC_IRDEO;
 #elif defined(LIRC_SERIAL_IRDEO_REMOTE)
-int type=LIRC_IRDEO_REMOTE;
+static int type=LIRC_IRDEO_REMOTE;
 #elif defined(LIRC_SERIAL_ANIMAX)
-int type=LIRC_ANIMAX;
+static int type=LIRC_ANIMAX;
+#elif defined(LIRC_SERIAL_IGOR)
+static int type=LIRC_IGOR;
 #else
-int type=LIRC_HOMEBREW;
+static int type=LIRC_HOMEBREW;
 #endif
 
 #ifdef LIRC_SERIAL_SOFTCARRIER
-int softcarrier=1;
+static int softcarrier=1;
 #else
-int softcarrier=0;
+static int softcarrier=0;
 #endif
+static int debug = 0;
+
+#define dprintk(fmt, args...)                                   \
+	do{                                                     \
+		if(debug)                                       \
+	                printk(KERN_DEBUG LIRC_DRIVER_NAME ": " \
+                               fmt, ## args);                   \
+	}while(0)
 
 /* forward declarations */
-long send_pulse_irdeo(unsigned long length);
-long send_pulse_homebrew(unsigned long length);
-void send_space_irdeo(long length);
-void send_space_homebrew(long length);
+static long send_pulse_irdeo(unsigned long length);
+static long send_pulse_homebrew(unsigned long length);
+static void send_space_irdeo(long length);
+static void send_space_homebrew(long length);
 
-struct lirc_serial hardware[]=
+static struct lirc_serial hardware[]=
 {
 	/* home-brew receiver/transmitter */
 	{
@@ -222,11 +229,28 @@ struct lirc_serial hardware[]=
 		NULL,
 		NULL,
 		LIRC_CAN_REC_MODE2
+	},
+	
+	/* home-brew receiver/transmitter (Igor Cesko's variation) */
+	{
+		LIRC_HOMEBREW,
+		UART_MSR_DSR,
+		UART_MSR_DDSR,
+		UART_MCR_RTS|UART_MCR_OUT2|UART_MCR_DTR,
+		UART_MCR_RTS|UART_MCR_OUT2,
+		send_pulse_homebrew,
+		send_space_homebrew,
+		(
+#ifdef LIRC_SERIAL_TRANSMITTER
+		 LIRC_CAN_SET_SEND_DUTY_CYCLE|
+		 LIRC_CAN_SET_SEND_CARRIER|
+		 LIRC_CAN_SEND_PULSE|
+#endif
+		 LIRC_CAN_REC_MODE2)
 	}
+	
 };
 
-#define LIRC_DRIVER_NAME "lirc_serial"
-
 #define RS_ISR_PASS_LIMIT 256
 
 /* A long pulse code from a remote might take upto 300 bytes.  The
@@ -241,62 +265,26 @@ struct lirc_serial hardware[]=
 #define RBUF_LEN 256
 #define WBUF_LEN 256
 
-static int major = LIRC_MAJOR;
 static int sense = -1;   /* -1 = auto, 0 = active high, 1 = active low */
 
-#ifdef KERNEL_2_3
-static DECLARE_WAIT_QUEUE_HEAD(lirc_wait_in);
-#else
-static struct wait_queue *lirc_wait_in = NULL;
-#endif
-
-#ifdef KERNEL_2_1
 static spinlock_t lirc_lock = SPIN_LOCK_UNLOCKED;
-#endif
 
 static int io = LIRC_PORT;
 static int irq = LIRC_IRQ;
 
 static struct timeval lasttv = {0, 0};
 
-static lirc_t rbuf[RBUF_LEN];
-static int rbh, rbt;
+static struct lirc_buffer rbuf;
 
 static lirc_t wbuf[WBUF_LEN];
 
-unsigned int freq = 38000;
-unsigned int duty_cycle = 50;
+static unsigned int freq = 38000;
+static unsigned int duty_cycle = 50;
 
-#ifdef USE_RDTSC
-
-/* This version does sub-microsecond timing using rdtsc instruction,
- * and does away with the fudged LIRC_SERIAL_TRANSMITTER_LATENCY
- * Implicitly i586 architecture...  - Steve
- */
-
-/* When we use the rdtsc instruction to measure clocks, we keep the
- * pulse and space widths as clock cycles.  As this is CPU speed
- * dependent, the widths must be calculated in init_port and ioctl
- * time
- */
-
-unsigned long period = 0;
-unsigned long pulse_width = 0;
-unsigned long space_width = 0;
-
-/* So send_pulse can quickly convert microseconds to clocks */
-unsigned long conv_us_to_clocks = 0;
-
-#else /* USE_RDTSC */
-
-/* Version using udelay() */
-
-/* period, pulse/space width are kept with 8 binary places -
-   IE multiplied by 256. */
-
-unsigned long period = 6736; /* 256*1000000L/freq; */
-unsigned long pulse_width = 3368; /* (period*duty_cycle/100) */
-unsigned long space_width = 3368; /* (period - pulse_width) */
+/* Initialized in init_timing_params() */
+static unsigned long period = 0;
+static unsigned long pulse_width = 0;
+static unsigned long space_width = 0;
 
 #if defined(__i386__)
 /*
@@ -332,8 +320,6 @@ unsigned long space_width = 3368; /* (period - pulse_width) */
 
 #endif  /* __i386__ */
 
-#endif /* USE_RDTSC */
-
 static inline unsigned int sinp(int offset)
 {
 	return inb(io + offset);
@@ -344,12 +330,12 @@ static inline void soutp(int offset, int value)
 	outb(value, io + offset);
 }
 
-void on(void)
+static inline void on(void)
 {
 	soutp(UART_MCR,hardware[type].on);
 }
   
-void off(void)
+static inline void off(void)
 {
 	soutp(UART_MCR,hardware[type].off);
 }
@@ -371,20 +357,28 @@ static inline void safe_udelay(unsigned long usecs)
 }
 
 #ifdef USE_RDTSC
-
 /* This is an overflow/precision juggle, complicated in that we can't
    do long long divide in the kernel */
 
-void calc_pulse_lengths_in_clocks(void)
+/* When we use the rdtsc instruction to measure clocks, we keep the
+ * pulse and space widths as clock cycles.  As this is CPU speed
+ * dependent, the widths must be calculated in init_port and ioctl
+ * time
+ */
+
+/* So send_pulse can quickly convert microseconds to clocks */
+static unsigned long conv_us_to_clocks = 0;
+
+static inline int init_timing_params(unsigned int new_duty_cycle,
+		unsigned int new_freq)
 {
 	unsigned long long loops_per_sec,work;
+	
+	duty_cycle=new_duty_cycle;
+	freq=new_freq;
 
-#ifdef LIRC_LOOPS_PER_JIFFY
 	loops_per_sec=current_cpu_data.loops_per_jiffy;
 	loops_per_sec*=HZ;
-#else
-	loops_per_sec=current_cpu_data.loops_per_sec;
-#endif
 	
 	/* How many clocks in a microsecond?, avoiding long long divide */
 	work=loops_per_sec;
@@ -400,29 +394,38 @@ void calc_pulse_lengths_in_clocks(void)
 
 	pulse_width = period*duty_cycle/100;
 	space_width = period - pulse_width;
-#ifdef DEBUG
-#ifdef LIRC_LOOPS_PER_JIFFY
-	printk(KERN_INFO LIRC_DRIVER_NAME
-	       ": in calc_pulse_lengths_in_clocks, freq=%d, duty_cycle=%d, "
-	       "clk/jiffy=%ld, pulse=%ld, space=%ld, conv_us_to_clocks=%ld\n",
-	       freq, duty_cycle, current_cpu_data.loops_per_jiffy,
-	       pulse_width, space_width, conv_us_to_clocks);
-#else
-	printk(KERN_INFO LIRC_DRIVER_NAME
-	       ": in calc_pulse_lengths_in_clocks, freq=%d, duty_cycle=%d, "
-	       "clk/sec=%ld, pulse=%ld, space=%ld, conv_us_to_clocks=%ld\n",
-	       freq, duty_cycle, current_cpu_data.loops_per_sec,
-	       pulse_width, space_width, conv_us_to_clocks);
-#endif
-#endif
+	dprintk("in init_timing_params, freq=%d, duty_cycle=%d, "
+		"clk/jiffy=%ld, pulse=%ld, space=%ld, "
+		"conv_us_to_clocks=%ld\n",
+		freq, duty_cycle, current_cpu_data.loops_per_jiffy,
+		pulse_width, space_width, conv_us_to_clocks);
+	return 0;
 }
-
-#endif
+#else /* ! USE_RDTSC */
+static inline int init_timing_params(unsigned int new_duty_cycle,
+		unsigned int new_freq)
+{
+/* period, pulse/space width are kept with 8 binary places -
+ * IE multiplied by 256. */
+	if(256*1000000L/new_freq*new_duty_cycle/100<=
+	   LIRC_SERIAL_TRANSMITTER_LATENCY) return(-EINVAL);
+	if(256*1000000L/new_freq*(100-new_duty_cycle)/100<=
+	   LIRC_SERIAL_TRANSMITTER_LATENCY) return(-EINVAL);
+	duty_cycle=new_duty_cycle;
+	freq=new_freq;
+	period=256*1000000L/freq;
+	pulse_width=period*duty_cycle/100;
+	space_width=period-pulse_width;
+	dprintk("in init_timing_params, freq=%d pulse=%ld, "
+		"space=%ld\n", freq, pulse_width, space_width);
+	return 0;
+}
+#endif /* USE_RDTSC */
 
 
 /* return value: space length delta */
 
-long send_pulse_irdeo(unsigned long length)
+static long send_pulse_irdeo(unsigned long length)
 {
 	long rawbits;
 	int i;
@@ -463,88 +466,96 @@ long send_pulse_irdeo(unsigned long length)
 	}
 }
 
-long send_pulse_homebrew(unsigned long length)
-{
-#       ifdef USE_RDTSC
-	unsigned long target, start, now;
-#       else
-	unsigned long actual, target, d;
-#       endif
-	int flag;
+#ifdef USE_RDTSC
+/* Version that uses Pentium rdtsc instruction to measure clocks */
 
+/* This version does sub-microsecond timing using rdtsc instruction,
+ * and does away with the fudged LIRC_SERIAL_TRANSMITTER_LATENCY
+ * Implicitly i586 architecture...  - Steve
+ */
+
+static inline long send_pulse_homebrew_softcarrier(unsigned long length)
+{
+	int flag;
+	unsigned long target, start, now;
+
+	/* Get going quick as we can */
+	rdtscl(start);on();
+	/* Convert length from microseconds to clocks */
+	length*=conv_us_to_clocks;
+	/* And loop till time is up - flipping at right intervals */
+	now=start;
+	target=pulse_width;
+	flag=1;
+	while((now-start)<length)
+	{
+		/* Delay till flip time */
+		do
+		{
+			rdtscl(now);
+		}
+		while ((now-start)<target);
+		/* flip */
+		if(flag)
+		{
+			rdtscl(now);off();
+			target+=space_width;
+		}
+		else
+		{
+			rdtscl(now);on();
+			target+=pulse_width;
+		}
+		flag=!flag;
+	}
+	rdtscl(now);
+	return(((now-start)-length)/conv_us_to_clocks);
+}
+#else /* ! USE_RDTSC */
+/* Version using udelay() */
+
+/* here we use fixed point arithmetic, with 8
+   fractional bits.  that gets us within 0.1% or so of the right average
+   frequency, albeit with some jitter in pulse length - Steve */
+
+/* To match 8 fractional bits used for pulse/space length */
+
+static inline long send_pulse_homebrew_softcarrier(unsigned long length)
+{
+	int flag;
+	unsigned long actual, target, d;
+	length<<=8;
+
+	actual=target=0; flag=0;
+	while(actual<length)
+	{
+		if(flag)
+		{
+			off();
+			target+=space_width;
+		}
+		else
+		{
+			on();
+			target+=pulse_width;
+		}
+		d=(target-actual-LIRC_SERIAL_TRANSMITTER_LATENCY+128)>>8;
+		/* Note - we've checked in ioctl that the pulse/space
+		   widths are big enough so that d is > 0 */
+		udelay(d);
+		actual+=(d<<8)+LIRC_SERIAL_TRANSMITTER_LATENCY;
+		flag=!flag;
+	}
+	return((actual-length)>>8);
+}
+#endif /* USE_RDTSC */
+
+static long send_pulse_homebrew(unsigned long length)
+{
 	if(length<=0) return 0;
 	if(softcarrier)
 	{
-#               ifdef USE_RDTSC
-		
-		/* Version that uses Pentium rdtsc instruction to
-                   measure clocks */
-
-		/* Get going quick as we can */
-		rdtscl(start);on();
-		/* Convert length from microseconds to clocks */
-		length*=conv_us_to_clocks;
-		/* And loop till time is up - flipping at right intervals */
-		now=start;
-		target=pulse_width;
-		flag=1;
-		while((now-start)<length)
-		{
-			/* Delay till flip time */
-			do
-			{
-				rdtscl(now);
-			}
-			while ((now-start)<target);
-			/* flip */
-			if(flag)
-			{
-				rdtscl(now);off();
-				target+=space_width;
-			}
-			else
-			{
-				rdtscl(now);on();
-				target+=pulse_width;
-			}
-			flag=!flag;
-		}
-		rdtscl(now);
-		return(((now-start)-length)/conv_us_to_clocks);
-		
-#               else /* ! USE_RDTSC */
-		
-		/* here we use fixed point arithmetic, with 8
-		   fractional bits.  that gets us within 0.1% or so of
-		   the right average frequency, albeit with some
-		   jitter in pulse length - Steve */
-
-		/* To match 8 fractional bits used for pulse/space
-                   length */
-		length<<=8;
-	
-		actual=target=0; flag=0;
-		while(actual<length)
-		{
-			if(flag)
-			{
-				off();
-				target+=space_width;
-			}
-			else
-			{
-				on();
-				target+=pulse_width;
-			}
-			d=(target-actual-LIRC_SERIAL_TRANSMITTER_LATENCY+128)>>8;
-			/* Note - we've checked in ioctl that the pulse/space
-			   widths are big enough so that d is > 0 */
-			udelay(d);
-			actual+=(d<<8)+LIRC_SERIAL_TRANSMITTER_LATENCY;
-			flag=!flag;
-		}
-		return((actual-length)>>8);
-#               endif /* USE_RDTSC */
+		return send_pulse_homebrew_softcarrier(length);
 	}
 	else
 	{
@@ -554,13 +565,13 @@ long send_pulse_homebrew(unsigned long length)
 	}
 }
 
-void send_space_irdeo(long length)
+static void send_space_irdeo(long length)
 {
 	if(length<=0) return;
 	safe_udelay(length);
 }
 
-void send_space_homebrew(long length)
+static void send_space_homebrew(long length)
 {
         off();
 	if(length<=0) return;
@@ -569,18 +580,12 @@ void send_space_homebrew(long length)
 
 static void inline rbwrite(lirc_t l)
 {
-	unsigned int nrbt;
-
-	nrbt=(rbt+1) & (RBUF_LEN-1);
-	if(nrbt==rbh)      /* no new signals will be accepted */
+	if(lirc_buffer_full(&rbuf))    /* no new signals will be accepted */
 	{
-#               ifdef DEBUG
-		printk(KERN_WARNING  LIRC_DRIVER_NAME  ": Buffer overrun\n");
-#               endif
+		dprintk("Buffer overrun\n");
 		return;
 	}
-	rbuf[rbt]=l;
-	rbt=nrbt;
+	_lirc_buffer_write_1(&rbuf, (void *)&l);
 }
 
 static void inline frbwrite(lirc_t l)
@@ -632,7 +637,7 @@ static void inline frbwrite(lirc_t l)
 	rbwrite(l);
 }
 
-void irq_handler(int i, void *blah, struct pt_regs *regs)
+static irqreturn_t irq_handler(int i, void *blah, struct pt_regs *regs)
 {
 	struct timeval tv;
 	int status,counter,dcd;
@@ -682,13 +687,10 @@ void irq_handler(int i, void *blah, struct pt_regs *regs)
 			deltv=tv.tv_sec-lasttv.tv_sec;
 			if(deltv>15) 
 			{
-#ifdef DEBUG
-				printk(KERN_WARNING LIRC_DRIVER_NAME
-				       ": AIEEEE: %d %d %lx %lx %lx %lx\n",
-				       dcd,sense,
-				       tv.tv_sec,lasttv.tv_sec,
-				       tv.tv_usec,lasttv.tv_usec);
-#endif
+				dprintk("AIEEEE: %d %d %lx %lx %lx %lx\n",
+					dcd,sense,
+					tv.tv_sec,lasttv.tv_sec,
+					tv.tv_usec,lasttv.tv_usec);
 				data=PULSE_MASK; /* really long time */
 				if(!(dcd^sense)) /* sanity check */
 				{
@@ -719,24 +721,13 @@ void irq_handler(int i, void *blah, struct pt_regs *regs)
 			}
 			frbwrite(dcd^sense ? data : (data|PULSE_BIT));
 			lasttv=tv;
-			wake_up_interruptible(&lirc_wait_in);
+			wake_up_interruptible(&rbuf.wait_poll);
 		}
 	} while(!(sinp(UART_IIR) & UART_IIR_NO_INT)); /* still pending ? */
+	return IRQ_RETVAL(IRQ_HANDLED);
 }
 
-#ifdef KERNEL_2_3
 static DECLARE_WAIT_QUEUE_HEAD(power_supply_queue);
-#else
-static struct wait_queue *power_supply_queue = NULL;
-#endif
-#ifndef KERNEL_2_1
-static struct timer_list power_supply_timer;
-
-static void power_supply_up(unsigned long ignored)
-{
-        wake_up(&power_supply_queue);
-}
-#endif
 
 static int init_port(void)
 {
@@ -746,27 +737,21 @@ static int init_port(void)
 	
         if((check_region(io,8))==-EBUSY)
 	{
-#if 0
-		/* this is the correct behaviour but many people have
-                   the serial driver compiled into the kernel... */
 		printk(KERN_ERR  LIRC_DRIVER_NAME  
 		       ": port %04x already in use\n", io);
-		return(-EBUSY);
-#else
-		printk(KERN_ERR LIRC_DRIVER_NAME  
-		       ": port %04x already in use, proceeding anyway\n", io);
 		printk(KERN_WARNING LIRC_DRIVER_NAME  
-		       ": compile the serial port driver as module and\n");
+		       ": use 'setserial /dev/ttySX uart none'\n");
+		printk(KERN_WARNING LIRC_DRIVER_NAME  
+		       ": or compile the serial port driver as module and\n");
 		printk(KERN_WARNING LIRC_DRIVER_NAME  
 		       ": make sure this module is loaded first\n");
-		release_region(io,8);
-#endif
+		return(-EBUSY);
 	}
 	
 	/* Reserve io region. */
 	request_region(io, 8, LIRC_DRIVER_NAME);
 	
-	save_flags(flags);cli();
+	local_irq_save(flags);
 	
 	/* Set DLAB 0. */
 	soutp(UART_LCR, sinp(UART_LCR) & (~UART_LCR_DLAB));
@@ -810,29 +795,17 @@ static int init_port(void)
 		break;
 	}
 	
-	restore_flags(flags);
+	local_irq_restore(flags);
 	
-#       ifdef USE_RDTSC
 	/* Initialize pulse/space widths */
-	calc_pulse_lengths_in_clocks();
-#       endif
+	init_timing_params(duty_cycle, freq);
 
 	/* If pin is high, then this must be an active low receiver. */
 	if(sense==-1)
 	{
 		/* wait 1 sec for the power supply */
 		
-#               ifdef KERNEL_2_1
 		sleep_on_timeout(&power_supply_queue,HZ);
-#               else
-		init_timer(&power_supply_timer);
-		power_supply_timer.expires=jiffies+HZ;
-		power_supply_timer.data=(unsigned long) current;
-		power_supply_timer.function=power_supply_up;
-		add_timer(&power_supply_timer);
-		sleep_on(&power_supply_queue);
-		del_timer(&power_supply_timer);
-#               endif
 		
 		sense=(sinp(UART_MSR) & hardware[type].signal_pin) ? 1:0;
 		printk(KERN_INFO  LIRC_DRIVER_NAME  ": auto-detected active "
@@ -847,22 +820,22 @@ static int init_port(void)
 	return 0;
 }
 
-static int lirc_open(struct inode *ino, struct file *filep)
+static int set_use_inc(void* data)
 {
 	int result;
 	unsigned long flags;
 	
-#       ifdef KERNEL_2_1
 	spin_lock(&lirc_lock);
-#       endif
 	if(MOD_IN_USE)
 	{
-#               ifdef KERNEL_2_1
 		spin_unlock(&lirc_lock);
-#               endif
 		return -EBUSY;
 	}
 	
+	/* Init read buffer. */
+	if (lirc_buffer_init(&rbuf, sizeof(lirc_t), RBUF_LEN) < 0)
+		return -ENOMEM;
+	
 	/* initialize timestamp */
 	do_gettimeofday(&lasttv);
 	
@@ -871,53 +844,38 @@ static int lirc_open(struct inode *ino, struct file *filep)
 	{
 	case -EBUSY:
 		printk(KERN_ERR LIRC_DRIVER_NAME ": IRQ %d busy\n", irq);
-#               ifdef KERNEL_2_1
 		spin_unlock(&lirc_lock);
-#               endif
+                lirc_buffer_free(&rbuf);
 		return -EBUSY;
 	case -EINVAL:
 		printk(KERN_ERR LIRC_DRIVER_NAME
 		       ": Bad irq number or handler\n");
-#               ifdef KERNEL_2_1
 		spin_unlock(&lirc_lock);
-#               endif
+                lirc_buffer_free(&rbuf);
 		return -EINVAL;
 	default:
-#               ifdef DEBUG
-		printk(KERN_INFO LIRC_DRIVER_NAME
-		       ": Interrupt %d, port %04x obtained\n", irq, io);
-#               endif
+		dprintk("Interrupt %d, port %04x obtained\n", irq, io);
 		break;
 	};
 
-	/* finally enable interrupts. */
-	save_flags(flags);cli();
+	local_irq_save(flags);
 	
 	/* Set DLAB 0. */
 	soutp(UART_LCR, sinp(UART_LCR) & (~UART_LCR_DLAB));
 	
 	soutp(UART_IER, sinp(UART_IER)|UART_IER_MSI);
 	
-	restore_flags(flags);
-	
-	/* Init read buffer pointers. */
-	rbh = rbt = 0;
+	local_irq_restore(flags);
 	
 	MOD_INC_USE_COUNT;
-#       ifdef KERNEL_2_1
 	spin_unlock(&lirc_lock);
-#       endif
 	return 0;
 }
 
-#ifdef KERNEL_2_1
-static int lirc_close(struct inode *node, struct file *file)
-#else
-static void lirc_close(struct inode *node, struct file *file)
-#endif
+static void set_use_dec(void* data)
 {	unsigned long flags;
 	
-	save_flags(flags);cli();
+	local_irq_save(flags);
 	
 	/* Set DLAB 0. */
 	soutp(UART_LCR, sinp(UART_LCR) & (~UART_LCR_DLAB));
@@ -925,104 +883,17 @@ static void lirc_close(struct inode *node, struct file *file)
 	/* First of all, disable all interrupts */
 	soutp(UART_IER, sinp(UART_IER)&
 	      (~(UART_IER_MSI|UART_IER_RLSI|UART_IER_THRI|UART_IER_RDI)));
-	restore_flags(flags);
+	local_irq_restore(flags);
 	
 	free_irq(irq, NULL);
-#       ifdef DEBUG
-	printk(KERN_INFO  LIRC_DRIVER_NAME  ": freed IRQ %d\n", irq);
-#       endif
+	dprintk("freed IRQ %d\n", irq);
+	lirc_buffer_free(&rbuf);
 	
 	MOD_DEC_USE_COUNT;
-	
-#       ifdef KERNEL_2_1
-	return 0;
-#       endif
 }
 
-#ifdef KERNEL_2_1
-static unsigned int lirc_poll(struct file *file, poll_table * wait)
-{
-	poll_wait(file, &lirc_wait_in, wait);
-	if (rbh != rbt)
-		return POLLIN | POLLRDNORM;
-	return 0;
-}
-#else
-static int lirc_select(struct inode *node, struct file *file,
-		       int sel_type, select_table * wait)
-{
-	if (sel_type != SEL_IN)
-		return 0;
-	if (rbh != rbt)
-		return 1;
-	select_wait(&lirc_wait_in, wait);
-	return 0;
-}
-#endif
-
-#ifdef KERNEL_2_1
-static ssize_t lirc_read(struct file *file, char *buf,
-			 size_t count, loff_t * ppos)
-#else
-static int lirc_read(struct inode *node, struct file *file, char *buf,
-		     int count)
-#endif
-{
-	int n=0,retval=0;
-#ifdef KERNEL_2_3
-	DECLARE_WAITQUEUE(wait,current);
-#else
-	struct wait_queue wait={current,NULL};
-#endif
-	
-	if(n%sizeof(lirc_t)) return(-EINVAL);
-	
-	add_wait_queue(&lirc_wait_in,&wait);
-	current->state=TASK_INTERRUPTIBLE;
-	while (n < count)
-	{
-		if (rbt != rbh) {
-#                       ifdef KERNEL_2_1
-			copy_to_user((void *) buf+n,
-				     (void *) &rbuf[rbh],sizeof(lirc_t));
-#                       else
-			memcpy_tofs((void *) buf+n,
-				    (void *) &rbuf[rbh],sizeof(lirc_t));
-#                       endif
-			rbh = (rbh + 1) & (RBUF_LEN - 1);
-			n+=sizeof(lirc_t);
-		} else {
-			if (file->f_flags & O_NONBLOCK) {
-				retval = -EAGAIN;
-				break;
-			}
-#                       ifdef KERNEL_2_1
-			if (signal_pending(current)) {
-				retval = -ERESTARTSYS;
-				break;
-			}
-#                       else
-			if (current->signal & ~current->blocked) {
-				retval = -EINTR;
-				break;
-			}
-#                       endif
-			schedule();
-			current->state=TASK_INTERRUPTIBLE;
-		}
-	}
-	remove_wait_queue(&lirc_wait_in,&wait);
-	current->state=TASK_RUNNING;
-	return (n ? n : retval);
-}
-
-#ifdef KERNEL_2_1
 static ssize_t lirc_write(struct file *file, const char *buf,
 			 size_t n, loff_t * ppos)
-#else
-static int lirc_write(struct inode *node, struct file *file, const char *buf,
-                     int n)
-#endif
 {
 	int retval,i,count;
 	unsigned long flags;
@@ -1038,12 +909,8 @@ static int lirc_write(struct inode *node, struct file *file, const char *buf,
 	if(retval) return(retval);
 	count=n/sizeof(lirc_t);
 	if(count>WBUF_LEN || count%2==0) return(-EINVAL);
-#       ifdef KERNEL_2_1
 	copy_from_user(wbuf,buf,n);
-#       else
-	memcpy_fromfs(wbuf,buf,n);
-#       endif
-	save_flags(flags);cli();
+	local_irq_save(flags);
 	if(hardware[type].type==LIRC_IRDEO)
 	{
 		/* DTR, RTS down */
@@ -1055,7 +922,7 @@ static int lirc_write(struct inode *node, struct file *file, const char *buf,
 		else delta=hardware[type].send_pulse(wbuf[i]);
 	}
 	off();
-	restore_flags(flags);
+	local_irq_restore(flags);
 	return(n);
 }
 
@@ -1068,59 +935,16 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 	
 	switch(cmd)
 	{
-	case LIRC_GET_FEATURES:
-#               ifdef KERNEL_2_1
-		result=put_user(hardware[type].features,
-				(unsigned long *) arg);
-		if(result) return(result); 
-#               else
-		result=verify_area(VERIFY_WRITE,(unsigned long*) arg,
-				   sizeof(unsigned long));
-		if(result) return(result);
-		put_user(hardware[type].features,(unsigned long *) arg);
-#               endif
-		break;
-		
 	case LIRC_GET_SEND_MODE:
 		if(!(hardware[type].features&LIRC_CAN_SEND_MASK))
 		{
 			return(-ENOIOCTLCMD);
 		}
 		
-#               ifdef KERNEL_2_1
 		result=put_user(LIRC_SEND2MODE
 				(hardware[type].features&LIRC_CAN_SEND_MASK),
 				(unsigned long *) arg);
 		if(result) return(result); 
-#               else
-		result=verify_area(VERIFY_WRITE,(unsigned long *) arg,
-				   sizeof(unsigned long));
-		if(result) return(result);
-		put_user(LIRC_SEND2MODE
-			 (hardware[type].features&LIRC_CAN_SEND_MASK),
-			 (unsigned long *) arg);
-#               endif
-		break;
-		
-	case LIRC_GET_REC_MODE:
-		if(!(hardware[type].features&LIRC_CAN_REC_MASK))
-		{
-			return(-ENOIOCTLCMD);
-		}
-		
-#               ifdef KERNEL_2_1
-		result=put_user(LIRC_REC2MODE
-				(hardware[type].features&LIRC_CAN_REC_MASK),
-				(unsigned long *) arg);
-		if(result) return(result); 
-#               else
-		result=verify_area(VERIFY_WRITE,(unsigned long *) arg,
-				   sizeof(unsigned long));
-		if(result) return(result);
-		put_user(LIRC_REC2MODE
-			 (hardware[type].features&LIRC_CAN_REC_MASK),
-			 (unsigned long *) arg);
-#               endif
 		break;
 		
 	case LIRC_SET_SEND_MODE:
@@ -1129,110 +953,40 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 			return(-ENOIOCTLCMD);
 		}
 		
-#               ifdef KERNEL_2_1
 		result=get_user(value,(unsigned long *) arg);
 		if(result) return(result);
-#               else
-		result=verify_area(VERIFY_READ,(unsigned long *) arg,
-				   sizeof(unsigned long));
-		if(result) return(result);
-		value=get_user((unsigned long *) arg);
-#               endif
 		/* only LIRC_MODE_PULSE supported */
 		if(value!=LIRC_MODE_PULSE) return(-ENOSYS);
 		break;
 		
-	case LIRC_SET_REC_MODE:
-		if(!(hardware[type].features&LIRC_CAN_REC_MASK))
-		{
-			return(-ENOIOCTLCMD);
-		}
-		
-#               ifdef KERNEL_2_1
-		result=get_user(value,(unsigned long *) arg);
-		if(result) return(result);
-#               else
-		result=verify_area(VERIFY_READ,(unsigned long *) arg,
-				   sizeof(unsigned long));
-		if(result) return(result);
-		value=get_user((unsigned long *) arg);
-#               endif
-		/* only LIRC_MODE_MODE2 supported */
-		if(value!=LIRC_MODE_MODE2) return(-ENOSYS);
+	case LIRC_GET_LENGTH:
+		return(-ENOSYS);
 		break;
 		
 	case LIRC_SET_SEND_DUTY_CYCLE:
+		dprintk("SET_SEND_DUTY_CYCLE\n");
 		if(!(hardware[type].features&LIRC_CAN_SET_SEND_DUTY_CYCLE))
 		{
 			return(-ENOIOCTLCMD);
 		}
 		
-#               ifdef KERNEL_2_1
 		result=get_user(ivalue,(unsigned int *) arg);
 		if(result) return(result);
-#               else
-		result=verify_area(VERIFY_READ,(unsigned int *) arg,
-				   sizeof(unsigned int));
-		if(result) return(result);
-		ivalue=get_user((unsigned int *) arg);
-#               endif
 		if(ivalue<=0 || ivalue>100) return(-EINVAL);
-#               ifdef USE_RDTSC
-		duty_cycle=ivalue;
-		calc_pulse_lengths_in_clocks();
-#               else /* ! USE_RDTSC */
-		if(256*1000000L/freq*ivalue/100<=
-		   LIRC_SERIAL_TRANSMITTER_LATENCY) return(-EINVAL);
-		if(256*1000000L/freq*(100-ivalue)/100<=
-		   LIRC_SERIAL_TRANSMITTER_LATENCY) return(-EINVAL);
-		duty_cycle=ivalue;
-		period=256*1000000L/freq;
-		pulse_width=period*duty_cycle/100;
-		space_width=period-pulse_width;
-#               endif /* USE_RDTSC */
-#               ifdef DEBUG
-		printk(KERN_WARNING LIRC_DRIVER_NAME
-		       ": after SET_SEND_DUTY_CYCLE, freq=%d pulse=%ld, "
-		       "space=%ld, conv_us_to_clocks=%ld\n",
-		       freq, pulse_width, space_width, conv_us_to_clocks);
-#               endif
+		return init_timing_params(ivalue, freq);
 		break;
 		
 	case LIRC_SET_SEND_CARRIER:
+		dprintk("SET_SEND_CARRIER\n");
 		if(!(hardware[type].features&LIRC_CAN_SET_SEND_CARRIER))
 		{
 			return(-ENOIOCTLCMD);
 		}
 		
-#               ifdef KERNEL_2_1
 		result=get_user(ivalue,(unsigned int *) arg);
 		if(result) return(result);
-#               else
-		result=verify_area(VERIFY_READ,(unsigned int *) arg,
-				   sizeof(unsigned int));
-		if(result) return(result);
-		ivalue=get_user((unsigned int *) arg);
-#               endif
 		if(ivalue>500000 || ivalue<20000) return(-EINVAL);
-#               ifdef USE_RDTSC
-		freq=ivalue;
-		calc_pulse_lengths_in_clocks();
-#               else /* !USE_RDTSC */
-		if(256*1000000L/freq*ivalue/100<=
-		   LIRC_SERIAL_TRANSMITTER_LATENCY) return(-EINVAL);
-		if(256*1000000L/freq*(100-ivalue)/100<=
-		   LIRC_SERIAL_TRANSMITTER_LATENCY) return(-EINVAL);
-		freq=ivalue;
-		period=256*1000000L/freq;
-		pulse_width=period*duty_cycle/100;
-		space_width=period-pulse_width;
-#               endif /* USE_RDTSC */
-#               ifdef DEBUG
-		printk(KERN_WARNING LIRC_DRIVER_NAME
-		       ": after SET_SEND_CARRIER, freq=%d pulse=%ld, "
-		       "space=%ld, conv_us_to_clocks=%ld\n",
-		       freq, pulse_width, space_width, conv_us_to_clocks);
-#               endif
+		return init_timing_params(duty_cycle, ivalue);
 		break;
 		
 	default:
@@ -1243,48 +997,26 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 
 static struct file_operations lirc_fops =
 {
-	read:    lirc_read,
 	write:   lirc_write,
-#       ifdef KERNEL_2_1
-	poll:    lirc_poll,
-#       else
-	select:  lirc_select,
-#       endif
-	ioctl:   lirc_ioctl,
-	open:    lirc_open,
-	release: lirc_close
+};
+
+static struct lirc_plugin plugin = {
+	name:		LIRC_DRIVER_NAME,
+	minor:		-1,
+	code_length:	1,
+	sample_rate:	0,
+	data:		NULL,
+	add_to_buf:	NULL,
+	get_queue:	NULL,
+	rbuf:		&rbuf,
+	set_use_inc:	set_use_inc,
+	set_use_dec:	set_use_dec,
+	ioctl:		lirc_ioctl,
+	fops:		&lirc_fops,
 };
 
 #ifdef MODULE
 
-#if LINUX_VERSION_CODE >= 0x020100
-MODULE_AUTHOR("Ralph Metzler, Trent Piepho, Ben Pfaff, Christoph Bartelmus");
-MODULE_DESCRIPTION("Infra-red receiver driver for serial ports.");
-#ifdef MODULE_LICENSE
-MODULE_LICENSE("GPL");
-#endif
-
-MODULE_PARM(type, "i");
-MODULE_PARM_DESC(type, "Hardware type (0 = home-brew, 1 = IRdeo,"
-		 " 2 = IRdeo Remote, 3 = AnimaX");
-
-MODULE_PARM(io, "i");
-MODULE_PARM_DESC(io, "I/O address base (0x3f8 or 0x2f8)");
-
-MODULE_PARM(irq, "i");
-MODULE_PARM_DESC(irq, "Interrupt (4 or 3)");
-
-MODULE_PARM(sense, "i");
-MODULE_PARM_DESC(sense, "Override autodetection of IR receiver circuit"
-		 " (0 = active high, 1 = active low )");
-
-MODULE_PARM(softcarrier, "i");
-MODULE_PARM_DESC(softcarrier, "Software carrier (0 = off, 1 = on)");
-
-
-EXPORT_NO_SYMBOLS;
-#endif
-
 int init_module(void)
 {
 	int result;
@@ -1295,6 +1027,7 @@ int init_module(void)
 	case LIRC_IRDEO:
 	case LIRC_IRDEO_REMOTE:
 	case LIRC_ANIMAX:
+	case LIRC_IGOR:
 		break;
 	default:
 		return(-EINVAL);
@@ -1306,7 +1039,8 @@ int init_module(void)
 	}
 	if ((result = init_port()) < 0)
 		return result;
-	if (register_chrdev(major, LIRC_DRIVER_NAME, &lirc_fops) < 0) {
+	plugin.features = hardware[type].features;
+	if ((plugin.minor = lirc_register_plugin(&plugin)) < 0) {
 		printk(KERN_ERR  LIRC_DRIVER_NAME  
 		       ": register_chrdev failed!\n");
 		release_region(io, 8);
@@ -1318,10 +1052,34 @@ int init_module(void)
 void cleanup_module(void)
 {
 	release_region(io, 8);
-	unregister_chrdev(major, LIRC_DRIVER_NAME);
-#       ifdef DEBUG
-	printk(KERN_INFO  LIRC_DRIVER_NAME  ": cleaned up module\n");
-#       endif
+	lirc_unregister_plugin(plugin.minor);
+	dprintk("cleaned up module\n");
 }
 
-#endif
+MODULE_DESCRIPTION("Infra-red receiver driver for serial ports.");
+MODULE_AUTHOR("Ralph Metzler, Trent Piepho, Ben Pfaff, Christoph Bartelmus");
+MODULE_LICENSE("GPL");
+
+module_param(type, int, 0444);
+MODULE_PARM_DESC(type, "Hardware type (0 = home-brew, 1 = IRdeo,"
+		 " 2 = IRdeo Remote, 3 = AnimaX, 4 = IgorPlug");
+
+module_param(io, int, 0444);
+MODULE_PARM_DESC(io, "I/O address base (0x3f8 or 0x2f8)");
+
+module_param(irq, int, 0444);
+MODULE_PARM_DESC(irq, "Interrupt (4 or 3)");
+
+module_param(sense, bool, 0444);
+MODULE_PARM_DESC(sense, "Override autodetection of IR receiver circuit"
+		 " (0 = active high, 1 = active low )");
+
+module_param(softcarrier, bool, 0444);
+MODULE_PARM_DESC(softcarrier, "Software carrier (0 = off, 1 = on)");
+
+module_param(debug, bool, 0644);
+MODULE_PARM_DESC(debug, "Enable debugging messages");
+
+EXPORT_NO_SYMBOLS;
+
+#endif /* MODULE */
