@@ -35,15 +35,11 @@
 
 
 #include <linux/version.h>
-#if LINUX_VERSION_CODE >= 0x020100
-#define KERNEL_2_1
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
-#define KERNEL_2_3
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 18)
+#error "**********************************************************"
+#error " Sorry, this driver needs kernel version 2.2.18 or higher "
+#error "**********************************************************"
 #endif
-#else
-#define KERNEL_2_0
-#endif
-
 #include <linux/module.h>
 
 #ifdef HAVE_CONFIG_H
@@ -52,7 +48,7 @@
  
 #include <linux/config.h>
 
-#if !defined(LIRC_ON_IPAQ) && !defined(CONFIG_SERIAL_MODULE)
+#if !defined(LIRC_ON_SA1100) && !defined(CONFIG_SERIAL_MODULE)
 #warning "******************************************"
 #warning " Your serial port driver is compiled into "
 #warning " the kernel. You will have to release the "
@@ -61,6 +57,7 @@
 #warning "******************************************"
 #endif
 
+#include <linux/module.h>
 #include <linux/sched.h>
 #include <linux/errno.h>
 #include <linux/signal.h>
@@ -76,21 +73,25 @@
 #include <linux/wait.h>
 #include <linux/mm.h>
 #include <linux/delay.h>
-#ifdef KERNEL_2_1
 #include <linux/poll.h>
-#endif
 #include <asm/system.h>
 #include <asm/segment.h>
 #include <asm/io.h>
 #include <asm/irq.h>
 #include <asm/fcntl.h>
-#ifdef LIRC_ON_IPAQ
+#ifdef LIRC_ON_SA1100
 #include <asm/hardware.h>
+#ifdef CONFIG_SA1100_COLLIE
+#include <asm/arch/tc35143.h>
+#include <asm/ucb1200.h>
+#endif
 #endif
 
 #include <linux/timer.h>
 
 #include "drivers/lirc.h"
+#include "drivers/lirc_dev/lirc_dev.h"
+#include "drivers/kcompat.h"
 
 /* SECTION: Definitions */
 
@@ -112,8 +113,12 @@
 
 #endif
 
-/******************************** iPAQ ********************************/
-#ifdef LIRC_ON_IPAQ
+#ifdef LIRC_SIR_ACTISYS_ACT200L
+static void init_act200(void);
+#endif
+
+/******************************* SA1100 ********************************/
+#ifdef LIRC_ON_SA1100
 struct sa1100_ser2_registers
 {
 	/* HSSP control register */
@@ -131,14 +136,14 @@ struct sa1100_ser2_registers
 
 static int irq=IRQ_Ser2ICP;
 
-#define LIRC_ON_IPAQ_TRANSMITTER_LATENCY 0
+#define LIRC_ON_SA1100_TRANSMITTER_LATENCY 0
 
 /* pulse/space ratio of 50/50 */
-unsigned long pulse_width = (13-LIRC_ON_IPAQ_TRANSMITTER_LATENCY);
+static unsigned long pulse_width = (13-LIRC_ON_SA1100_TRANSMITTER_LATENCY);
 /* 1000000/freq-pulse_width */
-unsigned long space_width = (13-LIRC_ON_IPAQ_TRANSMITTER_LATENCY);
-unsigned int freq = 38000;      /* modulation frequency */
-unsigned int duty_cycle = 50;   /* duty cycle of 50% */
+static unsigned long space_width = (13-LIRC_ON_SA1100_TRANSMITTER_LATENCY);
+static unsigned int freq = 38000;      /* modulation frequency */
+static unsigned int duty_cycle = 50;   /* duty cycle of 50% */
 
 #endif
 
@@ -159,11 +164,10 @@ unsigned int duty_cycle = 50;   /* duty cycle of 50% */
 /* must be longer than TIME_CONST */
 #define SIR_TIMEOUT	(HZ*5/100)
 
-static int major = LIRC_MAJOR;
-
-#ifndef LIRC_ON_IPAQ
+#ifndef LIRC_ON_SA1100
 static int io = LIRC_PORT;
 static int irq = LIRC_IRQ;
+static int threshold = 3;
 #endif
 
 static spinlock_t timer_lock = SPIN_LOCK_UNLOCKED;
@@ -174,11 +178,7 @@ static struct timeval last_tv = {0, 0};
 static struct timeval last_intr_tv = {0, 0};
 static int last_value = 0;
 
-#ifdef KERNEL_2_3
 static DECLARE_WAIT_QUEUE_HEAD(lirc_read_queue);
-#else
-static struct wait_queue * lirc_read_queue = NULL;
-#endif
 
 static spinlock_t hardware_lock = SPIN_LOCK_UNLOCKED;
 static spinlock_t dev_lock = SPIN_LOCK_UNLOCKED;
@@ -188,18 +188,19 @@ static lirc_t rx_buf[RBUF_LEN]; unsigned int rx_tail = 0, rx_head = 0;
 static lirc_t tx_buf[WBUF_LEN];
 #endif
 
+static int debug = 0;
+#define dprintk(fmt, args...)                                     \
+	do{                                                       \
+		if(debug) printk(KERN_DEBUG LIRC_DRIVER_NAME ": " \
+                                 fmt, ## args);                   \
+	}while(0)
+
 /* SECTION: Prototypes */
 
 /* Communication with user-space */
 static int lirc_open(struct inode * inode, struct file * file);
-#ifdef KERNEL_2_1
 static int lirc_close(struct inode * inode, struct file *file);
 static unsigned int lirc_poll(struct file * file, poll_table * wait);
-#else
-static void lirc_close(struct inode * inode, struct file *file);
-static int lirc_select(struct inode * inode, struct file * file,
-		int type, select_table * wait);
-#endif
 static ssize_t lirc_read(struct file * file, char * buf, size_t count,
 		loff_t * ppos);
 static ssize_t lirc_write(struct file * file, const char * buf, size_t n, loff_t * pos);
@@ -211,7 +212,8 @@ static int init_chrdev(void);
 static void drop_chrdev(void);
 #endif
 	/* Hardware */
-static void sir_interrupt(int irq, void * dev_id, struct pt_regs * regs);
+static irqreturn_t sir_interrupt(int irq, void * dev_id,
+				 struct pt_regs * regs);
 #ifndef LIRC_SIR_TEKRAM
 static void send_space(unsigned long len);
 static void send_pulse(unsigned long len);
@@ -224,13 +226,13 @@ static void drop_port(void);
 int init_module(void);
 void cleanup_module(void);
 
-#ifdef LIRC_ON_IPAQ
-void inline on(void)
+#ifdef LIRC_ON_SA1100
+static void inline on(void)
 {
 	PPSR|=PPC_TXD2;
 }
   
-void inline off(void)
+static void inline off(void)
 {
 	PPSR&=~PPC_TXD2;
 }
@@ -276,19 +278,12 @@ static int lirc_open(struct inode * inode, struct file * file)
 	return 0;
 }
 
-#ifdef KERNEL_2_1
 static int lirc_close(struct inode * inode, struct file *file)
-#else
-static void lirc_close(struct inode * inode, struct file *file)
-#endif
 {
 	MOD_DEC_USE_COUNT;
-#ifdef KERNEL_2_1
 	return 0;
-#endif
 }
 
-#ifdef KERNEL_2_1
 static unsigned int lirc_poll(struct file * file, poll_table * wait)
 {
 	poll_wait(file, &lirc_read_queue, wait);
@@ -296,29 +291,13 @@ static unsigned int lirc_poll(struct file * file, poll_table * wait)
 		return POLLIN | POLLRDNORM;
 	return 0;
 }
-#else
-static int lirc_select(struct inode * inode, struct file * file,
-		int type, select_table * wait)
-{
-	if (type != SEL_IN)
-		return 0;
-	if (rx_head != rx_tail)
-		return 1;
-	select_wait(&lirc_read_queue, wait);
-	return 0;
-}
-#endif
 
 static ssize_t lirc_read(struct file * file, char * buf, size_t count,
 		loff_t * ppos)
 {
 	int n=0;
 	int retval = 0;
-#ifdef KERNEL_2_3
 	DECLARE_WAITQUEUE(wait,current);
-#else
-	struct wait_queue wait={current,NULL};
-#endif
 	
 	if(n%sizeof(lirc_t)) return(-EINVAL);
 	
@@ -334,13 +313,8 @@ static ssize_t lirc_read(struct file * file, char * buf, size_t count,
 			{
 				return retval;
 			}
-#ifdef KERNEL_2_1
 			copy_to_user((void *) buf+n,(void *) (rx_buf+rx_head),
 				     sizeof(lirc_t));
-#else
-			memcpy_tofs((void *) buf+n,(void *) (rx_buf+rx_head),
-				    sizeof(lirc_t));
-#endif
 			rx_head=(rx_head+1)&(RBUF_LEN-1);
 			n+=sizeof(lirc_t);
 		}
@@ -351,19 +325,11 @@ static ssize_t lirc_read(struct file * file, char * buf, size_t count,
 				retval=-EAGAIN;
 				break;
 			}
-#                       ifdef KERNEL_2_1
 			if(signal_pending(current))
 			{
 				retval=-ERESTARTSYS;
 				break;
 			}
-#                       else
-			if(current->signal & ~current->blocked)
-			{
-				retval=-EINTR;
-				break;
-			}
-#                       endif
 			schedule();
 			current->state=TASK_INTERRUPTIBLE;
 		}
@@ -374,6 +340,7 @@ static ssize_t lirc_read(struct file * file, char * buf, size_t count,
 }
 static ssize_t lirc_write(struct file * file, const char * buf, size_t n, loff_t * pos)
 {
+	unsigned long flags;
 #ifdef LIRC_SIR_TEKRAM
 	return(-EBADF);
 #else
@@ -388,10 +355,11 @@ static ssize_t lirc_write(struct file * file, const char * buf, size_t n, loff_t
 	copy_from_user(tx_buf, buf, n);
 	i = 0;
 	n/=sizeof(lirc_t);
-#ifdef LIRC_ON_IPAQ
+#ifdef LIRC_ON_SA1100
 	/* disable receiver */
 	Ser2UTCR3=0;
 #endif
+	local_irq_save(flags);
 	while (1) {
 		if (i >= n)
 			break;
@@ -404,7 +372,8 @@ static ssize_t lirc_write(struct file * file, const char * buf, size_t n, loff_t
 			send_space(tx_buf[i]);
 		i++;
 	}
-#ifdef LIRC_ON_IPAQ
+	local_irq_restore(flags);
+#ifdef LIRC_ON_SA1100
 	off();
 	udelay(1000); /* wait 1ms for IR diode to recover */
 	Ser2UTCR3=0;
@@ -422,7 +391,7 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 {
 	int retval = 0;
 	unsigned long value = 0;
-#ifdef LIRC_ON_IPAQ
+#ifdef LIRC_ON_SA1100
 	unsigned int ivalue;
 #endif
 
@@ -433,7 +402,7 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 		value = 0;
 	else if (cmd == LIRC_GET_REC_MODE)
 		value = LIRC_MODE_MODE2;
-#elif defined(LIRC_ON_IPAQ)
+#elif defined(LIRC_ON_SA1100)
 	if (cmd == LIRC_GET_FEATURES)
 		value = LIRC_CAN_SEND_PULSE |
 			LIRC_CAN_SET_SEND_DUTY_CYCLE |
@@ -456,68 +425,38 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 	case LIRC_GET_FEATURES:
 	case LIRC_GET_SEND_MODE:
 	case LIRC_GET_REC_MODE:
-#ifdef KERNEL_2_0
-		retval = verify_area(VERIFY_WRITE, (unsigned long *) arg,
-			sizeof(unsigned long));
-		if (retval)
-			break;
-#else
-		retval =
-#endif
-		put_user(value, (unsigned long *) arg);
+		retval = put_user(value, (unsigned long *) arg);
 		break;
 
 	case LIRC_SET_SEND_MODE:
 	case LIRC_SET_REC_MODE:
-#ifdef KERNEL_2_0
-		retval = verify_area(VERIFY_READ, (unsigned long *) arg,
-			sizeof(unsigned long));
-		if (retval)
-			break;
-		value = get_user((unsigned long *) arg);
-#else
 		retval = get_user(value, (unsigned long *) arg);
-#endif
 		break;
-#ifdef LIRC_ON_IPAQ
+#ifdef LIRC_ON_SA1100
 	case LIRC_SET_SEND_DUTY_CYCLE:
-#               ifdef KERNEL_2_1
 		retval=get_user(ivalue,(unsigned int *) arg);
 		if(retval) return(retval);
-#               else
-		retval=verify_area(VERIFY_READ,(unsigned int *) arg,
-				   sizeof(unsigned int));
-		if(result) return(result);
-		ivalue=get_user((unsigned int *) arg);
-#               endif
 		if(ivalue<=0 || ivalue>100) return(-EINVAL);
 		/* (ivalue/100)*(1000000/freq) */
 		duty_cycle=ivalue;
 		pulse_width=(unsigned long) duty_cycle*10000/freq;
 		space_width=(unsigned long) 1000000L/freq-pulse_width;
-		if(pulse_width>=LIRC_ON_IPAQ_TRANSMITTER_LATENCY)
-			pulse_width-=LIRC_ON_IPAQ_TRANSMITTER_LATENCY;
-		if(space_width>=LIRC_ON_IPAQ_TRANSMITTER_LATENCY)
-			space_width-=LIRC_ON_IPAQ_TRANSMITTER_LATENCY;
+		if(pulse_width>=LIRC_ON_SA1100_TRANSMITTER_LATENCY)
+			pulse_width-=LIRC_ON_SA1100_TRANSMITTER_LATENCY;
+		if(space_width>=LIRC_ON_SA1100_TRANSMITTER_LATENCY)
+			space_width-=LIRC_ON_SA1100_TRANSMITTER_LATENCY;
 		break;
 	case LIRC_SET_SEND_CARRIER:
-#               ifdef KERNEL_2_1
 		retval=get_user(ivalue,(unsigned int *) arg);
 		if(retval) return(retval);
-#               else
-		retval=verify_area(VERIFY_READ,(unsigned int *) arg,
-				   sizeof(unsigned int));
-		if(retval) return(retval);
-		ivalue=get_user((unsigned int *) arg);
-#               endif
 		if(ivalue>500000 || ivalue<20000) return(-EINVAL);
 		freq=ivalue;
 		pulse_width=(unsigned long) duty_cycle*10000/freq;
 		space_width=(unsigned long) 1000000L/freq-pulse_width;
-		if(pulse_width>=LIRC_ON_IPAQ_TRANSMITTER_LATENCY)
-			pulse_width-=LIRC_ON_IPAQ_TRANSMITTER_LATENCY;
-		if(space_width>=LIRC_ON_IPAQ_TRANSMITTER_LATENCY)
-			space_width-=LIRC_ON_IPAQ_TRANSMITTER_LATENCY;
+		if(pulse_width>=LIRC_ON_SA1100_TRANSMITTER_LATENCY)
+			pulse_width-=LIRC_ON_SA1100_TRANSMITTER_LATENCY;
+		if(space_width>=LIRC_ON_SA1100_TRANSMITTER_LATENCY)
+			space_width-=LIRC_ON_SA1100_TRANSMITTER_LATENCY;
 		break;
 #endif
 	default:
@@ -552,11 +491,7 @@ static void add_read_queue(int flag, unsigned long val)
 	unsigned int new_rx_tail;
 	lirc_t newval;
 
-#ifdef DEBUG_SIGNAL
-	printk(KERN_DEBUG LIRC_DRIVER_NAME
-		": add flag %d with val %lu\n",
-		flag,val);
-#endif
+	dprintk("add flag %d with val %lu\n", flag, val);
 
 	newval = val & PULSE_MASK;
 
@@ -580,9 +515,7 @@ static void add_read_queue(int flag, unsigned long val)
 	}
 	new_rx_tail = (rx_tail + 1) & (RBUF_LEN - 1);
 	if (new_rx_tail == rx_head) {
-#               ifdef DEBUG
-		printk(KERN_WARNING LIRC_DRIVER_NAME ": Buffer overrun.\n");
-#               endif
+		dprintk("Buffer overrun.\n");
 		return;
 	}
 	rx_buf[rx_tail] = newval;
@@ -594,32 +527,54 @@ static struct file_operations lirc_fops =
 {
 	read:    lirc_read,
 	write:   lirc_write,
-#ifdef KERNEL_2_1
 	poll:    lirc_poll,
-#else
-	select:  lirc_select,
-#endif
 	ioctl:   lirc_ioctl,
 	open:    lirc_open,
 	release: lirc_close,
 };
 
-#ifdef MODULE
-int init_chrdev(void)
+static int set_use_inc(void* data)
 {
-	int retval;
+#if WE_DONT_USE_LOCAL_OPEN_CLOSE
+	MOD_INC_USE_COUNT;
+#endif
+       return 0;
+}
 
-	retval = register_chrdev(major, LIRC_DRIVER_NAME, &lirc_fops);
-	if (retval < 0) {
+static void set_use_dec(void* data)
+{
+#if WE_DONT_USE_LOCAL_OPEN_CLOSE
+	MOD_DEC_USE_COUNT;
+#endif
+}
+static struct lirc_plugin plugin = {
+       name:           LIRC_DRIVER_NAME,
+       minor:          -1,
+       code_length:    1,
+       sample_rate:    0,
+       data:           NULL,
+       add_to_buf:     NULL,
+       get_queue:      NULL,
+       set_use_inc:    set_use_inc,
+       set_use_dec:    set_use_dec,
+       fops:           &lirc_fops,
+};
+
+
+#ifdef MODULE
+static int init_chrdev(void)
+{
+	plugin.minor = lirc_register_plugin(&plugin);
+	if (plugin.minor < 0) {
 		printk(KERN_ERR LIRC_DRIVER_NAME ": init_chrdev() failed.\n");
-		return retval;
+		return -EIO;
 	}
 	return 0;
 }
 
 static void drop_chrdev(void)
 {
-	unregister_chrdev(major, LIRC_DRIVER_NAME);
+	lirc_unregister_plugin(plugin.minor);
 }
 #endif
 
@@ -652,16 +607,13 @@ static void sir_timeout(unsigned long data)
  	spin_lock_irqsave(&timer_lock, flags);
 	if (last_value)
 	{
-#ifndef LIRC_ON_IPAQ
+#ifndef LIRC_ON_SA1100
 		/* clear unread bits in UART and restart */
 		outb(UART_FCR_CLEAR_RCVR, io + UART_FCR);
 #endif
 		/* determine 'virtual' pulse end: */
 	 	pulse_end = delta(&last_tv, &last_intr_tv);
-#ifdef DEBUG_SIGNAL
-		printk(KERN_DEBUG LIRC_DRIVER_NAME
-			": timeout add %d for %lu usec\n",last_value,pulse_end);
-#endif
+		dprintk("timeout add %d for %lu usec\n",last_value,pulse_end);
 		add_read_queue(last_value,pulse_end);
 		last_value = 0;
 		last_tv=last_intr_tv;
@@ -669,12 +621,13 @@ static void sir_timeout(unsigned long data)
 	spin_unlock_irqrestore(&timer_lock, flags);		
 }
 
-static void sir_interrupt(int irq, void * dev_id, struct pt_regs * regs)
+static irqreturn_t sir_interrupt(int irq, void * dev_id,
+				 struct pt_regs * regs)
 {
 	unsigned char data;
 	struct timeval curr_tv;
 	static unsigned long deltv;
-#ifdef LIRC_ON_IPAQ
+#ifdef LIRC_ON_SA1100
 	int status;
 	static int n=0;
 	
@@ -687,18 +640,18 @@ static void sir_interrupt(int irq, void * dev_id, struct pt_regs * regs)
 	while (status & UTSR0_EIF)
 	{
 		int bstat;
+
+		if(debug) {
+			dprintk("EIF\n");
+			bstat = Ser2UTSR1;
 		
-#ifdef DEBUG
-		printk("EIF\n");
-		bstat = Ser2UTSR1;
-		
-		if (bstat & UTSR1_FRE)
-			printk("frame error\n");
-		if (bstat & UTSR1_ROR)
-			printk("receive fifo overrun\n");
-		if(bstat&UTSR1_PRE)
-			printk("parity error\n");
-#endif
+			if (bstat & UTSR1_FRE)
+				dprintk("frame error\n");
+			if (bstat & UTSR1_ROR)
+				dprintk("receive fifo overrun\n");
+			if (bstat & UTSR1_PRE)
+				dprintk("parity error\n");
+		}
 		
 		bstat = Ser2UTDR;
 		n++;
@@ -711,10 +664,7 @@ static void sir_interrupt(int irq, void * dev_id, struct pt_regs * regs)
 		deltv = delta(&last_tv, &curr_tv);
 		do
 		{
-#ifdef DEBUG_SIGNAL
-			printk(KERN_DEBUG LIRC_DRIVER_NAME": t %lu , d %d\n",
-			       deltintrtv,(int)data);
-#endif
+			dprintk("t %lu , d %d\n", deltintrtv, (int)data);
 			data=Ser2UTDR;
 			//printk("data: %d\n",data);
 			n++;
@@ -774,16 +724,12 @@ static void sir_interrupt(int irq, void * dev_id, struct pt_regs * regs)
 				do_gettimeofday(&curr_tv);
 				deltv = delta(&last_tv, &curr_tv);
 				deltintrtv = delta(&last_intr_tv, &curr_tv);
-#ifdef DEBUG_SIGNAL
-				printk(KERN_DEBUG LIRC_DRIVER_NAME": t %lu , d %d\n",deltintrtv,(int)data);
-#endif
-				/* if nothing came in last 2 cycles,
+				dprintk("t %lu , d %d\n", deltintrtv, (int)data);
+				/* if nothing came in last X cycles,
 				   it was gap */
-				if (deltintrtv > TIME_CONST * 2) {
+				if (deltintrtv > TIME_CONST * threshold) {
 					if (last_value) {
-#ifdef DEBUG_SIGNAL
-						printk(KERN_DEBUG LIRC_DRIVER_NAME ": GAP\n");
-#endif
+						dprintk("GAP\n");
 						/* simulate signal change */
 						add_read_queue(last_value,
 							       deltv-
@@ -831,10 +777,11 @@ static void sir_interrupt(int irq, void * dev_id, struct pt_regs * regs)
 		}
 	}
 #endif
+	return IRQ_RETVAL(IRQ_HANDLED);
 }
 
-#ifdef LIRC_ON_IPAQ
-void send_pulse(unsigned long length)
+#ifdef LIRC_ON_SA1100
+static void send_pulse(unsigned long length)
 {
 	unsigned long k,delay;
 	int flag;
@@ -860,7 +807,7 @@ void send_pulse(unsigned long length)
 	off();
 }
 
-void send_space(unsigned long length)
+static void send_space(unsigned long length)
 {
 	if(length==0) return;
 	off();
@@ -893,18 +840,46 @@ static void send_pulse(unsigned long len)
 }
 #endif
 
+#ifdef CONFIG_SA1100_COLLIE
+static inline int sa1100_irda_set_power_collie(int state)
+{
+	if (state) {
+		/*
+		 *  0 - off
+		 *  1 - short range, lowest power
+		 *  2 - medium range, medium power
+		 *  3 - maximum range, high power
+		 */
+		ucb1200_set_io_direction(TC35143_GPIO_IR_ON,
+					 TC35143_IODIR_OUTPUT);
+		ucb1200_set_io(TC35143_GPIO_IR_ON, TC35143_IODAT_LOW);
+		udelay(100);
+	}
+	else {
+		/* OFF */
+		ucb1200_set_io_direction(TC35143_GPIO_IR_ON,
+					 TC35143_IODIR_OUTPUT);
+		ucb1200_set_io(TC35143_GPIO_IR_ON, TC35143_IODAT_HIGH);
+	}
+	return 0;
+}
+#endif
+
 static int init_hardware(void)
 {
-	int flags;
+	unsigned long flags;
 	
 	spin_lock_irqsave(&hardware_lock, flags);
 	/* reset UART */
-#ifdef LIRC_ON_IPAQ
+#ifdef LIRC_ON_SA1100
 #ifdef CONFIG_SA1100_BITSY
 	if (machine_is_bitsy()) {
 		printk("Power on IR module\n");
 		set_bitsy_egpio(EGPIO_BITSY_IR_ON);
 	}
+#endif
+#ifdef CONFIG_SA1100_COLLIE
+	sa1100_irda_set_power_collie(3);	/* power on */
 #endif
 	sr.hscr0=Ser2HSCR0;
 
@@ -1029,6 +1004,9 @@ static int init_hardware(void)
 	outb(UART_IER_RDI, io + UART_IER);	
 	/* turn on UART */
 	outb(UART_MCR_DTR|UART_MCR_RTS|UART_MCR_OUT2, io + UART_MCR);
+#ifdef LIRC_SIR_ACTISYS_ACT200L
+	init_act200();
+#endif
 #endif
 	spin_unlock_irqrestore(&hardware_lock, flags);
 	return 0;
@@ -1036,11 +1014,11 @@ static int init_hardware(void)
 
 static void drop_hardware(void)
 {
-	int flags;
+	unsigned long flags;
 
 	spin_lock_irqsave(&hardware_lock, flags);
 
-#ifdef LIRC_ON_IPAQ
+#ifdef LIRC_ON_SA1100
 	Ser2UTCR3=0;
 	
 	Ser2UTCR0=sr.utcr0;
@@ -1055,6 +1033,9 @@ static void drop_hardware(void)
 		clr_bitsy_egpio(EGPIO_BITSY_IR_ON);
 	}
 #endif
+#ifdef CONFIG_SA1100_COLLIE
+	sa1100_irda_set_power_collie(0);	/* power off */
+#endif
 #else
 	/* turn off interrupts */
 	outb(0, io + UART_IER);	
@@ -1068,7 +1049,7 @@ static int init_port(void)
 {
 	int retval;
 	
-#ifndef LIRC_ON_IPAQ
+#ifndef LIRC_ON_SA1100
 	/* get I/O port access and IRQ line */
 	retval = check_region(io, 8);
 	if (retval < 0) {
@@ -1086,7 +1067,7 @@ static int init_port(void)
 			irq);
 		return retval;
 	}
-#ifndef LIRC_ON_IPAQ
+#ifndef LIRC_ON_SA1100
 	request_region(io, 8, LIRC_DRIVER_NAME);
 	printk(KERN_INFO LIRC_DRIVER_NAME
 		": I/O port 0x%.4x, IRQ %d.\n",
@@ -1104,25 +1085,163 @@ static void drop_port(void)
 {
 	disable_irq(irq);
 	free_irq(irq, NULL);
-#ifdef KERNEL_2_3
 	del_timer_sync(&timerlist);
-#else
-	start_bh_atomic();
-	del_timer(&timerlist);
-	end_bh_atomic();
-#endif
-#ifndef LIRC_ON_IPAQ
+#ifndef LIRC_ON_SA1100
 	release_region(io, 8);
 #endif
 }
 
-int init_lirc_sir(void)
+#ifdef LIRC_SIR_ACTISYS_ACT200L
+/******************************************************/
+/* Crystal/Cirrus CS8130 IR transceiver, used in Actisys Act200L dongle */
+/* some code borrowed from Linux IRDA driver */
+
+/* Regsiter 0: Control register #1 */
+#define ACT200L_REG0    0x00
+#define ACT200L_TXEN    0x01 /* Enable transmitter */
+#define ACT200L_RXEN    0x02 /* Enable receiver */
+#define ACT200L_ECHO    0x08 /* Echo control chars */
+
+/* Register 1: Control register #2 */
+#define ACT200L_REG1    0x10
+#define ACT200L_LODB    0x01 /* Load new baud rate count value */
+#define ACT200L_WIDE    0x04 /* Expand the maximum allowable pulse */
+
+/* Register 3: Transmit mode register #2 */
+#define ACT200L_REG3    0x30
+#define ACT200L_B0      0x01 /* DataBits, 0=6, 1=7, 2=8, 3=9(8P)  */
+#define ACT200L_B1      0x02 /* DataBits, 0=6, 1=7, 2=8, 3=9(8P)  */
+#define ACT200L_CHSY    0x04 /* StartBit Synced 0=bittime, 1=startbit */
+
+/* Register 4: Output Power register */
+#define ACT200L_REG4    0x40
+#define ACT200L_OP0     0x01 /* Enable LED1C output */
+#define ACT200L_OP1     0x02 /* Enable LED2C output */
+#define ACT200L_BLKR    0x04
+
+/* Register 5: Receive Mode register */
+#define ACT200L_REG5    0x50
+#define ACT200L_RWIDL   0x01 /* fixed 1.6us pulse mode */
+    /*.. other various IRDA bit modes, and TV remote modes..*/
+
+/* Register 6: Receive Sensitivity register #1 */
+#define ACT200L_REG6    0x60
+#define ACT200L_RS0     0x01 /* receive threshold bit 0 */
+#define ACT200L_RS1     0x02 /* receive threshold bit 1 */
+
+/* Register 7: Receive Sensitivity register #2 */
+#define ACT200L_REG7    0x70
+#define ACT200L_ENPOS   0x04 /* Ignore the falling edge */
+
+/* Register 8,9: Baud Rate Dvider register #1,#2 */
+#define ACT200L_REG8    0x80
+#define ACT200L_REG9    0x90
+
+#define ACT200L_2400    0x5f
+#define ACT200L_9600    0x17
+#define ACT200L_19200   0x0b
+#define ACT200L_38400   0x05
+#define ACT200L_57600   0x03
+#define ACT200L_115200  0x01
+
+/* Register 13: Control register #3 */
+#define ACT200L_REG13   0xd0
+#define ACT200L_SHDW    0x01 /* Enable access to shadow registers */
+
+/* Register 15: Status register */
+#define ACT200L_REG15   0xf0
+
+/* Register 21: Control register #4 */
+#define ACT200L_REG21   0x50
+#define ACT200L_EXCK    0x02 /* Disable clock output driver */
+#define ACT200L_OSCL    0x04 /* oscillator in low power, medium accuracy mode */
+
+static void init_act200(void)
+{
+	int i;
+	__u8 control[] = {
+		ACT200L_REG15,
+		ACT200L_REG13 | ACT200L_SHDW,
+		ACT200L_REG21 | ACT200L_EXCK | ACT200L_OSCL,
+		ACT200L_REG13,
+		ACT200L_REG7  | ACT200L_ENPOS,
+		ACT200L_REG6  | ACT200L_RS0  | ACT200L_RS1,
+		ACT200L_REG5  | ACT200L_RWIDL,
+		ACT200L_REG4  | ACT200L_OP0  | ACT200L_OP1 | ACT200L_BLKR,
+		ACT200L_REG3  | ACT200L_B0,
+		ACT200L_REG0  | ACT200L_TXEN | ACT200L_RXEN,
+		ACT200L_REG8 |  (ACT200L_115200       & 0x0f),
+		ACT200L_REG9 | ((ACT200L_115200 >> 4) & 0x0f),
+		ACT200L_REG1 | ACT200L_LODB | ACT200L_WIDE
+	};
+
+	/* Set DLAB 1. */
+	soutp(UART_LCR, UART_LCR_DLAB | UART_LCR_WLEN8);
+	
+	/* Set divisor to 12 => 9600 Baud */
+	soutp(UART_DLM,0);
+	soutp(UART_DLL,12);
+	
+	/* Set DLAB 0. */
+	soutp(UART_LCR, UART_LCR_WLEN8);
+	/* Set divisor to 12 => 9600 Baud */
+
+	/* power supply */
+	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_DTR|UART_MCR_OUT2);
+	for (i=0; i<50; i++) {
+		safe_udelay(1000);
+	}
+
+		/* Reset the dongle : set RTS low for 25 ms */
+	soutp(UART_MCR, UART_MCR_DTR|UART_MCR_OUT2);
+	for (i=0; i<25; i++) {
+		udelay(1000);
+	}
+
+	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_DTR|UART_MCR_OUT2);
+	udelay(100);
+
+	/* Clear DTR and set RTS to enter command mode */
+	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_OUT2);
+	udelay(7);
+
+/* send out the control register settings for 115K 7N1 SIR operation */
+	for (i=0; i<sizeof(control); i++) {
+		soutp(UART_TX, control[i]);
+		/* one byte takes ~1042 usec to transmit at 9600,8N1 */
+		udelay(1500);
+	}
+
+	/* back to normal operation */
+	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_DTR|UART_MCR_OUT2);
+	udelay(50);
+
+	udelay(1500);
+	soutp(UART_LCR, sinp(UART_LCR) | UART_LCR_DLAB);
+
+	/* Set DLAB 1. */
+	soutp(UART_LCR, UART_LCR_DLAB | UART_LCR_WLEN7);
+
+	/* Set divisor to 1 => 115200 Baud */
+	soutp(UART_DLM,0);
+	soutp(UART_DLL,1);
+
+	/* Set DLAB 0. */
+	soutp(UART_LCR, sinp(UART_LCR) & (~UART_LCR_DLAB));
+
+	/* Set DLAB 0, 7 Bit */
+	soutp(UART_LCR, UART_LCR_WLEN7);
+
+	/* enable interrupts */
+	soutp(UART_IER, sinp(UART_IER)|UART_IER_RDI);
+}
+#endif
+
+static int init_lirc_sir(void)
 {
 	int retval;
 
-#ifdef KERNEL_2_3
 	init_waitqueue_head(&lirc_read_queue);
-#endif
 	retval = init_port();
 	if (retval < 0)
 		return retval;
@@ -1135,41 +1254,6 @@ int init_lirc_sir(void)
 
 #ifdef MODULE
 
-#ifdef KERNEL_2_1
-#ifdef LIRC_SIR_TEKRAM
-MODULE_AUTHOR("Christoph Bartelmus");
-MODULE_DESCRIPTION("Infrared receiver driver for Tekram Irmate 210");
-#ifdef MODULE_LICENSE
-MODULE_LICENSE("GPL");
-#endif
-#else
-#ifdef LIRC_ON_IPAQ
-MODULE_AUTHOR("Christoph Bartelmus");
-MODULE_DESCRIPTION("LIRC driver for StrongARM SA1100 embedded microprocessor");
-#ifdef MODULE_LICENSE
-MODULE_LICENSE("GPL");
-#endif
-#else
-MODULE_AUTHOR("Milan Pikula");
-MODULE_DESCRIPTION("Infrared receiver driver for SIR type serial ports");
-#ifdef MODULE_LICENSE
-MODULE_LICENSE("GPL");
-#endif
-#endif
-#endif
-#ifdef LIRC_ON_IPAQ
-MODULE_PARM(irq, "i");
-MODULE_PARM_DESC(irq, "Interrupt (16)");
-#else
-MODULE_PARM(io, "i");
-MODULE_PARM_DESC(io, "I/O address base (0x3f8 or 0x2f8)");
-MODULE_PARM(irq, "i");
-MODULE_PARM_DESC(irq, "Interrupt (4 or 3)");
-#endif
-
-EXPORT_NO_SYMBOLS;
-#endif
-
 int init_module(void)
 {
 	int retval;
@@ -1192,4 +1276,39 @@ void cleanup_module(void)
 	drop_port();
 	printk(KERN_INFO LIRC_DRIVER_NAME ": Uninstalled.\n");
 }
+
+#ifdef LIRC_SIR_TEKRAM
+MODULE_DESCRIPTION("Infrared receiver driver for Tekram Irmate 210");
+MODULE_AUTHOR("Christoph Bartelmus");
+#elif defined(LIRC_ON_SA1100)
+MODULE_DESCRIPTION("LIRC driver for StrongARM SA1100 embedded microprocessor");
+MODULE_AUTHOR("Christoph Bartelmus");
+#elif defined(LIRC_SIR_ACTISYS_ACT200L)
+MODULE_DESCRIPTION("LIRC driver for Actisys Act200L");
+MODULE_AUTHOR("Karl Bongers");
+#else
+MODULE_DESCRIPTION("Infrared receiver driver for SIR type serial ports");
+MODULE_AUTHOR("Milan Pikula");
 #endif
+MODULE_LICENSE("GPL");
+
+#ifdef LIRC_ON_SA1100
+module_param(irq, int, 0444);
+MODULE_PARM_DESC(irq, "Interrupt (16)");
+#else
+module_param(io, int, 0444);
+MODULE_PARM_DESC(io, "I/O address base (0x3f8 or 0x2f8)");
+
+module_param(irq, int, 0444);
+MODULE_PARM_DESC(irq, "Interrupt (4 or 3)");
+
+module_param(threshold, int, 0444);
+MODULE_PARM_DESC(threshold, "space detection threshold (3)");
+#endif
+
+module_param(debug, bool, 0644);
+MODULE_PARM_DESC(debug, "Enable debugging messages");
+
+EXPORT_NO_SYMBOLS;
+
+#endif /* MODULE */
