@@ -1,4 +1,4 @@
-/*      $Id: lirc_client.c,v 5.11 2001/04/24 19:05:58 lirc Exp $      */
+/*      $Id: lirc_client.c,v 5.16 2004/03/07 19:01:08 lirc Exp $      */
 
 /****************************************************************************
  ** lirc_client.c ***********************************************************
@@ -9,6 +9,7 @@
  * Copyright (C) 1998 Trent Piepho <xyzzy@u.washington.edu>
  * Copyright (C) 1998 Christoph Bartelmus <lirc@bartelmus.de>
  *
+ * System wide LIRCRC support by Michal Svec <rebel@atrey.karlin.mff.cuni.cz>
  */ 
 
 #ifdef HAVE_CONFIG_H
@@ -29,20 +30,53 @@
 
 #include "lirc_client.h"
 
+
+/* internal defines */
+#define MAX_INCLUDES 10
+#define LIRC_READ 255
+
+/* internal data structures */
+struct filestack_t {
+	FILE *file;
+	char *name;
+	int line;
+	struct filestack_t *parent;
+};
+
 /* internal functions */
-char *lirc_startupmode(struct lirc_config_entry *first);
-void lirc_freeconfigentries(struct lirc_config_entry *first);
-void lirc_clearmode(struct lirc_config *config);
-char *lirc_execute(struct lirc_config *config,struct lirc_config_entry *scan);
-int lirc_iscode(struct lirc_config_entry *scan,char *remote,char *button,
-		 int rep);
+static void lirc_printf(char *format_str, ...);
+static void lirc_perror(const char *s);
+static int lirc_readline(char **line,FILE *f);
+static char *lirc_trim(char *s);
+static char lirc_parse_escape(char **s,const char *name,int line);
+static void lirc_parse_string(char *s,const char *name,int line);
+static void lirc_parse_include(char *s,const char *name,int line);
+static int lirc_mode(char *token,char *token2,char **mode,
+		     struct lirc_config_entry **new_config,
+		     struct lirc_config_entry **first_config,
+		     struct lirc_config_entry **last_config,
+		     int (check)(char *s),
+		     const char *name,int line);
+static unsigned int lirc_flags(char *string);
+static FILE *lirc_open(const char *file, const char *current_file,
+		       char **full_name);
+static struct filestack_t *stack_push(struct filestack_t *parent);
+static struct filestack_t *stack_pop(struct filestack_t *entry);
+static void stack_free(struct filestack_t *entry);
+static char *lirc_startupmode(struct lirc_config_entry *first);
+static void lirc_freeconfigentries(struct lirc_config_entry *first);
+static void lirc_clearmode(struct lirc_config *config);
+static char *lirc_execute(struct lirc_config *config,
+			  struct lirc_config_entry *scan);
+static int lirc_iscode(struct lirc_config_entry *scan, char *remote,
+		       char *button,int rep);
 
 static int lirc_lircd;
 static int lirc_verbose=0;
 static char *lirc_prog=NULL;
 static char *lirc_buffer=NULL;
 
-void lirc_printf(char *format_str, ...)
+static void lirc_printf(char *format_str, ...)
 {
 	va_list ap;  
 	
@@ -53,7 +87,7 @@ void lirc_printf(char *format_str, ...)
 	va_end(ap);
 }
 
-void lirc_perror(const char *s)
+static void lirc_perror(const char *s)
 {
 	if(!lirc_verbose) return;
 
@@ -113,9 +147,7 @@ int lirc_deinit(void)
 	return(close(lirc_lircd));
 }
 
-#define LIRC_READ 255
-
-int lirc_readline(char **line,FILE *f)
+static int lirc_readline(char **line,FILE *f)
 {
 	char *newline,*ret,*enlargeline;
 	int len;
@@ -162,7 +194,7 @@ int lirc_readline(char **line,FILE *f)
 	}
 }
 
-char *lirc_trim(char *s)
+static char *lirc_trim(char *s)
 {
 	int len;
 	
@@ -179,7 +211,7 @@ char *lirc_trim(char *s)
 
 /* parse standard C escape sequences + \@,\A-\Z is ^@,^A-^Z */
 
-char lirc_parse_escape(char **s,int line)
+static char lirc_parse_escape(char **s,const char *name,int line)
 {
 
 	char c;
@@ -242,8 +274,8 @@ char lirc_parse_escape(char **s,int line)
 		{
 			i&=(1<<CHAR_BIT)-1;
 			lirc_printf("%s: octal escape sequence "
-				    "out of range in line %d\n",lirc_prog,
-				    line);
+				    "out of range in %s:%d\n",lirc_prog,
+				    name,line);
 		}
 		return((char) i);
 	case 'x':
@@ -272,15 +304,15 @@ char lirc_parse_escape(char **s,int line)
 			if(!digits_found)
 			{
 				lirc_printf("%s: \\x used with no "
-					    "following hex digits in line %d\n",
-					    lirc_prog,line);
+					    "following hex digits in %s:%d\n",
+					    lirc_prog,name,line);
 			}
 			if(overflow || i>(1<<CHAR_BIT)-1)
 			{
 				i&=(1<<CHAR_BIT)-1;
 				lirc_printf("%s: hex escape sequence out "
-					    "of range in line %d\n",
-					    lirc_prog,line);
+					    "of range in %s:%d\n",
+					    lirc_prog,name,line);
 			}
 			return((char) i);
 		}
@@ -290,7 +322,7 @@ char lirc_parse_escape(char **s,int line)
 	}
 }
 
-void lirc_parse_string(char *s,int line)
+static void lirc_parse_string(char *s,const char *name,int line)
 {
 	char *t;
 
@@ -300,7 +332,7 @@ void lirc_parse_string(char *s,int line)
 		if(*s=='\\')
 		{
 			s++;
-			*t=lirc_parse_escape(&s,line);
+			*t=lirc_parse_escape(&s,name,line);
 			t++;
 		}
 		else
@@ -313,12 +345,39 @@ void lirc_parse_string(char *s,int line)
 	*t=0;
 }
 
+static void lirc_parse_include(char *s,const char *name,int line)
+{
+	char last;
+	size_t len;
+	
+	len=strlen(s);
+	if(len<2)
+	{
+		return;
+	}
+	last=s[len-1];
+	if(*s!='"' && *s!='<')
+	{
+		return;
+	}
+	if(*s=='"' && last!='"')
+	{
+		return;
+	}
+	else if(*s=='<' && last!='>')
+	{
+		return;
+	}
+	s[len-1]=0;
+	memmove(s, s+1, len-2+1); /* terminating 0 is copied */
+}
+
 int lirc_mode(char *token,char *token2,char **mode,
 	      struct lirc_config_entry **new_config,
 	      struct lirc_config_entry **first_config,
 	      struct lirc_config_entry **last_config,
 	      int (check)(char *s),
-	      int line)
+	      const char *name,int line)
 {
 	struct lirc_config_entry *new_entry;
 	
@@ -341,6 +400,7 @@ int lirc_mode(char *token,char *token2,char **mode,
 				{
 					new_entry->prog=NULL;
 					new_entry->code=NULL;
+					new_entry->rep_delay=0;
 					new_entry->rep=0;
 					new_entry->config=NULL;
 					new_entry->change_mode=NULL;
@@ -356,7 +416,7 @@ int lirc_mode(char *token,char *token2,char **mode,
 			else
 			{
 				lirc_printf("%s: bad file format, "
-					    "line %d\n",lirc_prog,line);
+					    "%s:%d\n",lirc_prog,name,line);
 				return(-1);
 			}
 		}
@@ -373,7 +433,7 @@ int lirc_mode(char *token,char *token2,char **mode,
 			else
 			{
 				lirc_printf("%s: bad file format, "
-					    "line %d\n",lirc_prog,line);
+					    "%s:%d\n",lirc_prog,name,line);
 				return(-1);
 			}
 		}
@@ -444,11 +504,16 @@ int lirc_mode(char *token,char *token2,char **mode,
 					}
 				}
 				
+				if (new_entry->rep_delay==0 &&
+				    new_entry->rep>0)
+				{
+					new_entry->rep_delay=new_entry->rep-1;
+				}
 			}
 			else
 			{
-				lirc_printf("%s: line %d: 'end' without "
-					    "'begin'\n",lirc_prog,line);
+				lirc_printf("%s: %s:%d: 'end' without "
+					    "'begin'\n",lirc_prog,name,line);
 				return(-1);
 			}
 		}
@@ -458,9 +523,9 @@ int lirc_mode(char *token,char *token2,char **mode,
 			{
 				if(new_entry!=NULL)
 				{
-					lirc_printf("%s: line %d: missing "
+					lirc_printf("%s: %s:%d: missing "
 						    "'end' token\n",lirc_prog,
-						    line);
+						    name,line);
 					return(-1);
 				}
 				if(strcasecmp(*mode,token2)==0)
@@ -478,8 +543,8 @@ int lirc_mode(char *token,char *token2,char **mode,
 			}
 			else
 			{
-				lirc_printf("%s: line %d: 'end %s' without "
-					    "'begin'\n",lirc_prog,line,
+				lirc_printf("%s: %s:%d: 'end %s' without "
+					    "'begin'\n",lirc_prog,name,line,
 					    token2);
 				return(-1);
 			}
@@ -487,13 +552,13 @@ int lirc_mode(char *token,char *token2,char **mode,
 	}
 	else
 	{
-		lirc_printf("%s: unknown token \"%s\" in line %d ignored\n",
-			    lirc_prog,token,line);
+		lirc_printf("%s: unknown token \"%s\" in %s:%d ignored\n",
+			    lirc_prog,token,name,line);
 	}
 	return(0);
 }
 
-unsigned int lirc_flags(char *string)
+static unsigned int lirc_flags(char *string)
 {
 	char *s;
 	unsigned int flags;
@@ -527,48 +592,198 @@ unsigned int lirc_flags(char *string)
 	return(flags);
 }
 
-int lirc_readconfig(char *file,
-		    struct lirc_config **config,
-		    int (check)(char *s))
+static FILE *lirc_open(const char *file, const char *current_file,
+		       char **full_name)
 {
-	char *home,*filename,*string,*eq,*token,*token2,*token3;
+	char *home, *filename;
 	FILE *fin;
-	struct lirc_config_entry *new_entry,*first,*last;
-	char *mode,*remote;
-	int line,ret;
-	
+
 	if(file==NULL)
 	{
 		home=getenv("HOME");
-		filename=(char *) malloc(strlen(home)+1+strlen(LIRCCFGFILE)+1);
+		if(home==NULL)
+		{
+			home="/";
+		}
+		filename=(char *) malloc(strlen(home)+1+
+					 strlen(LIRCRC_USER_FILE)+1);
 		if(filename==NULL)
-			return(-1);
+		{
+			lirc_printf("%s: out of memory\n",lirc_prog);
+			return NULL;
+		}
 		strcpy(filename,home);
 		if(strlen(home)>0 && filename[strlen(filename)-1]!='/')
 		{
 			strcat(filename,"/");
 		}
-		strcat(filename,LIRCCFGFILE);
+		strcat(filename,LIRCRC_USER_FILE);
+	}
+	else if(strncmp(file, "~/", 2)==0)
+	{
+		home=getenv("HOME");
+		if(home==NULL)
+		{
+			home="/";
+		}
+		filename=(char *) malloc(strlen(home)+strlen(file)+1);
+		if(filename==NULL)
+		{
+			lirc_printf("%s: out of memory\n",lirc_prog);
+			return NULL;
+		}
+		strcpy(filename,home);
+		strcat(filename,file+1);
+	}
+	else if(file[0]=='/' || current_file==NULL)
+	{
+		/* absulute path or root */
+		filename=strdup(file);
+		if(filename==NULL)
+		{
+			lirc_printf("%s: out of memory\n",lirc_prog);
+			return NULL;
+		}
 	}
 	else
 	{
-		filename=file;
+		/* get path from parent filename */
+		int pathlen = strlen(current_file);
+		while (pathlen>0 && current_file[pathlen-1]!='/')
+			pathlen--;
+		filename=(char *) malloc(pathlen+strlen(file)+1);
+		if(filename==NULL)
+		{
+			lirc_printf("%s: out of memory\n",lirc_prog);
+			return NULL;
+		}
+		memcpy(filename,current_file,pathlen);
+		filename[pathlen]=0;
+		strcat(filename,file);
 	}
 
 	fin=fopen(filename,"r");
-	if(file==NULL) free(filename);
-	if(fin==NULL)
+	if(fin==NULL && (file!=NULL || errno!=ENOENT))
 	{
-		lirc_printf("%s: could not open config file\n",lirc_prog);
+		lirc_printf("%s: could not open config file %s\n",
+			    lirc_prog,filename);
 		lirc_perror(lirc_prog);
-		return(-1);
 	}
-	line=1;
+	else if(fin==NULL)
+	{
+		fin=fopen(LIRCRC_ROOT_FILE,"r");
+		if(fin==NULL && errno!=ENOENT)
+		{
+			lirc_printf("%s: could not open config file %s\n",
+				    lirc_prog,LIRCRC_ROOT_FILE);
+			lirc_perror(lirc_prog);
+		}
+		else if(fin==NULL)
+		{
+			lirc_printf("%s: could not open config files "
+				    "%s and %s\n",
+				    lirc_prog,filename,LIRCRC_ROOT_FILE);
+			lirc_perror(lirc_prog);
+		}
+		else
+		{
+			free(filename);
+			filename = strdup(LIRCRC_ROOT_FILE);
+			if(filename==NULL)
+			{
+				fclose(fin);
+				lirc_printf("%s: out of memory\n",lirc_prog);
+				return NULL;
+			}
+		}
+	}
+	if(full_name && fin!=NULL)
+	{
+		*full_name = filename;
+	}
+	else
+	{
+		free(filename);
+	}
+	return fin;
+}
+
+static struct filestack_t *stack_push(struct filestack_t *parent)
+{
+	struct filestack_t *entry;
+	entry = malloc(sizeof(struct filestack_t));
+	if (entry == NULL)
+	{
+		lirc_printf("%s: out of memory\n",lirc_prog);
+		return NULL;
+	}
+	entry->file = NULL;
+	entry->name = NULL;
+	entry->line = 0;
+	entry->parent = parent;
+	return entry;
+}
+
+static struct filestack_t *stack_pop(struct filestack_t *entry)
+{
+	struct filestack_t *parent = NULL;
+	if (entry)
+	{
+		parent = entry->parent;
+		if (entry->name)
+			free(entry->name);
+		free(entry);
+	}
+	return parent;
+}
+
+static void stack_free(struct filestack_t *entry)
+{
+	while (entry)
+	{
+		entry = stack_pop(entry);
+	}
+}
+
+int lirc_readconfig(char *file,
+		    struct lirc_config **config,
+		    int (check)(char *s))
+{
+	char *string,*eq,*token,*token2,*token3;
+	struct filestack_t *filestack, *stack_tmp;
+	int open_files;
+	struct lirc_config_entry *new_entry,*first,*last;
+	char *mode,*remote;
+	int ret=0;
+	
+	filestack = stack_push(NULL);
+	if (filestack == NULL)
+	{
+		return -1;
+	}
+	filestack->file = lirc_open(file, NULL, &(filestack->name));
+	if (filestack->file == NULL)
+	{
+		stack_free(filestack);
+		return -1;
+	}
+	filestack->line = 0;
+	open_files = 1;
+
 	first=new_entry=last=NULL;
 	mode=NULL;
 	remote=LIRC_ALL;
-	while((ret=lirc_readline(&string,fin))!=-1 && string!=NULL)
+	while (filestack)
 	{
+		if((ret=lirc_readline(&string,filestack->file))==-1 ||
+		   string==NULL)
+		{
+			fclose(filestack->file);
+			filestack = stack_pop(filestack);
+			open_files--;
+			continue;
+		}
+		filestack->line++;
 		eq=strchr(string,'=');
 		if(eq==NULL)
 		{
@@ -581,22 +796,62 @@ int lirc_readconfig(char *file,
 			{
 				/* ignore comment */
 			}
+			else if(strcasecmp(token, "include") == 0)
+			{
+				if (open_files >= MAX_INCLUDES)
+				{
+					lirc_printf("%s: too many files "
+						    "included at %s:%d\n",
+						    lirc_prog,
+						    filestack->name,
+						    filestack->line);
+					ret=-1;
+				}
+				else
+				{
+					token2 = strtok(NULL, "");
+					token2 = lirc_trim(token2);
+					lirc_parse_include
+						(token2, filestack->name,
+						 filestack->line);
+					stack_tmp = stack_push(filestack);
+					if (stack_tmp == NULL)
+					{
+						ret=-1;
+					}
+					else
+					{
+						stack_tmp->file = lirc_open(token2, filestack->name, &(stack_tmp->name));
+						stack_tmp->line = 0;
+						if (stack_tmp->file)
+						{
+							open_files++;
+							filestack = stack_tmp;
+						}
+						else
+						{
+							stack_pop(stack_tmp);
+							ret=-1;
+						}
+					}
+				}
+			}
 			else
 			{
 				token2=strtok(NULL," \t");
 				if(token2!=NULL && 
 				   (token3=strtok(NULL," \t"))!=NULL)
 				{
-					lirc_printf("%s: unexpected "
-						    "token in line %d\n",
-						    lirc_prog,line);
+					lirc_printf("%s: unexpected token in line %s:%d\n",
+						    lirc_prog,filestack->name,filestack->line);
 				}
 				else
 				{
 					ret=lirc_mode(token,token2,&mode,
 						      &new_entry,&first,&last,
 						      check,
-						      line);
+						      filestack->name,
+						      filestack->line);
 					if(ret==0)
 					{
 						if(remote!=LIRC_ALL)
@@ -631,8 +886,8 @@ int lirc_readconfig(char *file,
 			}
 			else if(new_entry==NULL)
 			{
-				lirc_printf("%s: bad file format, "
-					    "line %d\n",lirc_prog,line);
+				lirc_printf("%s: bad file format, %s:%d\n",
+					lirc_prog,filestack->name,filestack->line);
 				ret=-1;
 			}
 			else
@@ -713,6 +968,24 @@ int lirc_readconfig(char *file,
 						}
 					}
 				}
+				else if(strcasecmp(token,"delay")==0)
+				{
+					char *end;
+
+					errno=ERANGE+1;
+					new_entry->rep_delay=strtoul(token2,&end,0);
+					if((new_entry->rep_delay==ULONG_MAX 
+					    && errno==ERANGE)
+					   || end[0]!=0
+					   || strlen(token2)==0)
+					{
+						lirc_printf("%s: \"%s\" not"
+							    " a  valid number for "
+							    "delay\n",lirc_prog,
+							    token2);
+					}
+					free(token2);
+				}
 				else if(strcasecmp(token,"repeat")==0)
 				{
 					char *end;
@@ -747,7 +1020,7 @@ int lirc_readconfig(char *file,
 					}
 					else
 					{
-						lirc_parse_string(token2,line);
+						lirc_parse_string(token2,filestack->name,filestack->line);
 						new_list->string=token2;
 						new_list->next=NULL;
 						if(new_entry->config==NULL)
@@ -775,14 +1048,12 @@ int lirc_readconfig(char *file,
 				else
 				{
 					free(token2);
-					lirc_printf("%s: unknown token "
-						    "\"%s\" in line %d ignored\n",
-						    lirc_prog,token,line);
+					lirc_printf("%s: unknown token \"%s\" in %s:%d ignored\n",
+						    lirc_prog,token,filestack->name,filestack->line);
 				}
 			}
 		}
 		free(string);
-		line++;
 		if(ret==-1) break;
 	}
 	if(remote!=LIRC_ALL)
@@ -792,7 +1063,7 @@ int lirc_readconfig(char *file,
 		if(ret==0)
 		{
 			ret=lirc_mode("end",NULL,&mode,&new_entry,
-				      &first,&last,check,line);
+				      &first,&last,check,"",0);
 			lirc_printf("%s: warning: end token missing at end "
 				    "of file\n",lirc_prog);
 		}
@@ -811,13 +1082,13 @@ int lirc_readconfig(char *file,
 		}
 		free(mode);
 	}
-	fclose(fin);
 	if(ret==0)
 	{
 		*config=(struct lirc_config *)
 			malloc(sizeof(struct lirc_config));
 		if(*config==NULL)
 		{
+			lirc_printf("%s: out of memory\n",lirc_prog);
 			lirc_freeconfigentries(first);
 			return(-1);
 		}
@@ -830,10 +1101,14 @@ int lirc_readconfig(char *file,
 		*config=NULL;
 		lirc_freeconfigentries(first);
 	}
+	if(filestack)
+	{
+		stack_free(filestack);
+	}
 	return(ret);
 }
 
-char *lirc_startupmode(struct lirc_config_entry *first)
+static char *lirc_startupmode(struct lirc_config_entry *first)
 {
 	struct lirc_config_entry *scan;
 	char *startupmode;
@@ -895,7 +1170,7 @@ void lirc_freeconfig(struct lirc_config *config)
 	}
 }
 
-void lirc_freeconfigentries(struct lirc_config_entry *first)
+static void lirc_freeconfigentries(struct lirc_config_entry *first)
 {
 	struct lirc_config_entry *c,*config_temp;
 	struct lirc_list *list,*list_temp;
@@ -934,7 +1209,7 @@ void lirc_freeconfigentries(struct lirc_config_entry *first)
 	}
 }
 
-void lirc_clearmode(struct lirc_config *config)
+static void lirc_clearmode(struct lirc_config *config)
 {
 	struct lirc_config_entry *scan;
 
@@ -957,27 +1232,19 @@ void lirc_clearmode(struct lirc_config *config)
 	config->current_mode=NULL;
 }
 
-char *lirc_execute(struct lirc_config *config,struct lirc_config_entry *scan)
+static char *lirc_execute(struct lirc_config *config,
+			  struct lirc_config_entry *scan)
 {
 	char *s;
 	int do_once=1;
 	
-	if(scan->flags&quit)
-	{
-		config->next=NULL;
-	}
-	else
-	{
-		config->next=scan->next;
-	}
 	if(scan->flags&mode)
 	{
 		lirc_clearmode(config);
 	}
 	if(scan->change_mode!=NULL)
 	{
-		config->current_mode
-		=scan->change_mode;
+		config->current_mode=scan->change_mode;
 		if(scan->flags&once)
 		{
 			if(scan->flags&ecno)
@@ -996,29 +1263,33 @@ char *lirc_execute(struct lirc_config *config,struct lirc_config_entry *scan)
 	   do_once==1)
 	{
 		s=scan->next_config->string;
-		scan->next_config
-		=scan->next_config->next;
+		scan->next_config=scan->next_config->next;
 		if(scan->next_config==NULL)
-			scan->next_config
-			=scan->config;
+			scan->next_config=scan->config;
 		return(s);
 	}
 	return(NULL);
 }
 
-int lirc_iscode(struct lirc_config_entry *scan,char *remote,char *button,int rep)
+static int lirc_iscode(struct lirc_config_entry *scan, char *remote,
+		       char *button,int rep)
 {
 	struct lirc_code *codes;
 	
 	if(scan->code==NULL)
-		return(1);
-
+	{
+		return rep==0 ||
+			(scan->rep>0 && rep>scan->rep_delay &&
+			 ((rep-scan->rep_delay-1)%scan->rep)==0);
+	}
+	
 	if(scan->next_code->remote==LIRC_ALL || 
 	   strcasecmp(scan->next_code->remote,remote)==0)
 	{
 		if(scan->next_code->button==LIRC_ALL || 
 		   strcasecmp(scan->next_code->button,button)==0)
 		{
+			int iscode=0;
 			if(scan->code->next==NULL || rep==0)
 			{
 				scan->next_code=scan->next_code->next;
@@ -1026,20 +1297,11 @@ int lirc_iscode(struct lirc_config_entry *scan,char *remote,char *button,int rep
 			if(scan->next_code==NULL)
 			{
 				scan->next_code=scan->code;
-                                if(scan->code->next!=NULL || 
-                                   (scan->rep==0 ? rep==0:(rep%scan->rep)==0))
-                                {
-                                        return(1);
-                                }
-                                else
-                                {
-                                        return(0);
-                                }
+				iscode=scan->code->next!=NULL || rep==0 ||
+				   (scan->rep>0 && rep>scan->rep_delay &&
+				    ((rep-scan->rep_delay-1)%scan->rep)==0);
                         }
-			else
-			{
-				return(0);
-			}
+			return iscode;
 		}
 	}
         if(rep!=0) return(0);
@@ -1116,10 +1378,12 @@ int lirc_code2char(struct lirc_config *config,char *code,char **string)
 	int rep;
 	char *backup;
 	char *remote,*button;
+	char *s=NULL;
 	struct lirc_config_entry *scan;
+	int quit_happened;
 
 	*string=NULL;
-	if(sscanf(code,"%*llx %x %*s %*s\n",&rep)==1)
+	if(sscanf(code,"%*x %x %*s %*s\n",&rep)==1)
 	{
 		backup=strdup(code);
 		if(backup==NULL) return(-1);
@@ -1136,34 +1400,39 @@ int lirc_code2char(struct lirc_config *config,char *code,char **string)
 		}
 		
 		scan=config->next;
+		quit_happened=0;
 		while(scan!=NULL)
 		{
 			if(lirc_iscode(scan,remote,button,rep) &&
 			   (scan->mode==NULL ||
 			    (scan->mode!=NULL && 
 			     config->current_mode!=NULL &&
-			     strcasecmp(scan->mode,config->current_mode)==0))
+			     strcasecmp(scan->mode,config->current_mode)==0)) &&
+			   quit_happened==0
 			   )
 			{
-				char *s;
 				s=lirc_execute(config,scan);
-				if(s!=NULL)
+				if(scan->flags&quit)
 				{
-					free(backup);
-					*string=s;
-					return(0);
+					quit_happened=1;
+					config->next=NULL;
+					scan=scan->next;
+					continue;
+				}
+				else if(s!=NULL)
+				{
+					config->next=scan->next;
+					break;
 				}
 			}
-			if(config->next!=NULL)
-			{
-				scan=scan->next;
-			}
-			else
-			{
-				scan=NULL;
-			}
+			scan=scan->next;
 		}
 		free(backup);
+		if(s!=NULL)
+		{
+			*string=s;
+			return(0);
+		}
 	}
 	config->next=config->first;
 	return(0);
@@ -1202,6 +1471,7 @@ int lirc_nextcode(char **code)
 		lirc_buffer=(char *) malloc(packet_size+1);
 		if(lirc_buffer==NULL)
 		{
+			lirc_printf("%s: out of memory\n",lirc_prog);
 			return(-1);
 		}
 		lirc_buffer[0]=0;
